@@ -1,39 +1,36 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
 #if defined(_WIN32)
@@ -57,6 +54,8 @@
 // "128 IWAD search directories should be enough for anybody".
 #define MAX_IWAD_DIRS   128
 
+char        screenshotfolder[MAX_PATH];
+
 static char *iwad_dirs[MAX_IWAD_DIRS];
 static int  num_iwad_dirs;
 
@@ -90,9 +89,9 @@ typedef struct
 // [AlexMax] From the perspective of a 64-bit executable, 32-bit registry
 // keys are located in a different spot.
 #if defined(_WIN64)
-#define SOFTWARE_KEY    "SOFTWARE\\WOW6432Node"
+#define SOFTWARE_KEY    "SOFTWARE\\WOW6432Node\\"
 #else
-#define SOFTWARE_KEY    "SOFTWARE"
+#define SOFTWARE_KEY    "SOFTWARE\\"
 #endif
 
 static registryvalue_t uninstall_values[] =
@@ -100,30 +99,30 @@ static registryvalue_t uninstall_values[] =
     // Ultimate DOOM, CD version (Depths of DOOM trilogy)
     {
         HKEY_LOCAL_MACHINE,
-        SOFTWARE_KEY "\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Ultimate Doom for Windows 95",
+        SOFTWARE_KEY "Microsoft\\Windows\\CurrentVersion\\Uninstall\\Ultimate Doom for Windows 95",
         "UninstallString"
     },
 
     // DOOM II, CD version (Depths of DOOM trilogy)
     {
         HKEY_LOCAL_MACHINE,
-        SOFTWARE_KEY "\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Doom II for Windows 95",
+        SOFTWARE_KEY "Microsoft\\Windows\\CurrentVersion\\Uninstall\\Doom II for Windows 95",
         "UninstallString"
     },
 
     // Final DOOM
     {
         HKEY_LOCAL_MACHINE,
-        SOFTWARE_KEY "\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Final Doom for Windows 95",
+        SOFTWARE_KEY "Microsoft\\Windows\\CurrentVersion\\Uninstall\\Final Doom for Windows 95",
         "UninstallString"
     },
 
     // Shareware version
     {
         HKEY_LOCAL_MACHINE,
-        SOFTWARE_KEY "\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Doom Shareware for Windows 95",
+        SOFTWARE_KEY "Microsoft\\Windows\\CurrentVersion\\Uninstall\\Doom Shareware for Windows 95",
         "UninstallString"
-    },
+    }
 };
 
 // Values installed by the GOG.com and Collector's Edition versions
@@ -132,35 +131,56 @@ static registryvalue_t root_path_keys[] =
     // DOOM Collector's Edition
     {
         HKEY_LOCAL_MACHINE,
-        SOFTWARE_KEY "\\Activision\\DOOM Collector's Edition\\v1.0",
+        SOFTWARE_KEY "Activision\\DOOM Collector's Edition\\v1.0",
+        "INSTALLPATH"
+    },
+
+    // DOOM + DOOM II
+    {
+        HKEY_LOCAL_MACHINE,
+        SOFTWARE_KEY "GOG.com\\Games\\1413291984",
+        "INSTALLPATH"
+    },
+
+    // DOOM
+    {
+        HKEY_LOCAL_MACHINE,
+        SOFTWARE_KEY "GOG.com\\Games\\2015545325",
         "INSTALLPATH"
     },
 
     // DOOM II
     {
         HKEY_LOCAL_MACHINE,
-        SOFTWARE_KEY "\\GOG.com\\Games\\1435848814",
+        SOFTWARE_KEY "GOG.com\\Games\\1435848814",
+        "PATH"
+    },
+
+    // DOOM II
+    {
+        HKEY_LOCAL_MACHINE,
+        SOFTWARE_KEY "GOG.com\\Games\\1426071866",
         "PATH"
     },
 
     // DOOM 3: BFG Edition
     {
         HKEY_LOCAL_MACHINE,
-        SOFTWARE_KEY "\\GOG.com\\Games\\1135892318",
+        SOFTWARE_KEY "GOG.com\\Games\\1135892318",
         "PATH"
     },
 
     // Final DOOM
     {
         HKEY_LOCAL_MACHINE,
-        SOFTWARE_KEY "\\GOG.com\\Games\\1435848742",
+        SOFTWARE_KEY "GOG.com\\Games\\1435848742",
         "PATH"
     },
 
     // Ultimate DOOM
     {
         HKEY_LOCAL_MACHINE,
-        SOFTWARE_KEY "\\GOG.com\\Games\\1435827232",
+        SOFTWARE_KEY "GOG.com\\Games\\1435827232",
         "PATH"
     }
 };
@@ -177,11 +197,30 @@ static const char *root_path_subdirs[] =
     "base\\wads"
 };
 
+// Location where the Bethesda.net Launcher is installed
+static registryvalue_t bethesda_install_location =
+{
+    HKEY_LOCAL_MACHINE,
+    SOFTWARE_KEY "Bethesda Softworks\\Bethesda.net",
+    "installLocation"
+};
+
+// Subdirs of the Bethesda.net Launcher install directory where IWADs are found
+static const char *bethesda_install_subdirs[] =
+{
+    "games\\DOOM_II_Classic_2019\\base",
+    "games\\DOOM_II_Classic_2019\\rerelease\\DOOM II_Data\\StreamingAssets",
+    "games\\DOOM_Classic_2019\\base",
+    "games\\DOOM_Classic_2019\\rerelease\\DOOM_Data\\StreamingAssets",
+    "games\\DOOM 3 BFG Edition\\base\\wads",
+    "games\\DOOM Eternal\\base\\classicwads"
+};
+
 // Locations where Steam is installed
 static registryvalue_t steam_install_locations[] =
 {
-    { HKEY_CURRENT_USER,  "SOFTWARE\\Valve\\Steam",      "SteamPath"   },
-    { HKEY_LOCAL_MACHINE, SOFTWARE_KEY "\\Valve\\Steam", "InstallPath" }
+    { HKEY_CURRENT_USER,  "SOFTWARE\\Valve\\Steam",    "SteamPath"   },
+    { HKEY_LOCAL_MACHINE, SOFTWARE_KEY "Valve\\Steam", "InstallPath" }
 };
 
 // Subdirs of the Steam install directory where IWADs are found
@@ -189,13 +228,19 @@ static const char *steam_install_subdirs[] =
 {
     "steamapps\\common\\Doom 2\\rerelease\\DOOM II_Data\\StreamingAssets",
     "steamapps\\common\\Doom 2\\base",
-    "steamapps\\common\\DOOM 3 BFG Edition\\base\\wads",
-    "steamapps\\common\\Final Doom\\base",
+    "steamapps\\common\\Doom 2\\finaldoombase",
+    "steamapps\\common\\Doom 2\\masterbase\\doom2",
+    "steamapps\\common\\Ultimate Doom\\rerelease",
     "steamapps\\common\\Ultimate Doom\\rerelease\\DOOM_Data\\StreamingAssets",
     "steamapps\\common\\Ultimate Doom\\base"
+    "steamapps\\common\\Ultimate Doom\\base\\doom2"
+    "steamapps\\common\\DOOM 3 BFG Edition\\base\\wads",
+    "steamapps\\common\\Final Doom\\base",
+    "steamapps\\common\\Master Levels of Doom\\doom2",
+    "steamapps\\common\\DOOMEternal\\base\\classicwads"
 };
 
-static char *GetRegistryString(registryvalue_t *reg_val)
+static char *GetRegistryString(const registryvalue_t *reg_val)
 {
     HKEY    key;
     DWORD   len = 0;
@@ -231,7 +276,7 @@ static char *GetRegistryString(registryvalue_t *reg_val)
 // Check for the uninstall strings from the CD versions
 static void CheckUninstallStrings(void)
 {
-    int len = (int)strlen(UNINSTALLER_STRING);
+    const int   len = (int)strlen(UNINSTALLER_STRING);
 
     for (size_t i = 0; i < arrlen(uninstall_values); i++)
     {
@@ -245,6 +290,8 @@ static void CheckUninstallStrings(void)
         {
             char    *path = unstr + len;
 
+            path[0] = toupper(path[0]);
+            M_NormalizeSlashes(path);
             AddIWADDir(path);
         }
         else
@@ -257,40 +304,55 @@ static void CheckInstallRootPaths(void)
 {
     for (size_t i = 0; i < arrlen(root_path_keys); i++)
     {
-        char    *install_path = GetRegistryString(&root_path_keys[i]);
+        char    *path = GetRegistryString(&root_path_keys[i]);
 
-        if (!install_path)
+        if (!path)
             continue;
 
+        path[0] = toupper(path[0]);
+        M_NormalizeSlashes(path);
+
         for (size_t j = 0; j < arrlen(root_path_subdirs); j++)
-        {
-            char    *path = M_StringJoin(install_path, DIR_SEPARATOR_S, root_path_subdirs[j], NULL);
+            AddIWADDir(M_StringJoin(path, DIR_SEPARATOR_S, root_path_subdirs[j], NULL));
 
-            AddIWADDir(path);
-        }
-
-        free(install_path);
+        free(path);
     }
 }
 
+// Check for DOOM downloaded via the Bethesda.net Launcher
+static void CheckBethesdaEdition(void)
+{
+    char    *path = GetRegistryString(&bethesda_install_location);
+
+    if (!path)
+        return;
+
+    path[0] = toupper(path[0]);
+    M_NormalizeSlashes(path);
+
+    for (size_t j = 0; j < arrlen(bethesda_install_subdirs); j++)
+        AddIWADDir(M_StringJoin(path, DIR_SEPARATOR_S, bethesda_install_subdirs[j], NULL));
+
+    free(path);
+}
+
 // Check for DOOM downloaded via Steam
 static void CheckSteamEdition(void)
 {
     for (size_t i = 0; i < arrlen(steam_install_locations); i++)
     {
-        char    *install_path = GetRegistryString(&steam_install_locations[i]);
+        char    *path = GetRegistryString(&steam_install_locations[i]);
 
-        if (!install_path)
+        if (!path)
             continue;
 
+        path[0] = toupper(path[0]);
+        M_NormalizeSlashes(path);
+
         for (size_t j = 0; j < arrlen(steam_install_subdirs); j++)
-        {
-            char    *path = M_StringJoin(install_path, DIR_SEPARATOR_S, steam_install_subdirs[j], NULL);
+            AddIWADDir(M_StringJoin(path, DIR_SEPARATOR_S, steam_install_subdirs[j], NULL));
 
-            AddIWADDir(path);
-        }
-
-        free(install_path);
+        free(path);
     }
 }
 
@@ -316,7 +378,7 @@ static void AddIWADPath(const char *path, const char *suffix)
     char    *p;
     char    *dup_path = M_StringDuplicate(path);
 
-    // Split into individual dirs within the list.
+    // Split into individual directories within the list.
     char    *left = dup_path;
 
     while (true)
@@ -421,7 +483,7 @@ static void AddSteamDirs(void)
 typedef struct
 {
     char            *name;
-    GameMission_t   mission;
+    gamemission_t   mission;
 } iwads_t;
 
 static const iwads_t iwads[] =
@@ -439,7 +501,7 @@ static const iwads_t iwads[] =
 #if !defined(_WIN32) && !defined(__APPLE__)
 // Returns true if the specified path is a path to a file
 // of the specified name.
-static dboolean DirIsFile(char *path, char *filename)
+static bool DirIsFile(char *path, char *filename)
 {
     return (strchr(path, DIR_SEPARATOR) && !strcasecmp(leafname(path), filename));
 }
@@ -475,7 +537,7 @@ static char *CheckDirectoryHasIWAD(char *dir, char *iwadname)
 // Returns the location of the IWAD if found, otherwise NULL.
 static char *SearchDirectoryForIWAD(char *dir)
 {
-    for (size_t i = 0; i < arrlen(iwads); ++i)
+    for (size_t i = 0; i < arrlen(iwads); i++)
     {
         char    *filename = CheckDirectoryHasIWAD(dir, iwads[i].name);
 
@@ -492,7 +554,7 @@ static char *SearchDirectoryForIWAD(char *dir)
 
 // When given an IWAD with the '-iwad' parameter,
 // attempt to identify it by its name.
-void D_IdentifyIWADByName(char *name)
+void D_IdentifyIWADByName(const char *name)
 {
     // Trim down the name to just the filename, ignoring the path.
     char    *p = strrchr(name, '\\');
@@ -514,7 +576,6 @@ void D_IdentifyIWADByName(char *name)
         {
             gamemission = iwads[i].mission;
             free(iwad);
-
             break;
         }
 
@@ -523,6 +584,8 @@ void D_IdentifyIWADByName(char *name)
 
     if (M_StringCompare(name, "HACX.WAD"))
         hacx = true;
+    else if (M_StringCompare(name, "harmony.wad") || M_StringCompare(name, "harm1.wad"))
+        harmony = true;
 }
 
 //
@@ -539,7 +602,7 @@ static void AddDoomWADPath(void)
     // Add the initial directory
     AddIWADDir(doomwadpath);
 
-    // Split into individual dirs within the list.
+    // Split into individual directories within the list.
     p = doomwadpath;
 
     while (true)
@@ -562,8 +625,8 @@ static void AddDoomWADPath(void)
 //
 static void BuildIWADDirList(void)
 {
-    char            *doomwaddir;
-    static dboolean iwad_dirs_built;
+    char        *doomwaddir;
+    static bool iwad_dirs_built;
 
     if (iwad_dirs_built)
         return;
@@ -572,11 +635,12 @@ static void BuildIWADDirList(void)
     if ((doomwaddir = getenv("DOOMWADDIR")))
         AddIWADDir(doomwaddir);
 
-    // Add dirs from DOOMWADPATH
+    // Add directories from DOOMWADPATH
     AddDoomWADPath();
 
 #if defined(_WIN32)
     // Search the registry and find where IWADs have been installed.
+    CheckBethesdaEdition();
     CheckSteamEdition();
     CheckInstallRootPaths();
     CheckUninstallStrings();
@@ -633,7 +697,7 @@ char *D_FindWADByName(char *filename)
     return NULL;
 }
 
-void D_InitIWADFolder(void)
+void D_InitWADfolder(void)
 {
 #if defined(_WIN32)
     char    path[MAX_PATH];
@@ -644,20 +708,18 @@ void D_InitIWADFolder(void)
     for (int i = 0; i < num_iwad_dirs; i++)
         if (M_FolderExists(iwad_dirs[i]))
         {
-            iwadfolder = M_StringDuplicate(iwad_dirs[i]);
-            M_StringReplaceAll(iwadfolder, "/", "\\");
-
+            wadfolder = M_StringDuplicate(iwad_dirs[i]);
             break;
         }
 
 #if defined(_WIN32)
-    M_snprintf(path, sizeof(path), "%s" DIR_SEPARATOR_S "DOOM.WAD", iwadfolder);
+    M_snprintf(path, sizeof(path), "%s" DIR_SEPARATOR_S "DOOM.WAD", wadfolder);
 
     if (M_FileExists(path))
         wad = "DOOM.WAD";
     else
     {
-        M_snprintf(path, sizeof(path), "%s" DIR_SEPARATOR_S "DOOM2.WAD", iwadfolder);
+        M_snprintf(path, sizeof(path), "%s" DIR_SEPARATOR_S "DOOM2.WAD", wadfolder);
 
         if (M_FileExists(path))
             wad = "DOOM2.WAD";
@@ -687,13 +749,16 @@ char *D_TryFindWADByName(char *filename)
 char *D_FindIWAD(void)
 {
     char    *result = NULL;
-    int     iwadparm = M_CheckParmWithArgs("-iwad", 1, 1);
+    int     iwadparm = M_CheckParmWithArgs("-iwad", 1);
 
     if (iwadparm)
     {
-        // Search through IWAD dirs for an IWAD with the given name.
+        // Search through IWAD directories for an IWAD with the given name.
         char    *iwadfile = myargv[iwadparm + 1];
 
+        if (!M_StringEndsWith(iwadfile, ".wad"))
+            iwadfile = M_StringJoin(iwadfile, ".wad", NULL);
+
         if (!(result = D_FindWADByName(iwadfile)))
             I_Error("The IWAD file \"%s\" wasn't found!", iwadfile);
 
@@ -722,14 +787,16 @@ static char *SaveGameIWADName(void)
     //
     // The directory depends on the IWAD, so that savegames for
     // different IWADs are kept separate.
-    //
-    // Note that we match on gamemission rather than on IWAD name.
-    // This ensures that DOOM1.WAD and DOOM.WAD saves are stored
-    // in the same place.
+    if (gamemode == shareware)
+        return "DOOM1";
     if (FREEDOOM)
         return (gamemode == commercial ? "freedoom2" : "freedoom");
     else if (hacx)
         return "hacx";
+    else if (chex)
+        return "chex";
+    else if (REKKRSA)
+        return "rekkrsa";
 
     for (size_t i = 0; iwads[i].name[0]; i++)
         if (gamemission == iwads[i].mission)
@@ -743,9 +810,9 @@ static char *SaveGameIWADName(void)
 //
 // Chooses the directory used to store saved games.
 //
-void D_SetSaveGameFolder(dboolean output)
+void D_SetSaveGameFolder(bool output)
 {
-    int p = M_CheckParmsWithArgs("-save", "-savedir", "", 1, 1);
+    const int   p = M_CheckParmsWithArgs("-save", "-savedir", "", 1);
 
     if (p)
     {
@@ -755,35 +822,36 @@ void D_SetSaveGameFolder(dboolean output)
     else
     {
         char    *appdatafolder = M_GetAppDataFolder();
-        char    *savegamefolder_free;
+        char    *tempsavegamefolder;
 
         M_MakeDirectory(appdatafolder);
-        savegamefolder = M_StringJoin(appdatafolder, DIR_SEPARATOR_S, "savegames", DIR_SEPARATOR_S, NULL);
-        M_MakeDirectory(savegamefolder);
-        savegamefolder_free = savegamefolder;
+
+        tempsavegamefolder = M_StringJoin(appdatafolder,
+            DIR_SEPARATOR_S DOOMRETRO_SAVEGAMESFOLDER DIR_SEPARATOR_S, NULL);
+        M_MakeDirectory(tempsavegamefolder);
 
         if (*pwadfile)
         {
-            char    *temp = removeext(pwadfile);
+            char    *temp = removeext(GetCorrectCase(pwadfile));
 
-            savegamefolder = M_StringJoin(savegamefolder, temp, DIR_SEPARATOR_S, NULL);
+            savegamefolder = M_StringJoin(tempsavegamefolder, temp, DIR_SEPARATOR_S, NULL);
             free(temp);
         }
         else
-            savegamefolder = M_StringJoin(savegamefolder, SaveGameIWADName(), DIR_SEPARATOR_S, NULL);
+            savegamefolder = M_StringJoin(tempsavegamefolder, SaveGameIWADName(), DIR_SEPARATOR_S, NULL);
 
         free(appdatafolder);
-        free(savegamefolder_free);
+        free(tempsavegamefolder);
     }
 
     M_MakeDirectory(savegamefolder);
 
     if (output)
     {
-        int numsavegames = M_CountSaveGames();
+        const int   numsavegames = M_CountSaveGames();
 
         if (!numsavegames)
-            C_Output("All savegames will be saved in " BOLD("%s") ".", savegamefolder);
+            C_Output("Savegames are placed in " BOLD("%s") ".", savegamefolder);
         else if (numsavegames == 1)
             C_Output("There is 1 savegame in " BOLD("%s") ".", savegamefolder);
         else
@@ -791,6 +859,58 @@ void D_SetSaveGameFolder(dboolean output)
     }
 }
 
+void D_SetAutoloadFolder(void)
+{
+    const int   p = M_CheckParmsWithArgs("-autoload", "-autoloaddir", "", 1);
+
+    if (p)
+        M_StringCopy(autoloadfolder, myargv[p + 1], sizeof(autoloadfolder));
+    else
+    {
+        char    *appdatafolder = M_GetAppDataFolder();
+
+        M_MakeDirectory(appdatafolder);
+        autoloadfolder = M_StringJoin(appdatafolder,
+            DIR_SEPARATOR_S DOOMRETRO_AUTOLOADFOLDER DIR_SEPARATOR_S, NULL);
+        free(appdatafolder);
+    }
+
+    M_MakeDirectory(autoloadfolder);
+
+    autoloadiwadsubfolder = M_StringJoin(autoloadfolder, SaveGameIWADName(), DIR_SEPARATOR_S, NULL);
+    M_MakeDirectory(autoloadiwadsubfolder);
+
+    if (*pwadfile)
+    {
+        char    *temp = removeext(GetCorrectCase(pwadfile));
+
+        autoloadpwadsubfolder = M_StringJoin(autoloadfolder, temp, DIR_SEPARATOR_S, NULL);
+        M_MakeDirectory(autoloadpwadsubfolder);
+        free(temp);
+    }
+}
+
+void D_SetScreenshotsFolder(void)
+{
+    const int   p = M_CheckParmsWithArgs("-shot", "-shotdir", "", 1);
+
+    if (p)
+        M_StringCopy(screenshotfolder, myargv[p + 1], sizeof(screenshotfolder));
+    else
+    {
+        char    *appdatafolder = M_GetAppDataFolder();
+
+        M_snprintf(screenshotfolder, sizeof(screenshotfolder),
+            "%s" DIR_SEPARATOR_S DOOMRETRO_SCREENSHOTSFOLDER DIR_SEPARATOR_S, appdatafolder);
+
+        free(appdatafolder);
+    }
+
+    M_MakeDirectory(screenshotfolder);
+
+    C_Output("Screenshots taken are placed in " BOLD("%s") ".", screenshotfolder);
+}
+
 //
 // Find out what version of DOOM is playing.
 //
@@ -807,8 +927,7 @@ void D_IdentifyVersion(void)
             gamemission = doom2;
         else if (W_CheckNumForName("E1M1") >= 0)
             gamemission = doom;
-
-        if (gamemission == none)
+        else
             // Still no idea. I don't think this is going to work.
             I_Error("Unknown or invalid IWAD file.");
     }
@@ -849,58 +968,75 @@ void D_SetGameDescription(void)
         M_StringCopy(gamedescription, s_CAPTION_REKKRSL, sizeof(gamedescription));
     else if (REKKR)
         M_StringCopy(gamedescription, s_CAPTION_REKKR, sizeof(gamedescription));
+    else if (anomalyreport)
+        M_StringCopy(gamedescription, s_CAPTION_ANOMALYREPORT, sizeof(gamedescription));
+    else if (arrival)
+        M_StringCopy(gamedescription, s_CAPTION_ARRIVAL, sizeof(gamedescription));
+    else if (dbimpact)
+        M_StringCopy(gamedescription, s_CAPTION_DBIMPACT, sizeof(gamedescription));
+    else if (deathless)
+        M_StringCopy(gamedescription, s_CAPTION_DEATHLESS, sizeof(gamedescription));
+    else if (doomzero)
+        M_StringCopy(gamedescription, s_CAPTION_DOOMZERO, sizeof(gamedescription));
+    else if (earthless)
+        M_StringCopy(gamedescription, s_CAPTION_EARTHLESS, sizeof(gamedescription));
+    else if (ganymede)
+        M_StringCopy(gamedescription, s_CAPTION_GANYMEDE, sizeof(gamedescription));
+    else if (harmony || harmonyc)
+        M_StringCopy(gamedescription, s_CAPTION_HARMONY, sizeof(gamedescription));
+    else if (ID1)
+        M_StringCopy(gamedescription, s_CAPTION_ID1, sizeof(gamedescription));
+    else if (KDIKDIZD)
+        M_StringCopy(gamedescription, s_CAPTION_KDIKDIZD, sizeof(gamedescription));
+    else if (masterlevels)
+        M_StringCopy(gamedescription, s_CAPTION_MASTERLEVELS, sizeof(gamedescription));
+    else if (neis)
+        M_StringCopy(gamedescription, s_CAPTION_NEIS, sizeof(gamedescription));
+    else if (revolution)
+        M_StringCopy(gamedescription, s_CAPTION_REVOLUTION, sizeof(gamedescription));
+    else if (scientist)
+        M_StringCopy(gamedescription, s_CAPTION_SCIENTIST, sizeof(gamedescription));
+    else if (syringe)
+        M_StringCopy(gamedescription, s_CAPTION_SYRINGE, sizeof(gamedescription));
+    else if (TTNS)
+        M_StringCopy(gamedescription, s_CAPTION_TTNS, sizeof(gamedescription));
+    else if (TTP)
+        M_StringCopy(gamedescription, s_CAPTION_TTP, sizeof(gamedescription));
     else if (gamemission == doom)
     {
         // DOOM 1. But which version?
         if (modifiedgame && *pwadfile)
-            M_StringCopy(gamedescription, pwadfile, sizeof(gamedescription));
+        {
+            M_StringCopy(gamedescription, GetCorrectCase(pwadfile), sizeof(gamedescription));
+            return;
+        }
         else if (FREEDOOM)
             M_StringCopy(gamedescription, s_CAPTION_FREEDOOM1, sizeof(gamedescription));
-        else if (gamemode == retail)
-            M_StringCopy(gamedescription, s_CAPTION_ULTIMATE, sizeof(gamedescription));
-        else if (gamemode == registered)
-            M_StringCopy(gamedescription, s_CAPTION_REGISTERED, sizeof(gamedescription));
-        else if (gamemode == shareware)
-            M_StringCopy(gamedescription, s_CAPTION_SHAREWARE, sizeof(gamedescription));
+        else
+            M_StringCopy(gamedescription, s_CAPTION_DOOM, sizeof(gamedescription));
     }
     else
     {
         // DOOM 2 of some kind. But which mission?
         if (modifiedgame && *pwadfile)
         {
-            if (M_StringCompare(pwadfile, "nerve.wad"))
+            if (D_IsNERVEWAD(pwadfile))
                 M_StringCopy(gamedescription, s_CAPTION_DOOM2, sizeof(gamedescription));
             else
-                M_StringCopy(gamedescription, pwadfile, sizeof(gamedescription));
+            {
+                M_StringCopy(gamedescription, GetCorrectCase(pwadfile), sizeof(gamedescription));
+                return;
+            }
         }
         else if (FREEDOOM)
             M_StringCopy(gamedescription, (FREEDM ? s_CAPTION_FREEDM : s_CAPTION_FREEDOOM2), sizeof(gamedescription));
         else if (nerve)
             M_StringCopy(gamedescription, s_CAPTION_DOOM2, sizeof(gamedescription));
         else if (gamemission == doom2)
-            M_snprintf(gamedescription, sizeof(gamedescription), "%s: %s", s_CAPTION_DOOM2, s_CAPTION_HELLONEARTH);
+            M_snprintf(gamedescription, sizeof(gamedescription), "%s: %s", s_CAPTION_DOOM2, s_CAPTION_EXPANSION1);
         else if (gamemission == pack_plut)
             M_StringCopy(gamedescription, s_CAPTION_PLUTONIA, sizeof(gamedescription));
         else if (gamemission == pack_tnt)
             M_StringCopy(gamedescription, s_CAPTION_TNT, sizeof(gamedescription));
     }
-
-    if (nerve)
-    {
-        if (bfgedition)
-            C_Output("Playing " ITALICS("%s: %s (%s)") " and " ITALICS("%s: %s (%s)."), s_CAPTION_DOOM2, s_CAPTION_HELLONEARTH,
-                s_CAPTION_BFGEDITION, s_CAPTION_DOOM2, s_CAPTION_NERVE, s_CAPTION_BFGEDITION);
-        else
-            C_Output("Playing " ITALICS("%s: %s") " and " ITALICS("%s: %s."), s_CAPTION_DOOM2, s_CAPTION_HELLONEARTH,
-                s_CAPTION_DOOM2, s_CAPTION_NERVE);
-    }
-    else if (modifiedgame && !sigil && !chex && !BTSX && !REKKR)
-        C_Output("Playing " BOLD("%s") ".", gamedescription);
-    else
-    {
-        if (bfgedition && !chex && !BTSX && !REKKR)
-            C_Output("Playing " ITALICS("%s (%s)."), gamedescription, s_CAPTION_BFGEDITION);
-        else
-            C_Output("Playing " ITALICS("%s."), gamedescription);
-    }
 }
