@@ -1,52 +1,49 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
 #include <ctype.h>
 
 #include "c_console.h"
 #include "d_deh.h"
-#include "d_englsh.h"
+#include "d_english.h"
 #include "d_main.h"
 #include "doomstat.h"
 #include "g_game.h"
 #include "hu_stuff.h"
 #include "i_colors.h"
-#include "i_gamepad.h"
+#include "i_controller.h"
 #include "i_swap.h"
 #include "m_config.h"
 #include "m_menu.h"
@@ -57,8 +54,14 @@
 #include "v_data.h"
 #include "v_video.h"
 #include "w_wad.h"
+#include "wi_stuff.h"
 #include "z_zone.h"
 
+#define TEXTSPEED       (3 * FRACUNIT)          // original value
+#define TEXTWAIT        (250 * FRACUNIT)        // original value
+#define NEWTEXTSPEED    ((FRACUNIT + 50) / 100) // new value
+#define NEWTEXTWAIT     (1000 * FRACUNIT)       // new value
+
 typedef enum
 {
     F_STAGE_TEXT,
@@ -69,26 +72,17 @@ typedef enum
 // Stage of animation:
 static finalestage_t    finalestage;
 static int              finalecount;
-
-#define TEXTSPEED       (3 * FRACUNIT)          // original value
-#define TEXTWAIT        (250 * FRACUNIT)        // original value
-#define NEWTEXTSPEED    ((FRACUNIT + 50) / 100) // new value
-#define NEWTEXTWAIT     (1000 * FRACUNIT)       // new value
-
 static char             *finaletext;
 static char             *finaleflat;
 
+static bool             midstage;
+
 static void F_StartCast(void);
 static void F_CastTicker(void);
-static dboolean F_CastResponder(event_t *ev);
+static bool F_CastResponder(const event_t *ev);
 
-void WI_CheckForAccelerate(void);
 void A_RandomJump(mobj_t *actor, player_t *player, pspdef_t *psp);
 
-static dboolean         midstage;               // whether we're in "mid-stage"
-
-extern dboolean         acceleratestage;        // accelerate intermission screens
-
 //
 // F_ConsoleFinaleText
 //
@@ -111,8 +105,8 @@ static void F_ConsoleFinaleText(void)
 //
 void F_StartFinale(void)
 {
-    char    *intertext = P_GetInterText(gamemap);
-    char    *intersecret = P_GetInterSecretText(gamemap);
+    char    *intertext = P_GetInterText(gameepisode, gamemap);
+    char    *intersecret = P_GetInterSecretText(gameepisode, gamemap);
 
     viewactive = false;
     automapactive = false;
@@ -125,8 +119,8 @@ void F_StartFinale(void)
 
     if (*intertext || (*intersecret && secretexit))
     {
-        char    *interbackdrop = P_GetInterBackrop(gamemap);
-        int     mus = P_GetInterMusic(gamemap);
+        char    *interbackdrop = P_GetInterBackrop(gameepisode, gamemap);
+        int     mus = P_GetInterMusic(gameepisode, gamemap);
 
         if (!secretexit)
         {
@@ -156,7 +150,7 @@ void F_StartFinale(void)
         else
             S_ChangeMusic((gamemode == commercial ? mus_read_m : mus_victor), true, false, false);
     }
-    else if (P_GetMapEndCast(gamemap))
+    else if (P_GetMapEndCast(gameepisode, gamemap))
     {
         gameaction = ga_nothing;
         gamestate = GS_FINALE;
@@ -181,31 +175,31 @@ void F_StartFinale(void)
                     case 1:
                         finaleflat = bgflatE1;
                         finaletext = s_E1TEXT;
-
                         break;
 
                     case 2:
                         finaleflat = bgflatE2;
                         finaletext = s_E2TEXT;
-
                         break;
 
                     case 3:
                         finaleflat = bgflatE3;
                         finaletext = s_E3TEXT;
-
                         break;
 
                     case 4:
                         finaleflat = bgflatE4;
                         finaletext = s_E4TEXT;
-
                         break;
 
                     case 5:
                         finaleflat = bgflatE5;
                         finaletext = s_E5TEXT;
+                        break;
 
+                    case 6:
+                        finaleflat = bgflatE6;
+                        finaletext = s_E6TEXT;
                         break;
                 }
 
@@ -215,12 +209,12 @@ void F_StartFinale(void)
             case commercial:
                 S_ChangeMusic(mus_read_m, true, false, false);
 
-                switch (gamemap)      // This is regular DOOM II
+                switch (gamemap)
                 {
                     case 6:
                         finaleflat = bgflat06;
-                        finaletext = (gamemission == pack_tnt ? s_T1TEXT : (gamemission == pack_plut ? s_P1TEXT : s_C1TEXT));
-
+                        finaletext = (gamemission == pack_tnt ? s_T1TEXT :
+                            (gamemission == pack_plut ? s_P1TEXT : s_C1TEXT));
                         break;
 
                     case 8:
@@ -234,32 +228,32 @@ void F_StartFinale(void)
 
                     case 11:
                         finaleflat = bgflat11;
-                        finaletext = (gamemission == pack_tnt ? s_T2TEXT : (gamemission == pack_plut ? s_P2TEXT : s_C2TEXT));
-
+                        finaletext = (gamemission == pack_tnt ? s_T2TEXT :
+                            (gamemission == pack_plut ? s_P2TEXT : s_C2TEXT));
                         break;
 
                     case 20:
                         finaleflat = bgflat20;
-                        finaletext = (gamemission == pack_tnt ? s_T3TEXT : (gamemission == pack_plut ? s_P3TEXT : s_C3TEXT));
-
+                        finaletext = (gamemission == pack_tnt ? s_T3TEXT :
+                            (gamemission == pack_plut ? s_P3TEXT : s_C3TEXT));
                         break;
 
                     case 30:
                         finaleflat = bgflat30;
-                        finaletext = (gamemission == pack_tnt ? s_T4TEXT : (gamemission == pack_plut ? s_P4TEXT : s_C4TEXT));
-
+                        finaletext = (gamemission == pack_tnt ? s_T4TEXT :
+                            (gamemission == pack_plut ? s_P4TEXT : s_C4TEXT));
                         break;
 
                     case 15:
                         finaleflat = bgflat15;
-                        finaletext = (gamemission == pack_tnt ? s_T5TEXT : (gamemission == pack_plut ? s_P5TEXT : s_C5TEXT));
-
+                        finaletext = (gamemission == pack_tnt ? s_T5TEXT :
+                            (gamemission == pack_plut ? s_P5TEXT : s_C5TEXT));
                         break;
 
                     case 31:
                         finaleflat = bgflat31;
-                        finaletext = (gamemission == pack_tnt ? s_T6TEXT : (gamemission == pack_plut ? s_P6TEXT : s_C6TEXT));
-
+                        finaletext = (gamemission == pack_tnt ? s_T6TEXT :
+                            (gamemission == pack_plut ? s_P6TEXT : s_C6TEXT));
                         break;
                 }
 
@@ -270,7 +264,6 @@ void F_StartFinale(void)
                 S_ChangeMusic(mus_read_m, true, false, false);
                 finaleflat = "F_SKY1";
                 finaletext = s_C1TEXT;
-
                 break;
         }
 
@@ -289,7 +282,7 @@ void F_StartFinale(void)
     F_ConsoleFinaleText();
 }
 
-dboolean F_Responder(event_t *ev)
+bool F_Responder(const event_t *ev)
 {
     if (finalestage == F_STAGE_CAST)
         return F_CastResponder(ev);
@@ -299,6 +292,9 @@ dboolean F_Responder(event_t *ev)
 
 static fixed_t TextSpeed(void)
 {
+    if (acceleratestage)
+        S_StartSound(NULL, sfx_swtchn);
+
     return (midstage ? NEWTEXTSPEED : ((midstage = acceleratestage) ? (acceleratestage = false), NEWTEXTSPEED : TEXTSPEED));
 }
 
@@ -307,7 +303,7 @@ static fixed_t TextSpeed(void)
 //
 void F_Ticker(void)
 {
-    if (menuactive || paused || consoleactive)
+    if (menuactive || paused || consoleactive || !windowfocused)
         return;
 
     WI_CheckForAccelerate();
@@ -323,16 +319,16 @@ void F_Ticker(void)
         if (finalecount > FixedMul((fixed_t)strlen(finaletext) * FRACUNIT, TextSpeed()) + (midstage ? NEWTEXTWAIT : TEXTWAIT)
             || (midstage && acceleratestage))
         {
-            if (P_GetMapEndCast(gamemap))
+            if (P_GetMapEndCast(gameepisode, gamemap))
                 F_StartCast();
-            else if (P_GetMapEndBunny(gamemap))
+            else if (P_GetMapEndBunny(gameepisode, gamemap))
             {
                 finalecount = 0;
                 finalestage = F_STAGE_ARTSCREEN;
                 wipegamestate = GS_NONE;
                 S_StartMusic(mus_bunny);
             }
-            else if (P_GetMapEndGame(gamemap))
+            else if (P_GetMapEndGame(gameepisode, gamemap))
             {
                 finalecount = 0;
                 finalestage = F_STAGE_ARTSCREEN;
@@ -349,10 +345,15 @@ void F_Ticker(void)
             }
             else if (midstage)
             {
-                if (gamemap == 30 || (gamemission == pack_nerve && gamemap == 8))
+                if ((gamemap == 30 && !P_GetMapNext(gameepisode, gamemap))
+                    || (gamemission == pack_nerve && gamemap == 8))
                     F_StartCast();
                 else
+                {
+                    viewplayer->attackdown = true;
+                    viewplayer->usedown = true;
                     gameaction = ga_worlddone;
+                }
             }
         }
     }
@@ -364,37 +365,35 @@ void F_Ticker(void)
 static void F_TextWrite(void)
 {
     // draw some of the text onto the screen
-    int         w;
-    int         count = MAX(0, FixedDiv((finalecount - 10) * FRACUNIT, TextSpeed()) >> FRACBITS);
     const char  *ch = finaletext;
     int         cx = 12;
     int         cy = 10;
-    char        prev = ' ';
+    char        prev = '\0';
 
-    // erase the entire screen to a tiled background
-    int         lumpnum = W_GetNumForName(finaleflat);
-
-    if (W_LumpLength(lumpnum) == 64 * 64)
+    if (R_CheckFlatNumForName(finaleflat) == -1)
+        V_DrawPagePatch(0, W_CacheLumpName(finaleflat));
+    else if (W_CheckNumForName(finaleflat) >= 0)
     {
-        byte    *src = (byte *)W_CacheLumpNum(lumpnum);
-        byte    *dest = screens[0];
+        // erase the entire screen to a tiled background
+        const byte  *source = (byte *)W_CacheLumpName(finaleflat);
+        byte        *dest = screens[0];
 
         for (int y = 0; y < SCREENHEIGHT; y++)
             for (int x = 0; x < SCREENWIDTH; x += 2)
             {
-                byte    dot = src[(((y >> 1) & 63) << 6) + ((x >> 1) & 63)];
+                byte    dot = source[(((y >> 1) & 63) << 6) + ((x >> 1) & 63)];
 
                 *dest++ = dot;
                 *dest++ = dot;
             }
     }
     else
-        V_DrawPatch(0, 0, 0, W_CacheLumpNum(lumpnum));
+        memset(screens[0], nearestblack, SCREENAREA);
 
-    for (; count; count--)
+    for (int count = MAX(0, FixedDiv((finalecount - 10) * FRACUNIT, TextSpeed()) >> FRACBITS); count; count--)
     {
         char    letter = *ch++;
-        char    c;
+        int     c;
 
         if (!letter)
             break;
@@ -403,33 +402,33 @@ static void F_TextWrite(void)
         {
             cx = 12;
             cy += (prev == '\n' ? 8 : 11);
-            prev = letter;
-
+            prev = '\0';
             continue;
         }
 
-        c = toupper(letter) - HU_FONTSTART;
+        letter = toupper(letter);
+        c = letter - HU_FONTSTART;
 
         if (c < 0 || c >= HU_FONTSIZE)
         {
             cx += (prev == '.' || prev == '!' || prev == '?' || prev == '"' ? 5 : 3);
             prev = letter;
-
             continue;
         }
 
         if (cx > VANILLAWIDTH - 12)
             continue;
 
-        if (STCFN034)
+        if (STCFNxxx)
         {
-            w = SHORT(hu_font[c]->width);
-            V_DrawPatchWithShadow(cx + 1, cy + 1, hu_font[c], false);
+            if (cy + SHORT(hu_font[c]->height) > VANILLAHEIGHT)
+                break;
+
+            V_DrawMenuPatch(cx + 1, cy + 1, hu_font[c], false, VANILLAWIDTH);
+            cx += SHORT(hu_font[c]->width);
         }
         else
         {
-            int k = 0;
-
             if (prev == ' ')
             {
                 if (letter == '"')
@@ -438,23 +437,21 @@ static void F_TextWrite(void)
                     c = 65;
             }
 
-            while (kern[k].char1)
-            {
-                if (prev == kern[k].char1 && c == kern[k].char2)
+            for (int k = 0; kern[k].char1; k++)
+                if (prev == kern[k].char1 && letter == kern[k].char2)
                 {
                     cx += kern[k].adjust;
                     break;
                 }
 
-                k++;
-            }
+            if (cy + 10 > VANILLAHEIGHT)
+                break;
 
-            w = (int)strlen(smallcharset[c]) / 10 - 1;
-            M_DrawSmallChar(cx + 1, cy + 1, c, true);
+            M_DrawSmallChar(cx + 1, cy + 1, c, false, true);
+            cx += (int)strlen(smallcharset[c]) / 10 - 1;
         }
 
         prev = letter;
-        cx += w;
     }
 }
 
@@ -470,53 +467,54 @@ typedef struct
     char        *name;
     char        **dehackedname;
     mobjtype_t  type;
+    int         shadowoffset;
 } castinfo_t;
 
 static castinfo_t castorder[CASTNUMMAX] =
 {
-    { CC_ZOMBIE,  &s_CC_ZOMBIE,  MT_POSSESSED },
-    { CC_SHOTGUN, &s_CC_SHOTGUN, MT_SHOTGUY   },
-    { CC_HEAVY,   &s_CC_HEAVY,   MT_CHAINGUY  },
-    { CC_IMP,     &s_CC_IMP,     MT_TROOP     },
-    { CC_DEMON,   &s_CC_DEMON,   MT_SERGEANT  },
-    { CC_SPECTRE, &s_CC_SPECTRE, MT_SHADOWS   },
-    { CC_LOST,    &s_CC_LOST,    MT_SKULL     },
-    { CC_CACO,    &s_CC_CACO,    MT_HEAD      },
-    { CC_HELL,    &s_CC_HELL,    MT_KNIGHT    },
-    { CC_BARON,   &s_CC_BARON,   MT_BRUISER   },
-    { CC_ARACH,   &s_CC_ARACH,   MT_BABY      },
-    { CC_PAIN,    &s_CC_PAIN,    MT_PAIN      },
-    { CC_REVEN,   &s_CC_REVEN,   MT_UNDEAD    },
-    { CC_MANCU,   &s_CC_MANCU,   MT_FATSO     },
-    { CC_ARCH,    &s_CC_ARCH,    MT_VILE      },
-    { CC_SPIDER,  &s_CC_SPIDER,  MT_SPIDER    },
-    { CC_CYBER,   &s_CC_CYBER,   MT_CYBORG    },
-    { CC_HERO,    &s_CC_HERO,    MT_PLAYER    }
+    { CC_ZOMBIE,  &s_CC_ZOMBIE,  MT_POSSESSED, 2 },
+    { CC_SHOTGUN, &s_CC_SHOTGUN, MT_SHOTGUY,   2 },
+    { CC_HEAVY,   &s_CC_HEAVY,   MT_CHAINGUY,  2 },
+    { CC_IMP,     &s_CC_IMP,     MT_TROOP,     2 },
+    { CC_DEMON,   &s_CC_DEMON,   MT_SERGEANT,  2 },
+    { CC_SPECTRE, &s_CC_SPECTRE, MT_SHADOWS,   2 },
+    { CC_LOST,    &s_CC_LOST,    MT_SKULL,     0 },
+    { CC_CACO,    &s_CC_CACO,    MT_HEAD,      4 },
+    { CC_HELL,    &s_CC_HELL,    MT_KNIGHT,    4 },
+    { CC_BARON,   &s_CC_BARON,   MT_BRUISER,   4 },
+    { CC_ARACH,   &s_CC_ARACH,   MT_BABY,      0 },
+    { CC_PAIN,    &s_CC_PAIN,    MT_PAIN,      0 },
+    { CC_REVEN,   &s_CC_REVEN,   MT_UNDEAD,    4 },
+    { CC_MANCU,   &s_CC_MANCU,   MT_FATSO,     0 },
+    { CC_ARCH,    &s_CC_ARCH,    MT_VILE,      4 },
+    { CC_SPIDER,  &s_CC_SPIDER,  MT_SPIDER,    8 },
+    { CC_CYBER,   &s_CC_CYBER,   MT_CYBORG,    4 },
+    { CC_HERO,    &s_CC_HERO,    MT_PLAYER,    0 }
 };
 
 static int      castnum;
 static int      casttics;
 static state_t  *caststate;
 static int      castrot;
-static dboolean castdeath;
-static dboolean castdeathflip;
+static bool     castdeath;
+static bool     castdeathflip;
 static int      castframes;
-static dboolean castonmelee;
-static dboolean castattacking;
+static bool     castonmelee;
+static bool     castattacking;
 
-dboolean        firstevent;
+bool            firstevent;
 
 // [crispy] randomize seestate and deathstate sounds in the cast
 static int F_RandomizeSound(int sound)
 {
     if (sound >= sfx_posit1 && sound <= sfx_posit3)
-        return (sfx_posit1 + M_Random() % 3);
+        return (sfx_posit1 + M_BigRandom() % 3);
     else if (sound == sfx_bgsit1 || sound == sfx_bgsit2)
-        return (sfx_bgsit1 + M_Random() % 2);
+        return (sfx_bgsit1 + M_BigRandom() % 2);
     else if (sound >= sfx_podth1 && sound <= sfx_podth3)
-        return (sfx_podth1 + M_Random() % 3);
+        return (sfx_podth1 + M_BigRandom() % 3);
     else if (sound == sfx_bgdth1 || sound == sfx_bgdth2)
-        return (sfx_bgdth1 + M_Random() % 2);
+        return (sfx_bgdth1 + M_BigRandom() % 2);
     else
         return sound;
 }
@@ -580,7 +578,7 @@ static void F_CastTicker(void)
         if (!castdeath && caststate == &states[S_PLAY_ATK1])
             goto stopattack;    // Oh, gross hack!
 
-        st = (caststate->action == &A_RandomJump && M_Random() < caststate->misc2 ? caststate->misc1 : caststate->nextstate);
+        st = (caststate->action == &A_RandomJump && (M_BigRandom() & 255) < caststate->misc2 ? caststate->misc1 : caststate->nextstate);
         caststate = &states[st];
         castframes++;
 
@@ -675,7 +673,7 @@ static void F_CastTicker(void)
         }
 
         if (caststate == &states[S_PLAY_ATK1])
-            S_StartSound(viewplayer->mo, sfx_dshtgn);
+            S_StartSound(NULL, sfx_dshtgn);
     }
 
     if (castattacking && (castframes == 24 || caststate == &states[mobjinfo[castorder[castnum].type].seestate]))
@@ -692,7 +690,7 @@ stopattack:
     {
         if (caststate->action == &A_RandomJump)
         {
-            caststate = &states[M_Random() < caststate->misc2 ? caststate->misc1 : caststate->nextstate];
+            caststate = &states[((M_BigRandom() & 255) < caststate->misc2 ? caststate->misc1 : caststate->nextstate)];
             casttics = caststate->tics;
         }
 
@@ -704,7 +702,7 @@ stopattack:
 //
 // F_CastResponder
 //
-static dboolean F_CastResponder(event_t *ev)
+static bool F_CastResponder(const event_t *ev)
 {
     mobjtype_t  type;
 
@@ -733,7 +731,7 @@ static dboolean F_CastResponder(event_t *ev)
     if (ev->type == ev_mouse && !(ev->data1 & mousefire) && !(ev->data1 & mouseuse))
         return false;
 
-    if (ev->type == ev_gamepad && !(gamepadbuttons & gamepadfire) && !(gamepadbuttons & gamepaduse))
+    if (ev->type == ev_controller && !(controllerbuttons & controllerfire) && !(controllerbuttons & controlleruse))
         return false;
 
     if (castdeath)
@@ -761,14 +759,14 @@ static dboolean F_CastResponder(event_t *ev)
     castdeath = true;
 
     if (r_corpses_mirrored && type != MT_CHAINGUY && type != MT_CYBORG)
-        castdeathflip = M_Random() & 1;
+        castdeathflip = (M_Random() & 1);
 
     caststate = &states[mobjinfo[type].deathstate];
     casttics = caststate->tics;
 
     if (casttics == -1 && caststate->action == &A_RandomJump)
     {
-        caststate = &states[(M_Random() < caststate->misc2 ? caststate->misc1 : caststate->nextstate)];
+        caststate = &states[((M_BigRandom() & 255) < caststate->misc2 ? caststate->misc1 : caststate->nextstate)];
         casttics = caststate->tics;
     }
 
@@ -825,7 +823,7 @@ static void F_CastPrint(const char *text)
             continue;
         }
 
-        V_DrawPatchWithShadow(cx + 1, 181, hu_font[c], false);
+        V_DrawMenuPatch(cx + 1, 181, hu_font[c], false, VANILLAWIDTH);
         cx += SHORT(hu_font[c]->width);
     }
 }
@@ -835,13 +833,13 @@ static void F_CastPrint(const char *text)
 //
 static void F_CastDrawer(void)
 {
-    spritedef_t     *sprdef;
-    spriteframe_t   *sprframe;
-    int             lump;
-    int             rot = 0;
-    patch_t         *patch;
-    int             y = VANILLAHEIGHT - 30;
-    mobjtype_t      type = castorder[castnum].type;
+    spritedef_t         *sprdef;
+    spriteframe_t       *sprframe;
+    int                 lump;
+    int                 rot = 0;
+    patch_t             *patch;
+    int                 y = VANILLAHEIGHT - 30;
+    const mobjtype_t    type = castorder[castnum].type;
 
     if (gamemission == pack_plut)
         patch = W_CacheLumpName("BOSSBAC2");
@@ -850,13 +848,10 @@ static void F_CastDrawer(void)
     else
         patch = (FREEDOOM || hacx ? W_CacheLastLumpName(bgcastcall) : W_CacheLumpName(bgcastcall));
 
-    if (SCREENWIDTH != NONWIDEWIDTH)
-        memset(screens[0], FindDominantEdgeColor(patch), SCREENAREA);
+    V_DrawPagePatch(0, patch);
 
-    V_DrawWidePatch((SCREENWIDTH / SCREENSCALE - SHORT(patch->width)) / 2, 0, 0, patch);
-
-    if (M_StringCompare(castorder[castnum].name, *castorder[castnum].dehackedname))
-        F_CastPrint(type == MT_PLAYER ? playername : mobjinfo[type].name1);
+    if (type == MT_PLAYER && M_StringCompare(castorder[castnum].name, *castorder[castnum].dehackedname))
+        F_CastPrint(M_StringCompare(playername, playername_default) ? "You" : playername);
     else
         F_CastPrint(*castorder[castnum].dehackedname);
 
@@ -880,25 +875,23 @@ static void F_CastDrawer(void)
     {
         patch->leftoffset = (spritewidth[lump] - (r_fixspriteoffsets ? newspriteoffset[lump] : spriteoffset[lump])) >> FRACBITS;
 
-        if (r_shadows && ((type != MT_SKULL && type != MT_PAIN) || !castdeath))
+        if (r_shadows)
         {
             if (r_shadows_translucency)
             {
                 if (type == MT_SHADOWS)
-                    V_DrawFlippedSpectreShadowPatch(VANILLAWIDTH / 2, VANILLAHEIGHT - 28, patch);
+                    V_DrawFlippedSpectreShadowPatch(VANILLAWIDTH / 2,
+                        VANILLAHEIGHT - 32 - castorder[castnum].shadowoffset, patch);
                 else
-                    V_DrawFlippedShadowPatch(VANILLAWIDTH / 2, VANILLAHEIGHT - 28, patch);
+                    V_DrawFlippedShadowPatch(VANILLAWIDTH / 2,
+                        VANILLAHEIGHT - 32 - castorder[castnum].shadowoffset, patch);
             }
             else
-            {
-                if (type == MT_SHADOWS)
-                    V_DrawFlippedSolidSpectreShadowPatch(VANILLAWIDTH / 2, VANILLAHEIGHT - 28, patch);
-                else
-                    V_DrawFlippedSolidShadowPatch(VANILLAWIDTH / 2, VANILLAHEIGHT - 28, patch);
-            }
+                V_DrawFlippedSolidShadowPatch(VANILLAWIDTH / 2,
+                    VANILLAHEIGHT - 32 - castorder[castnum].shadowoffset, patch);
         }
 
-        if (r_translucency && (type == MT_SKULL || (type == MT_PAIN && castdeath)))
+        if (r_sprites_translucency && (type == MT_SKULL || (type == MT_PAIN && castdeath)))
             V_DrawFlippedTranslucentRedPatch(VANILLAWIDTH / 2, y, patch);
         else if (type == MT_SHADOWS)
             V_DrawFlippedFuzzPatch(VANILLAWIDTH / 2, y, patch);
@@ -909,25 +902,23 @@ static void F_CastDrawer(void)
     {
         patch->leftoffset = (r_fixspriteoffsets ? newspriteoffset[lump] : spriteoffset[lump]) >> FRACBITS;
 
-        if (r_shadows && ((type != MT_SKULL && type != MT_PAIN) || !castdeath))
+        if (r_shadows)
         {
             if (r_shadows_translucency)
             {
                 if (type == MT_SHADOWS)
-                    V_DrawSpectreShadowPatch(VANILLAWIDTH / 2, VANILLAHEIGHT - 28, patch);
+                    V_DrawSpectreShadowPatch(VANILLAWIDTH / 2,
+                        VANILLAHEIGHT - 32 - castorder[castnum].shadowoffset, patch);
                 else
-                    V_DrawShadowPatch(VANILLAWIDTH / 2, VANILLAHEIGHT - 28, patch);
+                    V_DrawShadowPatch(VANILLAWIDTH / 2,
+                        VANILLAHEIGHT - 32 - castorder[castnum].shadowoffset, patch);
             }
             else
-            {
-                if (type == MT_SHADOWS)
-                    V_DrawSolidSpectreShadowPatch(VANILLAWIDTH / 2, VANILLAHEIGHT - 28, patch);
-                else
-                    V_DrawSolidShadowPatch(VANILLAWIDTH / 2, VANILLAHEIGHT - 28, patch);
-            }
+                V_DrawSolidShadowPatch(VANILLAWIDTH / 2,
+                    VANILLAHEIGHT - 32 - castorder[castnum].shadowoffset, patch);
         }
 
-        if (r_translucency && (type == MT_SKULL || (type == MT_PAIN && castdeath)))
+        if (r_sprites_translucency && (type == MT_SKULL || (type == MT_PAIN && castdeath)))
             V_DrawTranslucentRedPatch(VANILLAWIDTH / 2, y, patch);
         else if (type == MT_SHADOWS)
             V_DrawFuzzPatch(VANILLAWIDTH / 2, y, patch);
@@ -937,20 +928,20 @@ static void F_CastDrawer(void)
 }
 
 //
-// F_DrawPatchCol
+// F_DrawPatchColumn
 //
-static void F_DrawPatchCol(int x, patch_t *patch, int col)
+static void F_DrawPatchColumn(int x, patch_t *patch, int col)
 {
-    column_t    *column = (column_t *)((byte *)patch + LONG(patch->columnofs[col]));
+    column_t    *column = (column_t *)((byte *)patch + LONG(patch->columnoffset[col]));
     byte        *desttop = &screens[0][x];
 
     // step through the posts in a column
     while (column->topdelta != 0xFF)
     {
-        int     srccol = 0;
-        byte    *source = (byte *)column + 3;
-        byte    *dest = &desttop[((column->topdelta * DY) >> FRACBITS) * SCREENWIDTH];
-        int     count = (column->length * DY) >> FRACBITS;
+        int         srccol = 0;
+        const byte  *source = (byte *)column + 3;
+        byte        *dest = &desttop[((column->topdelta * DY) >> FRACBITS) * SCREENWIDTH];
+        int         count = (column->length * DY) >> FRACBITS;
 
         while (count--)
         {
@@ -968,24 +959,26 @@ static void F_DrawPatchCol(int x, patch_t *patch, int col)
 //
 static void F_BunnyScroll(void)
 {
-    int     scrolled = BETWEEN(0, VANILLAWIDTH - (finalecount - 230) / 2, VANILLAWIDTH);
-    patch_t *p1 = (FREEDOOM || hacx ? W_CacheLastLumpName("PFUB2") : W_CacheLumpName("PFUB2"));
-    patch_t *p2 = (FREEDOOM || hacx ? W_CacheLastLumpName("PFUB1") : W_CacheLumpName("PFUB1"));
-    int     p1offset = (VANILLAWIDTH - SHORT(p1->width)) / 2;
-    int     p2offset = VANILLAWIDTH + (SHORT(p2->width) == VANILLAWIDTH ? -p1offset : p1offset);
-    int     pillarwidth = MAX(0, (SCREENWIDTH - (SHORT(p1->width) << FRACBITS) / DXI) / 2);
+    const int   scrolled = BETWEEN(0, VANILLAWIDTH - (finalecount - 230) / 2, VANILLAWIDTH);
+    patch_t     *p1 = (FREEDOOM || hacx ? W_CacheLastLumpName("PFUB2") :
+                    W_CacheLumpName(REKKR && W_CheckNumForName("PFUB2W") >= 0 ? "PFUB2W" : "PFUB2"));
+    patch_t     *p2 = (FREEDOOM || hacx ? W_CacheLastLumpName("PFUB1") :
+                    W_CacheLumpName(REKKR && W_CheckNumForName("PFUB1W") ? "PFUB1W" : "PFUB1"));
+    const int   p1offset = (VANILLAWIDTH - SHORT(p1->width)) / 2;
+    const int   p2offset = VANILLAWIDTH + (SHORT(p2->width) == VANILLAWIDTH ? -p1offset : p1offset);
+    const int   pillarwidth = MAX(0, (SCREENWIDTH - (SHORT(p1->width) << FRACBITS) / DXI) / 2);
 
     if (pillarwidth && SCREENWIDTH != NONWIDEWIDTH)
-        memset(screens[0], FindDominantEdgeColor(p1), SCREENAREA);
+        memset(screens[0], FindDominantEdgeColor(p1, SHORT(p1->height), 16), SCREENAREA);
 
     for (int x = pillarwidth; x < SCREENWIDTH - pillarwidth; x++)
     {
-        int x2 = ((x * DXI) >> FRACBITS) - WIDESCREENDELTA + scrolled;
+        const int   x2 = ((x * DXI) >> FRACBITS) - WIDESCREENDELTA + scrolled;
 
         if (x2 < p2offset)
-            F_DrawPatchCol(x, p1, x2 - p1offset);
+            F_DrawPatchColumn(x, p1, x2 - p1offset);
         else
-            F_DrawPatchCol(x, p2, x2 - p2offset);
+            F_DrawPatchColumn(x, p2, x2 - p2offset);
     }
 
     if (finalecount >= 1130)
@@ -997,14 +990,14 @@ static void F_BunnyScroll(void)
             if (finalecount == 1130)
                 D_FadeScreen(false);
 
-            V_DrawPatchWithShadow((VANILLAWIDTH - 104) / 2 + 1, (VANILLAHEIGHT - 64) / 2 + 1,
-                (FREEDOOM || hacx ? W_CacheLastLumpName("END0") : W_CacheLumpName("END0")), false);
+            V_DrawMenuPatch((VANILLAWIDTH - 104) / 2 + 1, (VANILLAHEIGHT - 64) / 2 + 1,
+                (FREEDOOM || hacx ? W_CacheLastLumpName("END0") : W_CacheLumpName("END0")), false, VANILLAWIDTH);
             laststage = 0;
         }
         else
         {
-            char    name[10];
-            int     stage = MIN((finalecount - 1180) / 5, 6);
+            char        name[10];
+            const int   stage = MIN((finalecount - 1180) / 5, 6);
 
             if (stage > laststage)
             {
@@ -1013,8 +1006,8 @@ static void F_BunnyScroll(void)
             }
 
             M_snprintf(name, sizeof(name), "END%i", stage);
-            V_DrawPatchWithShadow((VANILLAWIDTH - 104) / 2 + 1, (VANILLAHEIGHT - 64) / 2 + 1,
-                (FREEDOOM || hacx ? W_CacheLastLumpName(name) : W_CacheLumpName(name)), false);
+            V_DrawMenuPatch((VANILLAWIDTH - 104) / 2 + 1, (VANILLAHEIGHT - 64) / 2 + 1,
+                (FREEDOOM || hacx ? W_CacheLastLumpName(name) : W_CacheLumpName(name)), false, VANILLAWIDTH);
         }
     }
 }
@@ -1024,19 +1017,16 @@ static void F_BunnyScroll(void)
 //
 static void F_ArtScreenDrawer(void)
 {
-    int lumpnum = P_GetMapEndPic(gamemap);
+    const int   lumpnum = P_GetMapEndPic(gameepisode, gamemap);
 
     if (lumpnum > 0)
     {
         if (!finalestage)
             F_TextWrite();
         else
-        {
             V_DrawPatch(0, 0, 0, W_CacheLumpNum(lumpnum));
-            return;
-        }
     }
-    else if (P_GetMapEndBunny(gamemap) || gameepisode == 3)
+    else if (P_GetMapEndBunny(gameepisode, gamemap) || gameepisode == 3)
         F_BunnyScroll();
     else
     {
@@ -1045,18 +1035,25 @@ static void F_ArtScreenDrawer(void)
         switch (gameepisode)
         {
             case 1:
-                lump = (gamemode == retail ? creditlump : W_CacheLumpName("HELP2"));
+                lump = (gamemode == retail || gamemode == commercial ? creditlump :
+                    W_CacheLumpName("HELP2"));
                 break;
 
             case 2:
-                lump = (FREEDOOM || hacx ? W_CacheLastLumpName("VICTORY2") : W_CacheLumpName("VICTORY2"));
+                lump = (FREEDOOM || hacx ? W_CacheLastLumpName("VICTORY2") :
+                    W_CacheLumpName(REKKR && W_CheckNumForName("VICTORW2") >= 0 ? "VICTORW2" : "VICTORY2"));
                 break;
 
             case 4:
-                lump = (FREEDOOM || hacx ? W_CacheLastLumpName("ENDPIC") : W_CacheLumpName("ENDPIC"));
+                lump = (FREEDOOM || hacx ? W_CacheLastLumpName("ENDPIC") :
+                    W_CacheLumpName(REKKR && W_CheckNumForName("ENDPICW") >= 0 ? "ENDPICW" : "ENDPIC"));
                 break;
 
             case 5:
+                lump = W_CacheLastLumpName("SIGILEND");
+                break;
+
+            case 6:
                 lump = W_CacheLumpName("SIGILEND");
                 break;
 
@@ -1064,10 +1061,7 @@ static void F_ArtScreenDrawer(void)
                 return;
         }
 
-        if (SCREENWIDTH != NONWIDEWIDTH)
-            memset(screens[0], FindDominantEdgeColor(lump), SCREENAREA);
-
-        V_DrawWidePatch((SCREENWIDTH / SCREENSCALE - SHORT(lump->width)) / 2, 0, 0, lump);
+        V_DrawPagePatch(0, lump);
     }
 }
 
