@@ -1,45 +1,42 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
 #if defined(_WIN32)
 #pragma warning( disable : 4091 )
 
-#include <ShlObj.h>
+#include <Windows.h>
 #include <io.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -61,6 +58,7 @@
 
 #include "c_console.h"
 #include "i_system.h"
+#include "m_config.h"
 #include "m_misc.h"
 #include "version.h"
 #include "w_file.h"
@@ -89,9 +87,18 @@
 #endif
 #endif
 
-//
+const char *daynames[7] =
+{
+    "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"
+};
+
+const char *monthnames[12] =
+{
+    "January", "February", "March",     "April",   "May",      "June",
+    "July",    "August",   "September", "October", "November", "December"
+};
+
 // Create a directory
-//
 void M_MakeDirectory(const char *path)
 {
 #if defined(_WIN32)
@@ -102,7 +109,7 @@ void M_MakeDirectory(const char *path)
 }
 
 // Check if a file exists
-dboolean M_FileExists(const char *filename)
+bool M_FileExists(const char *filename)
 {
     FILE    *fstream = fopen(filename, "r");
 
@@ -122,6 +129,7 @@ char *M_FileCaseExists(const char *path)
 {
     char    *path_dup = M_StringDuplicate(path);
     char    *filename;
+    char    *ufilename;
     char    *ext;
 
     // actual path
@@ -140,37 +148,41 @@ char *M_FileCaseExists(const char *path)
         return path_dup;
 
     // uppercase filename, e.g. DOOM2.WAD
-    uppercase(filename);
-
-    if (M_FileExists(path_dup))
-        return path_dup;
+    ufilename = uppercase(filename);
 
     // uppercase basename with lowercase extension, e.g. DOOM2.wad
-    if ((ext = strrchr(path_dup, '.')) && ext > filename)
+    if ((ext = strrchr(path_dup, '.')) && ext > ufilename)
     {
         lowercase(ext + 1);
 
         if (M_FileExists(path_dup))
+        {
+            free(ufilename);
             return path_dup;
+        }
     }
 
     // lowercase filename with uppercase first letter, e.g. Doom2.wad
-    if (strlen(filename) > 1)
+    if (strlen(ufilename) > 1)
     {
-        lowercase(filename + 1);
+        lowercase(ufilename + 1);
 
         if (M_FileExists(path_dup))
+        {
+            free(ufilename);
             return path_dup;
+        }
     }
 
     // no luck
+    free(ufilename);
     free(path_dup);
     return NULL;
 }
 #endif
 
 // Check if a folder exists
-dboolean M_FolderExists(const char *folder)
+bool M_FolderExists(const char *folder)
 {
     struct stat status;
 
@@ -183,11 +195,13 @@ void M_StringCopy(char *dest, const char *src, const size_t dest_size)
     if (dest_size >= 1)
     {
         dest[dest_size - 1] = '\0';
-        strncpy(dest, src, dest_size - 1);
+
+        if (dest_size > 1)
+            strncpy(dest, src, dest_size - 1);
     }
 }
 
-char *M_ExtractFolder(char *path)
+char *M_ExtractFolder(const char *path)
 {
     char    *pos;
     char    *folder;
@@ -213,7 +227,7 @@ char *M_GetAppDataFolder(void)
     // On Linux and macOS, if ../share/doomretro doesn't exist then we're dealing with
     // a portable installation, and we write doomretro.cfg to the executable directory.
     char    *resourcefolder = M_StringJoin(executablefolder,
-                DIR_SEPARATOR_S ".." DIR_SEPARATOR_S "share" DIR_SEPARATOR_S PACKAGE, NULL);
+                DIR_SEPARATOR_S ".." DIR_SEPARATOR_S "share" DIR_SEPARATOR_S DOOMRETRO, NULL);
     DIR     *resourcedir = opendir(resourcefolder);
 
     free(resourcefolder);
@@ -225,7 +239,7 @@ char *M_GetAppDataFolder(void)
         NSFileManager   *manager = [NSFileManager defaultManager];
         NSURL           *baseAppSupportURL = [manager URLsForDirectory : NSApplicationSupportDirectory
                             inDomains : NSUserDomainMask].firstObject;
-        NSURL           *appSupportURL = [baseAppSupportURL URLByAppendingPathComponent : @PACKAGE_NAME];
+        NSURL           *appSupportURL = [baseAppSupportURL URLByAppendingPathComponent : @DOOMRETRO_NAME];
 
         closedir(resourcedir);
 
@@ -240,7 +254,11 @@ char *M_GetAppDataFolder(void)
         closedir(resourcedir);
         free(executablefolder);
 
-        return M_StringJoin(buffer, DIR_SEPARATOR_S ".config" DIR_SEPARATOR_S PACKAGE, NULL);
+#if defined(__HAIKU__)
+        return M_StringJoin(buffer, DIR_SEPARATOR_S "config" DIR_SEPARATOR_S "settings" DIR_SEPARATOR_S DOOMRETRO, NULL);
+#else
+        return M_StringJoin(buffer, DIR_SEPARATOR_S ".config" DIR_SEPARATOR_S DOOMRETRO, NULL);
+#endif
 #endif
     }
     else
@@ -256,7 +274,7 @@ char *M_GetResourceFolder(void)
     // On Linux and macOS, first assume that the executable is in ../bin and
     // try to load resources from ../share/doomretro.
     char    *resourcefolder = M_StringJoin(executablefolder,
-                DIR_SEPARATOR_S ".." DIR_SEPARATOR_S "share" DIR_SEPARATOR_S PACKAGE, NULL);
+                DIR_SEPARATOR_S ".." DIR_SEPARATOR_S "share" DIR_SEPARATOR_S DOOMRETRO, NULL);
     DIR     *resourcedir = opendir(resourcefolder);
 
     if (resourcedir)
@@ -274,6 +292,7 @@ char *M_GetResourceFolder(void)
 
     return (char *)resourceURL.fileSystemRepresentation;
 #else
+    free(resourcefolder);
     // And on Linux, fall back to the same folder as the executable.
     return executablefolder;
 #endif
@@ -287,15 +306,17 @@ char *M_GetResourceFolder(void)
 char *M_GetExecutableFolder(void)
 {
 #if defined(_WIN32)
-    char    *pos;
     char    *folder = malloc(MAX_PATH);
-    TCHAR   buffer[MAX_PATH];
 
-    GetModuleFileName(NULL, buffer, MAX_PATH);
-    M_StringCopy(folder, buffer, MAX_PATH);
+    if (folder)
+    {
+        char    *pos;
 
-    if (folder && (pos = strrchr(folder, DIR_SEPARATOR)))
-        *pos = '\0';
+        GetModuleFileName(NULL, folder, MAX_PATH);
+
+        if ((pos = strrchr(folder, DIR_SEPARATOR)))
+            *pos = '\0';
+    }
 
     return folder;
 #elif defined(__linux__) || defined(__NetBSD__) || defined(__sun)
@@ -320,41 +341,41 @@ char *M_GetExecutableFolder(void)
         return M_StringDuplicate(dirname(exe));
     }
 #elif defined(__FreeBSD__) || defined(__DragonFly__)
-    char    *exe = malloc(MAX_PATH);
+    char    exe[MAX_PATH];
     size_t  len = MAX_PATH;
     int     mib[] = { CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1 };
 
     if (!sysctl(mib, 4, exe, &len, NULL, 0))
     {
         exe[len] = '\0';
-        return dirname(exe);
+        return M_StringDuplicate(dirname(exe));
     }
     else
     {
         strcpy(exe, ".");
-        return exe;
+        return M_StringDuplicate(exe);
     }
 #elif defined(__APPLE__)
-    char        *exe = malloc(MAX_PATH);
+    char        exe[MAX_PATH];
     uint32_t    len = MAX_PATH;
 
     if (_NSGetExecutablePath(exe, &len))
     {
         strcpy(exe, ".");
-        return exe;
+        return M_StringDuplicate(exe);
     }
 
-    return dirname(exe);
+    return M_StringDuplicate(dirname(exe));
 #elif defined(__HAIKU__)
-    char    *exe = malloc(MAX_PATH);
+    char    exe[MAX_PATH];
 
     exe[0] = '\0';
 
     if (find_path(B_APP_IMAGE_SYMBOL, B_FIND_PATH_IMAGE_PATH, NULL, exe, MAX_PATH) == B_OK)
-        return dirname(exe);
+        return M_StringDuplicate(dirname(exe));
 
     strcpy(exe, ".");
-    return exe;
+    return M_StringDuplicate(exe);
 #else
     char    *folder = malloc(2);
 
@@ -381,7 +402,7 @@ char *M_TempFile(char *s)
 
 // Return a newly-malloced string with all the strings given as arguments
 // concatenated together.
-char *M_StringJoin(char *s, ...)
+char *M_StringJoin(const char *s, ...)
 {
     char        *result;
     const char  *v;
@@ -416,26 +437,23 @@ char *M_StringJoin(char *s, ...)
     }
 
     va_end(args);
-
     return result;
 }
 
-dboolean M_StrToInt(const char *str, unsigned int *result)
+bool M_StrToInt(const char *str, int *result)
 {
-    return (sscanf(str, " 0x%2x", result) == 1 || sscanf(str, " 0X%2x", result) == 1
-        || sscanf(str, " 0%3o", result) == 1 || sscanf(str, " %10u", result) == 1);
+    return (sscanf(str, " 0x%2x", (unsigned int *)result) == 1
+        || sscanf(str, " 0X%2x", (unsigned int *)result) == 1
+        || sscanf(str, " 0%3o", (unsigned int *)result) == 1
+        || sscanf(str, " %12d", result) == 1);
 }
 
-//
-// M_StrCaseStr
-//
 // Case-insensitive version of strstr()
-//
 const char *M_StrCaseStr(const char *haystack, const char *needle)
 {
-    int haystack_len = (int)strlen(haystack);
-    int needle_len = (int)strlen(needle);
-    int len;
+    const int   haystack_len = (int)strlen(haystack);
+    const int   needle_len = (int)strlen(needle);
+    int         len;
 
     if (haystack_len < needle_len)
         return NULL;
@@ -449,7 +467,8 @@ const char *M_StrCaseStr(const char *haystack, const char *needle)
     return NULL;
 }
 
-static char *stristr(char *ch1, char *ch2)
+#if !defined(stristr)
+static char *stristr(char *ch1, const char *ch2)
 {
     char    *chN1 = M_StringDuplicate(ch1);
     char    *chN2 = M_StringDuplicate(ch2);
@@ -482,16 +501,15 @@ static char *stristr(char *ch1, char *ch2)
 
     return chRet;
 }
+#endif
 
-//
 // String replace function.
-//
-char *M_StringReplace(char *haystack, char *needle, const char *replacement)
+char *M_StringReplaceFirst(char *haystack, const char *needle, const char *replacement)
 {
     static char buffer[4096];
     char        *p;
 
-    if (!(p = stristr(haystack, needle)))
+    if (!(p = stristr(haystack, (char *)needle)))
         return haystack;
 
     strncpy(buffer, haystack, p - haystack);
@@ -501,31 +519,67 @@ char *M_StringReplace(char *haystack, char *needle, const char *replacement)
     return buffer;
 }
 
-void M_StringReplaceAll(char *haystack, char *needle, const char *replacement)
+#if !defined(strrstr)
+char *strrstr(const char *haystack, const char *needle)
 {
-    char    buffer[1024] = "";
-    char    *insert_point = &buffer[0];
-    char    *tmp = haystack;
-    int     needle_len = (int)strlen(needle);
-    int     repl_len = (int)strlen(replacement);
+    char    *r = NULL;
+
+    if (!needle[0])
+        return (char *)haystack + strlen(haystack);
 
     while (true)
     {
-        char    *p = stristr(tmp, needle);
+        char    *p = strstr(haystack, needle);
+
+        if (!p)
+            return r;
+
+        r = p;
+        haystack = p + 1;
+    }
+}
+#endif
+
+char *M_StringReplaceLast(char *haystack, const char *needle, const char *replacement)
+{
+    static char buffer[4096];
+    char        *p;
+
+    if (!(p = strrstr(haystack, (char *)needle)))
+        return haystack;
+
+    strncpy(buffer, haystack, p - haystack);
+    buffer[p - haystack] = '\0';
+    sprintf(buffer + (p - haystack), "%s%s", replacement, p + strlen(needle));
+
+    return buffer;
+}
+
+void M_StringReplaceAll(char *haystack, const char *needle, const char *replacement, bool usecase)
+{
+    char        buffer[1024] = "";
+    char        *insert_point = &buffer[0];
+    char        *temp = haystack;
+    const int   needle_len = (int)strlen(needle);
+    const int   repl_len = (int)strlen(replacement);
+
+    while (true)
+    {
+        char    *p = (usecase ? strstr(temp, (char *)needle) : stristr(temp, (char *)needle));
 
         if (!p)
         {
-            strcpy(insert_point, tmp);
+            strcpy(insert_point, temp);
             break;
         }
 
-        memcpy(insert_point, tmp, p - tmp);
-        insert_point += p - tmp;
+        memcpy(insert_point, temp, p - temp);
+        insert_point += p - temp;
 
         memcpy(insert_point, replacement, repl_len);
         insert_point += repl_len;
 
-        tmp = p + needle_len;
+        temp = p + needle_len;
     }
 
     strcpy(haystack, buffer);
@@ -544,32 +598,24 @@ char *M_StringDuplicate(const char *orig)
 
 // Returns true if str1 and str2 are the same.
 // (Case-insensitive, return value reverse of strcasecmp() to avoid confusion.
-dboolean M_StringCompare(const char *str1, const char *str2)
+bool M_StringCompare(const char *str1, const char *str2)
 {
     return !strcasecmp(str1, str2);
 }
 
 // Returns true if string begins with the specified prefix.
-dboolean M_StringStartsWith(const char *s, const char *prefix)
+bool M_StringStartsWith(const char *s, const char *prefix)
 {
-    size_t  len = strlen(prefix);
+    const size_t    len = strlen(prefix);
 
     return (strlen(s) >= len && !strncasecmp(s, prefix, len));
 }
 
-// Returns true if string begins with the specified prefix.
-dboolean M_StringStartsWithExact(const char *s, const char *prefix)
-{
-    size_t  len = strlen(prefix);
-
-    return (strlen(s) >= len && !strncmp(s, prefix, len));
-}
-
 // Returns true if string ends with the specified suffix.
-dboolean M_StringEndsWith(const char *s, const char *suffix)
+bool M_StringEndsWith(const char *s, const char *suffix)
 {
-    size_t  len1 = strlen(s);
-    size_t  len2 = strlen(suffix);
+    const size_t    len1 = strlen(s);
+    const size_t    len2 = strlen(suffix);
 
     return (len1 >= len2 && M_StringCompare(s + len1 - len2, suffix));
 }
@@ -579,10 +625,10 @@ void M_vsnprintf(char *buf, int buf_len, const char *s, va_list args)
 {
     if (buf_len >= 1)
     {
-        // Windows (and other OSes?) has a vsnprintf() that doesn't always
+        // Windows (and other OSes?) have a vsnprintf() that doesn't always
         // append a trailing \0. So we must do it, and write into a buffer
         // that is one byte shorter; otherwise this function is unsafe.
-        int result = vsnprintf(buf, buf_len, s, args);
+        const int   result = vsnprintf(buf, buf_len, s, args);
 
         // If truncated, change the final char in the buffer to a \0.
         // A negative result indicates a truncated buffer on Windows.
@@ -604,20 +650,20 @@ void M_snprintf(char *buf, int buf_len, const char *s, ...)
 #if !defined(strndup)
 char *strndup(const char *s, size_t n)
 {
-    size_t  len = strnlen(s, n);
-    char    *new = malloc(len + 1);
+    const size_t    len = strnlen(s, n);
+    char            *new = malloc(len + 1);
 
     if (!new)
         return NULL;
 
     new[len] = '\0';
-    return memcpy(new, s, len);
+    return (char *)memcpy(new, s, len);
 }
 #endif
 
 char *M_SubString(const char *str, size_t begin, size_t len)
 {
-    size_t  length = strlen(str);
+    const size_t    length = strlen(str);
 
     if (!length || length < begin || length < begin + len)
         return 0;
@@ -646,8 +692,8 @@ char *lowercase(char *str)
 
 char *titlecase(const char *str)
 {
-    char    *newstr = M_StringDuplicate(str);
-    int     len = (int)strlen(newstr);
+    char        *newstr = M_StringDuplicate(str);
+    const int   len = (int)strlen(newstr);
 
     if (len > 0)
     {
@@ -674,6 +720,28 @@ char *sentencecase(const char *str)
     return newstr;
 }
 
+bool isuppercase(const char *str)
+{
+    const int   len = (int)strlen(str);
+
+    for (int i = 0; i < len; i++)
+        if (islower(str[i]))
+            return false;
+
+    return true;
+}
+
+bool islowercase(const char *str)
+{
+    const int   len = (int)strlen(str);
+
+    for (int i = 0; i < len; i++)
+        if (isupper(str[i]))
+            return false;
+
+    return true;
+}
+
 char *commify(int64_t value)
 {
     char    result[64];
@@ -689,11 +757,8 @@ char *commify(int64_t value)
 
         n = result + sizeof(result) - pt;
 
-        do
-        {
-            pt -= 3;
-
-            if (pt > result)
+        while (true)
+            if ((pt -= 3) > result)
             {
                 memmove(pt + 1, pt, n);
                 *pt = ',';
@@ -701,7 +766,6 @@ char *commify(int64_t value)
             }
             else
                 break;
-        } while (true);
     }
 
     return M_StringDuplicate(result);
@@ -722,8 +786,7 @@ char *commifystat(uint64_t value)
 
         n = result + sizeof(result) - pt;
 
-        do
-        {
+        while (true)
             if ((pt -= 3) > result)
             {
                 memmove(pt + 1, pt, n);
@@ -732,7 +795,6 @@ char *commifystat(uint64_t value)
             }
             else
                 break;
-        } while (true);
     }
 
     return M_StringDuplicate(result);
@@ -761,9 +823,9 @@ char *uncommify(const char *input)
     return p;
 }
 
-dboolean wildcard(char *input, char *pattern)
+bool wildcard(char *input, char *pattern)
 {
-    if (!*pattern)
+    if (!*pattern || M_StringCompare(input, pattern))
         return true;
 
     for (int i = 0; pattern[i] != '\0'; i++)
@@ -772,8 +834,8 @@ dboolean wildcard(char *input, char *pattern)
             continue;
         else if (pattern[i] == '*')
         {
-            for (int z = i; input[z] != '\0'; z++)
-                if (wildcard(input + z, pattern + i + 1))
+            for (int j = i; input[j] != '\0'; j++)
+                if (wildcard(input + j, pattern + i + 1))
                     return true;
 
             return false;
@@ -790,10 +852,10 @@ int gcd(int a, int b)
     return (!b ? a : gcd(b, a % b));
 }
 
-int numspaces(char *str)
+int numspaces(const char *str)
 {
-    int result = 0;
-    int len = (int)strlen(str);
+    int         result = 0;
+    const int   len = (int)strlen(str);
 
     for (int i = 0; i < len; i++)
         result += (str[i] == ' ');
@@ -836,7 +898,7 @@ char *removenonalpha(const char *input)
         char    *p2 = p;
 
         while (*input != '\0')
-            if (!isspace((unsigned char)*input) && *input != '-' && *input != '(' && *input != ')')
+            if (isalnum((unsigned char)*input))
                 *p2++ = *input++;
             else
                 input++;
@@ -869,8 +931,8 @@ char *trimwhitespace(char *input)
 
 char *makevalidfilename(const char *input)
 {
-    char    *newstr = M_StringDuplicate(input);
-    int     len = (int)strlen(newstr);
+    char        *newstr = M_StringDuplicate(input);
+    const int   len = (int)strlen(newstr);
 
     for (int i = 0; i < len; i++)
         if (strchr("\\/:?\"<>|", newstr[i]))
@@ -905,17 +967,17 @@ char *removeext(const char *file)
     return newstr;
 }
 
-dboolean isvowel(const char ch)
+bool isvowel(const char ch)
 {
     return !!strchr("aeiouAEIOU", ch);
 }
 
-dboolean ispunctuation(const char ch)
+bool ispunctuation(const char ch)
 {
     return !!strchr(".!?", ch);
 }
 
-dboolean isbreak(const char ch)
+bool isbreak(const char ch)
 {
     return !!strchr(" /\\-", ch);
 }
@@ -928,7 +990,8 @@ char *striptrailingzero(float value, int precision)
     {
         int len;
 
-        M_snprintf(result, sizeof(result), "%.*f", (precision == 2 ? 2 : (value != floor(value))), value);
+        M_snprintf(result, sizeof(result), "%.*f",
+            (precision == 2 ? 2 : (value != floor(value))), value);
         len = (int)strlen(result);
 
         if (len >= 4 && result[len - 3] == '.' && result[len - 1] == '0')
@@ -938,44 +1001,140 @@ char *striptrailingzero(float value, int precision)
     return result;
 }
 
-static const long hextable[] =
+void M_StripQuotes(char *str)
 {
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-     0,  1,  2,  3,  4,  5,  6,  7,  8,  9, -1, -1, -1, -1, -1, -1,
-    -1, 10, 11, 12, 13, 14, 15, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, 10, 11, 12, 13, 14, 15, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
-};
+    int len = (int)strlen(str);
 
-int hextodec(char *hex)
-{
-    int ret = 0;
-
-    while (*hex && ret >= 0)
-        ret = ((ret << 4) || hextable[*hex++]);
-
-    return ret;
-}
-
-void M_StripQuotes(char *string)
-{
-    int len = (int)strlen(string);
-
-    if (len > 2 && ((string[0] == '"' && string[len - 1] == '"') || (string[0] == '\'' && string[len - 1] == '\'')))
+    if (len > 2
+        && (((str[0] == '"' || str[0] == '\x93')
+            && (str[len - 1] == '"' || str[len - 1] == '\x94'))
+            || ((str[0] == '\'' || str[0] == '\x91')
+                && (str[len - 1] == '\'' || str[len - 1] == '\x92'))))
     {
         len -= 2;
-        memmove(string, string + 1, len);
-        string[len] = '\0';
+        memmove(str, str + 1, len);
+        str[len] = '\0';
     }
 }
+
+void M_NormalizeSlashes(char *str)
+{
+    char    *p;
+
+    // Convert all slashes/backslashes to DIR_SEPARATOR
+    for (p = str; *p; p++)
+        if ((*p == '/' || *p == '\\') && *p != DIR_SEPARATOR)
+            *p = DIR_SEPARATOR;
+
+    // Remove trailing slashes
+    while (p > str && *(--p) == DIR_SEPARATOR)
+        *p = 0;
+
+    // Collapse multiple slashes
+    for (p = str; (*str++ = *p); )
+        if (*p++ == DIR_SEPARATOR)
+            while (*p == DIR_SEPARATOR)
+                p++;
+}
+
+const char *pronoun(const pronoun_t type)
+{
+    if (type == personal)
+        return (playergender == playergender_male ? "he" :
+            (playergender == playergender_female ? "she" : "they"));
+    else if (type == possessive)
+        return (playergender == playergender_male ? "his" :
+            (playergender == playergender_female ? "her" : "their"));
+    else
+        return (playergender == playergender_male ? "himself" :
+            (playergender == playergender_female ? "herself" : "themselves"));
+}
+
+const char *words[][2] =
+{
+    { "agoniz",     "agonis"     }, { "airplane",   "aeroplane"  },
+    { "analog",     "analogue"   }, { "armor",      "armour"     },
+    { "artifact",   "artefact"   }, { "barreled",   "barrelled"  },
+    { "behavior",   "behaviour"  }, { "caliber",    "calibre"    },
+    { "centered",   "centred"    }, { "centering",  "centring"   },
+    { "center",     "centre"     }, { "color",      "colour"     },
+    { "defense",    "defence"    }, { "dialog",     "dialogue"   },
+    { "disk",       "disc"       }, { "donut",      "doughnut"   },
+    { "endeavor",   "endeavour"  }, { "favor",      "favour"     },
+    { "fiber",      "fibre"      }, { "flavor",     "flavour"    },
+    { "gray",       "grey"       }, { "harbor",     "harbour"    },
+    { "honor",      "honour"     }, { "humor",      "humour"     },
+    { "initializ",  "initialis"  }, { "inquir",     "enquir"     },
+    { "jewelry",    "jewellery"  }, { "judgment",   "judgement"  },
+    { "labor",      "labour"     }, { "license",    "licence"    },
+    { "liter",      "litre"      }, { "meter",      "metre"      },
+    { "neighbor",   "neighbour"  }, { "offense",    "offence"    },
+    { "organiz",    "organis"    }, { "practice",   "practise"   },
+    { "program",    "programme"  }, { "realiz",     "realis"     },
+    { "randomiz",   "randomis"   }, { "recogniz",   "recognis"   },
+    { "refueling",  "refuelling" }, { "rumor",      "rumour"     },
+    { "savior",     "saviour"    }, { "savor",      "savour"     },
+    { "skeptic",    "sceptic"    }, { "specializ",  "specialis"  },
+    { "stabiliz",   "stabilis"   }, { "standardiz", "standardis" },
+    { "synchroniz", "synchronis" }, { "theater",    "theatre"    },
+    { "traveled",   "travelled"  }, { "traveling",  "travelling" },
+    { "utiliz",     "utilis"     }, { "vapor",      "vapour"     },
+    { "whiskey",    "whisky"     }, { "yogurt",     "yoghurt"    },
+    { "",           ""           }
+};
+
+static void M_Translate(char *string, const char *word1, const char *word2)
+{
+    char    *temp1 = M_StringDuplicate(word1);
+    char    *temp2 = M_StringDuplicate(word2);
+
+    M_StringReplaceAll(string, temp1, temp2, true);
+
+    temp1[0] = toupper(temp1[0]);
+    temp2[0] = toupper(temp2[0]);
+
+    M_StringReplaceAll(string, temp1, temp2, true);
+
+    free(temp1);
+    free(temp2);
+
+    temp1 = uppercase(word1);
+    temp2 = uppercase(word2);
+
+    M_StringReplaceAll(string, temp1, temp2, true);
+
+    free(temp1);
+    free(temp2);
+}
+
+void M_AmericanToBritishEnglish(char *string)
+{
+    for (int i = 0; *words[i][0]; i++)
+        M_Translate(string, words[i][0], words[i][1]);
+}
+
+void M_BritishToAmericanEnglish(char *string)
+{
+    for (int i = 0; *words[i][0]; i++)
+        M_Translate(string, words[i][1], words[i][0]);
+}
+
+void M_TranslateAutocomplete(void)
+{
+    if (english == english_american)
+        for (int i = 0; *autocompletelist[i].text; i++)
+            M_BritishToAmericanEnglish(autocompletelist[i].text);
+    else
+        for (int i = 0; *autocompletelist[i].text; i++)
+            M_AmericanToBritishEnglish(autocompletelist[i].text);
+}
+
+const char *dayofweek(int day, int month, int year)
+{
+    const int   adjustment = (14 - month) / 12;
+
+    month += 12 * adjustment - 2;
+    year -= adjustment;
+
+    return daynames[(day + (13 * month - 1) / 5 + year + year / 4 - year / 100 + year / 400) % 7];
+}
