@@ -1,57 +1,54 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
+#include "c_console.h"
 #include "d_deh.h"
 #include "doomstat.h"
 #include "hu_stuff.h"
 #include "m_config.h"
 #include "m_misc.h"
 #include "p_local.h"
+#include "p_setup.h"
 #include "p_tick.h"
 #include "s_sound.h"
 #include "z_zone.h"
 
-dboolean    r_graduallighting = r_graduallighting_default;
-
 static void T_GradualLightingToDoor(vldoor_t *door)
 {
-    sector_t    *sec = door->sector;
-    fixed_t     level = door->topheight - sec->floorheight;
+    sector_t        *sec = door->sector;
+    const fixed_t   level = door->topheight - sec->floorheight;
 
     if (level > 0)
     {
@@ -80,31 +77,27 @@ void T_VerticalDoor(vldoor_t *door)
             if (!--door->topcountdown)
                 switch (door->type)
                 {
-                    case doorBlazeRaise:
-                    case genBlazeRaise:
+                    case DoorBlazeRaise:
+                    case GenBlazeRaise:
                         door->direction = -1;   // time to go back down
                         S_StartSectorSound(&door->sector->soundorg, sfx_bdcls);
-
                         break;
 
-                    case doorNormal:
-                    case genRaise:
+                    case DoorNormal:
+                    case GenRaise:
                         door->direction = -1;   // time to go back down
                         S_StartSectorSound(&door->sector->soundorg, sfx_dorcls);
-
                         break;
 
-                    case doorClose30ThenOpen:
-                    case genCdO:
+                    case DoorClose30ThenOpen:
+                    case GenCdO:
                         door->direction = 1;    // time to go back up
                         S_StartSectorSound(&door->sector->soundorg, sfx_doropn);
-
                         break;
 
-                    case genBlazeCdO:
+                    case GenBlazeCdO:
                         door->direction = 1;    // time to go back up
                         S_StartSectorSound(&door->sector->soundorg, sfx_bdopn);
-
                         break;
 
                     default:
@@ -118,11 +111,10 @@ void T_VerticalDoor(vldoor_t *door)
             if (!--door->topcountdown)
                 switch (door->type)
                 {
-                    case doorRaiseIn5Mins:
+                    case DoorRaiseIn5Mins:
                         door->direction = 1;
-                        door->type = doorNormal;
+                        door->type = DoorNormal;
                         S_StartSectorSound(&door->sector->soundorg, sfx_doropn);
-
                         break;
 
                     default:
@@ -137,37 +129,34 @@ void T_VerticalDoor(vldoor_t *door)
 
             // killough 10/98: implement gradual lighting effects
             // [BH] enhanced to apply effects to all doors
-            if (r_graduallighting)
+            if (r_graduallighting && !nograduallighting)
                 T_GradualLightingToDoor(door);
 
             if (res == pastdest)
             {
                 switch (door->type)
                 {
-                    case doorBlazeRaise:
-                    case doorBlazeClose:
-                    case genBlazeRaise:
-                    case genBlazeClose:
-                    case doorNormal:
-                    case doorClose:
-                    case genRaise:
-                    case genClose:
+                    case DoorBlazeRaise:
+                    case DoorBlazeClose:
+                    case GenBlazeRaise:
+                    case GenBlazeClose:
+                    case DoorNormal:
+                    case DoorClose:
+                    case GenRaise:
+                    case GenClose:
                         door->sector->ceilingdata = NULL;
-                        P_RemoveThinker(&door->thinker);        // unlink and free
-
+                        P_RemoveThinker2(&door->thinker);     // unlink and free
                         break;
 
-                    case doorClose30ThenOpen:
+                    case DoorClose30ThenOpen:
                         door->direction = 0;
                         door->topcountdown = TICRATE * 30;
-
                         break;
 
-                    case genCdO:
-                    case genBlazeCdO:
+                    case GenCdO:
+                    case GenBlazeCdO:
                         door->direction = 0;
-                        door->topcountdown = door->topwait;     // jff 5/8/98 insert delay
-
+                        door->topcountdown = door->topwait;     // jff 05/08/98 insert delay
                         break;
 
                     default:
@@ -178,24 +167,22 @@ void T_VerticalDoor(vldoor_t *door)
             {
                 switch (door->type)
                 {
-                    case doorBlazeClose:
-                    case doorClose:
-                    case genClose:
-                    case genBlazeClose:                         // DO NOT GO BACK UP!
+                    case DoorBlazeClose:
+                    case DoorClose:
+                    case GenClose:
+                    case GenBlazeClose:                         // DO NOT GO BACK UP!
                         break;
 
                     // [BH] play correct sound when raising fast doors
-                    case doorBlazeRaise:
-                    case genBlazeRaise:
+                    case DoorBlazeRaise:
+                    case GenBlazeRaise:
                         door->direction = 1;
                         S_StartSectorSound(&door->sector->soundorg, sfx_bdopn);
-
                         break;
 
                     default:
                         door->direction = 1;
                         S_StartSectorSound(&door->sector->soundorg, sfx_doropn);
-
                         break;
                 }
             }
@@ -208,32 +195,30 @@ void T_VerticalDoor(vldoor_t *door)
 
             // killough 10/98: implement gradual lighting effects
             // [BH] enhanced to apply effects to all doors
-            if (r_graduallighting)
+            if (r_graduallighting && !nograduallighting)
                 T_GradualLightingToDoor(door);
 
             if (res == pastdest)
             {
                 switch (door->type)
                 {
-                    case doorBlazeRaise:
-                    case doorNormal:
-                    case genRaise:
-                    case genBlazeRaise:
+                    case DoorBlazeRaise:
+                    case DoorNormal:
+                    case GenRaise:
+                    case GenBlazeRaise:
                         door->direction = 0;                    // wait at top
                         door->topcountdown = door->topwait;
-
                         break;
 
-                    case doorClose30ThenOpen:
-                    case doorBlazeOpen:
-                    case doorOpen:
-                    case genBlazeOpen:
-                    case genOpen:
-                    case genCdO:
-                    case genBlazeCdO:
+                    case DoorClose30ThenOpen:
+                    case DoorBlazeOpen:
+                    case DoorOpen:
+                    case GenBlazeOpen:
+                    case GenOpen:
+                    case GenCdO:
+                    case GenBlazeCdO:
                         door->sector->ceilingdata = NULL;
-                        P_RemoveThinker(&door->thinker);        // unlink and free
-
+                        P_RemoveThinker2(&door->thinker);     // unlink and free
                         break;
 
                     default:
@@ -249,7 +234,7 @@ void T_VerticalDoor(vldoor_t *door)
 // EV_DoLockedDoor
 // Move a locked door up/down
 //
-dboolean EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing, fixed_t speed)
+bool EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing, fixed_t speed)
 {
     player_t    *player = thing->player;
     static char buffer[1024];
@@ -265,31 +250,37 @@ dboolean EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing, fixed_t spe
             {
                 // [BH] display player message distinguishing between keycard and skull key
                 // [BH] flash needed key on HUD
-                if (player->cards[it_bluecard] == CARDNOTFOUNDYET)
+                if (player->cards[it_blueskull] == CARDNOTFOUNDYET)
                 {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_bluecard))
-                    {
-                        player->neededcard = it_bluecard;
-                        player->neededcardflash = NEEDEDCARDFLASH;
-                    }
-
-                    M_snprintf(buffer, sizeof(buffer), s_PD_BLUEO, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_KEYCARD);
-                    HU_PlayerMessage(buffer, false, false);
-                }
-                else if (player->cards[it_blueskull] == CARDNOTFOUNDYET)
-                {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_blueskull))
+                    if (!player->neededcardflash || player->neededcard != it_blueskull)
                     {
                         player->neededcard = it_blueskull;
                         player->neededcardflash = NEEDEDCARDFLASH;
                     }
 
-                    M_snprintf(buffer, sizeof(buffer), s_PD_BLUEO, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_SKULLKEY);
-                    HU_PlayerMessage(buffer, false, false);
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_BLUEO, "You", "", s_PD_SKULLKEY);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_BLUEO, playername, "s", s_PD_SKULLKEY);
+                }
+                else
+                {
+                    if (!player->neededcardflash || player->neededcard != it_bluecard)
+                    {
+                        player->neededcard = it_bluecard;
+                        player->neededcardflash = NEEDEDCARDFLASH;
+                    }
+
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_BLUEO, "You", "", s_PD_KEYCARD);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_BLUEO, playername, "s", s_PD_KEYCARD);
                 }
 
+                if (autousing && M_StringCompare(buffer, console[numconsolestrings - 1].string))
+                    return false;
+
+                HU_PlayerMessage(buffer, false, false);
                 S_StartSound(player->mo, sfx_noway);    // [BH] use sfx_noway instead of sfx_oof
                 return false;
             }
@@ -302,31 +293,37 @@ dboolean EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing, fixed_t spe
             {
                 // [BH] display player message distinguishing between keycard and skull key
                 // [BH] flash needed key on HUD
-                if (player->cards[it_redcard] == CARDNOTFOUNDYET)
+                if (player->cards[it_redskull] == CARDNOTFOUNDYET)
                 {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_redcard))
-                    {
-                        player->neededcard = it_redcard;
-                        player->neededcardflash = NEEDEDCARDFLASH;
-                    }
-
-                    M_snprintf(buffer, sizeof(buffer), s_PD_REDO, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_KEYCARD);
-                    HU_PlayerMessage(buffer, false, false);
-                }
-                else if (player->cards[it_redskull] == CARDNOTFOUNDYET)
-                {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_redskull))
+                    if (!player->neededcardflash || player->neededcard != it_redskull)
                     {
                         player->neededcard = it_redskull;
                         player->neededcardflash = NEEDEDCARDFLASH;
                     }
 
-                    M_snprintf(buffer, sizeof(buffer), s_PD_REDO, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_SKULLKEY);
-                    HU_PlayerMessage(buffer, false, false);
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_REDO, "You", "", s_PD_SKULLKEY);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_REDO, playername, "s", s_PD_SKULLKEY);
+                }
+                else
+                {
+                    if (!player->neededcardflash || player->neededcard != it_redcard)
+                    {
+                        player->neededcard = it_redcard;
+                        player->neededcardflash = NEEDEDCARDFLASH;
+                    }
+
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_REDO, "You", "", s_PD_KEYCARD);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_REDO, playername, "s", s_PD_KEYCARD);
                 }
 
+                if (autousing && M_StringCompare(buffer, console[numconsolestrings - 1].string))
+                    return false;
+
+                HU_PlayerMessage(buffer, false, false);
                 S_StartSound(player->mo, sfx_noway);    // [BH] use sfx_noway instead of sfx_oof
                 return false;
             }
@@ -339,31 +336,37 @@ dboolean EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing, fixed_t spe
             {
                 // [BH] display player message distinguishing between keycard and skull key
                 // [BH] flash needed key on HUD
-                if (player->cards[it_yellowcard] == CARDNOTFOUNDYET)
+                if (player->cards[it_yellowskull] == CARDNOTFOUNDYET)
                 {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_yellowcard))
-                    {
-                        player->neededcard = it_yellowcard;
-                        player->neededcardflash = NEEDEDCARDFLASH;
-                    }
-
-                    M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWO, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_KEYCARD);
-                    HU_PlayerMessage(buffer, false, false);
-                }
-                else if (player->cards[it_yellowskull] == CARDNOTFOUNDYET)
-                {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_yellowskull))
+                    if (!player->neededcardflash || player->neededcard != it_yellowskull)
                     {
                         player->neededcard = it_yellowskull;
                         player->neededcardflash = NEEDEDCARDFLASH;
                     }
 
-                    M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWO, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_SKULLKEY);
-                    HU_PlayerMessage(buffer, false, false);
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWO, "You", "", s_PD_SKULLKEY);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWO, playername, "s", s_PD_SKULLKEY);
+                }
+                else
+                {
+                    if (!player->neededcardflash || player->neededcard != it_yellowcard)
+                    {
+                        player->neededcard = it_yellowcard;
+                        player->neededcardflash = NEEDEDCARDFLASH;
+                    }
+
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWO, "You", "", s_PD_KEYCARD);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWO, playername, "s", s_PD_KEYCARD);
                 }
 
+                if (autousing && M_StringCompare(buffer, console[numconsolestrings - 1].string))
+                    return false;
+
+                HU_PlayerMessage(buffer, false, false);
                 S_StartSound(player->mo, sfx_noway);    // [BH] use sfx_noway instead of sfx_oof
                 return false;
             }
@@ -374,10 +377,10 @@ dboolean EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing, fixed_t spe
     return EV_DoDoor(line, type, speed);
 }
 
-dboolean EV_DoDoor(line_t *line, vldoor_e type, fixed_t speed)
+bool EV_DoDoor(line_t *line, vldoor_e type, fixed_t speed)
 {
     int         secnum = -1;
-    dboolean    rtn = false;
+    bool        rtn = false;
     sector_t    *sec;
 
     if (P_ProcessNoTagLines(line, &sec, &secnum))
@@ -415,11 +418,11 @@ manual_door:
         door->type = type;
         door->topwait = VDOORWAIT;
         door->speed = speed;
-        door->line = line;      // jff 1/31/98 remember line that triggered us
+        door->line = line;      // jff 01/31/98 remember line that triggered us
 
         switch (type)
         {
-            case doorBlazeClose:
+            case DoorBlazeClose:
                 door->topheight = P_FindLowestCeilingSurrounding(sec);
                 door->topheight -= 4 * FRACUNIT;
                 door->direction = -1;
@@ -429,7 +432,7 @@ manual_door:
 
                 break;
 
-            case doorClose:
+            case DoorClose:
                 door->topheight = P_FindLowestCeilingSurrounding(sec);
                 door->topheight -= 4 * FRACUNIT;
                 door->direction = -1;
@@ -439,7 +442,7 @@ manual_door:
 
                 break;
 
-            case doorClose30ThenOpen:
+            case DoorClose30ThenOpen:
                 door->topheight = sec->ceilingheight;
                 door->direction = -1;
 
@@ -448,8 +451,8 @@ manual_door:
 
                 break;
 
-            case doorBlazeRaise:
-            case doorBlazeOpen:
+            case DoorBlazeRaise:
+            case DoorBlazeOpen:
                 door->direction = 1;
                 door->topheight = P_FindLowestCeilingSurrounding(sec);
                 door->topheight -= 4 * FRACUNIT;
@@ -459,8 +462,8 @@ manual_door:
 
                 break;
 
-            case doorNormal:
-            case doorOpen:
+            case DoorNormal:
+            case DoorOpen:
                 door->direction = 1;
                 door->topheight = P_FindLowestCeilingSurrounding(sec);
                 door->topheight -= 4 * FRACUNIT;
@@ -479,7 +482,7 @@ manual_door:
             sec->lines[i]->flags &= ~ML_SECRET;
 
         if (zerotag_manual)
-            return rtn; //e6y
+            return rtn; // e6y
     }
 
     return rtn;
@@ -497,9 +500,11 @@ void EV_VerticalDoor(line_t *line, mobj_t *thing)
     vldoor_t    *door;
 
     // if the wrong side of door is pushed, give oof sound
-    if (line->sidenum[1] == NO_INDEX && player) // killough
+    if (line->sidenum[1] == NO_INDEX && player)     // killough
     {
-        S_StartSound(player->mo, sfx_noway);    // [BH] use sfx_noway instead of sfx_oof
+        if (!autousing && P_DoorClosed(line))
+            S_StartSound(player->mo, sfx_noway);    // [BH] use sfx_noway instead of sfx_oof
+
         return;
     }
 
@@ -514,31 +519,37 @@ void EV_VerticalDoor(line_t *line, mobj_t *thing)
             {
                 // [BH] display player message distinguishing between keycard and skull key
                 // [BH] flash needed key on HUD
-                if (player->cards[it_bluecard] == CARDNOTFOUNDYET)
+                if (player->cards[it_blueskull] == CARDNOTFOUNDYET)
                 {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_bluecard))
-                    {
-                        player->neededcard = it_bluecard;
-                        player->neededcardflash = NEEDEDCARDFLASH;
-                    }
-
-                    M_snprintf(buffer, sizeof(buffer), s_PD_BLUEK, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_KEYCARD);
-                    HU_PlayerMessage(buffer, false, false);
-                }
-                else if (player->cards[it_blueskull] == CARDNOTFOUNDYET)
-                {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_blueskull))
+                    if (!player->neededcardflash || player->neededcard != it_blueskull)
                     {
                         player->neededcard = it_blueskull;
                         player->neededcardflash = NEEDEDCARDFLASH;
                     }
 
-                    M_snprintf(buffer, sizeof(buffer), s_PD_BLUEK, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_SKULLKEY);
-                    HU_PlayerMessage(buffer, false, false);
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_BLUEK, "You", "", s_PD_SKULLKEY);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_BLUEK, playername, "s", s_PD_SKULLKEY);
+                }
+                else
+                {
+                    if (!player->neededcardflash || player->neededcard != it_bluecard)
+                    {
+                        player->neededcard = it_bluecard;
+                        player->neededcardflash = NEEDEDCARDFLASH;
+                    }
+
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_BLUEK, "You", "", s_PD_KEYCARD);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_BLUEK, playername, "s", s_PD_KEYCARD);
                 }
 
+                if (autousing && M_StringCompare(buffer, console[numconsolestrings - 1].string))
+                    return;
+
+                HU_PlayerMessage(buffer, false, false);
                 S_StartSound(player->mo, sfx_noway);    // [BH] use sfx_noway instead of sfx_oof
                 return;
             }
@@ -554,31 +565,37 @@ void EV_VerticalDoor(line_t *line, mobj_t *thing)
             {
                 // [BH] display player message distinguishing between keycard and skull key
                 // [BH] flash needed key on HUD
-                if (player->cards[it_yellowcard] == CARDNOTFOUNDYET)
+                if (player->cards[it_yellowskull] == CARDNOTFOUNDYET)
                 {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_yellowcard))
-                    {
-                        player->neededcard = it_yellowcard;
-                        player->neededcardflash = NEEDEDCARDFLASH;
-                    }
-
-                    M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWK, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_KEYCARD);
-                    HU_PlayerMessage(buffer, false, false);
-                }
-                else if (player->cards[it_yellowskull] == CARDNOTFOUNDYET)
-                {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_yellowskull))
+                    if (!player->neededcardflash || player->neededcard != it_yellowskull)
                     {
                         player->neededcard = it_yellowskull;
                         player->neededcardflash = NEEDEDCARDFLASH;
                     }
 
-                    M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWK, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_SKULLKEY);
-                    HU_PlayerMessage(buffer, false, false);
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWK, "You", "", s_PD_SKULLKEY);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWK, playername, "s", s_PD_SKULLKEY);
+                }
+                else
+                {
+                    if (!player->neededcardflash || player->neededcard != it_yellowcard)
+                    {
+                        player->neededcard = it_yellowcard;
+                        player->neededcardflash = NEEDEDCARDFLASH;
+                    }
+
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWK, "You", "", s_PD_KEYCARD);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_YELLOWK, playername, "s", s_PD_KEYCARD);
                 }
 
+                if (autousing && M_StringCompare(buffer, console[numconsolestrings - 1].string))
+                    return;
+
+                HU_PlayerMessage(buffer, false, false);
                 S_StartSound(player->mo, sfx_noway);    // [BH] use sfx_noway instead of sfx_oof
                 return;
             }
@@ -594,31 +611,37 @@ void EV_VerticalDoor(line_t *line, mobj_t *thing)
             {
                 // [BH] display player message distinguishing between keycard and skull key
                 // [BH] flash needed key on HUD
-                if (player->cards[it_redcard] == CARDNOTFOUNDYET)
+                if (player->cards[it_redskull] == CARDNOTFOUNDYET)
                 {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_redcard))
-                    {
-                        player->neededcard = it_redcard;
-                        player->neededcardflash = NEEDEDCARDFLASH;
-                    }
-
-                    M_snprintf(buffer, sizeof(buffer), s_PD_REDK, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_KEYCARD);
-                    HU_PlayerMessage(buffer, false, false);
-                }
-                else if (player->cards[it_redskull] == CARDNOTFOUNDYET)
-                {
-                    if (vid_widescreen && r_hud && (!player->neededcardflash || player->neededcard != it_redskull))
+                    if (!player->neededcardflash || player->neededcard != it_redskull)
                     {
                         player->neededcard = it_redskull;
                         player->neededcardflash = NEEDEDCARDFLASH;
                     }
 
-                    M_snprintf(buffer, sizeof(buffer), s_PD_REDK, playername,
-                        (M_StringCompare(playername, playername_default) ? "" : "s"), s_SKULLKEY);
-                    HU_PlayerMessage(buffer, false, false);
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_REDK, "You", "", s_PD_SKULLKEY);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_REDK, playername, "s", s_PD_SKULLKEY);
+                }
+                else
+                {
+                    if (!player->neededcardflash || player->neededcard != it_redcard)
+                    {
+                        player->neededcard = it_redcard;
+                        player->neededcardflash = NEEDEDCARDFLASH;
+                    }
+
+                    if (M_StringCompare(playername, playername_default))
+                        M_snprintf(buffer, sizeof(buffer), s_PD_REDK, "You", "", s_PD_KEYCARD);
+                    else
+                        M_snprintf(buffer, sizeof(buffer), s_PD_REDK, playername, "s", s_PD_KEYCARD);
                 }
 
+                if (autousing && M_StringCompare(buffer, console[numconsolestrings - 1].string))
+                    return;
+
+                HU_PlayerMessage(buffer, false, false);
                 S_StartSound(player->mo, sfx_noway);    // [BH] use sfx_noway instead of sfx_oof
                 return;
             }
@@ -644,7 +667,7 @@ void EV_VerticalDoor(line_t *line, mobj_t *thing)
                     door->direction = 1;        // go back up
 
                     // [BH] play correct door sound
-                    S_StartSectorSound(&sec->soundorg, (door->type == doorBlazeRaise ? sfx_bdopn : sfx_doropn));
+                    S_StartSectorSound(&sec->soundorg, (door->type == DoorBlazeRaise ? sfx_bdopn : sfx_doropn));
                 }
                 else
                 {
@@ -656,7 +679,7 @@ void EV_VerticalDoor(line_t *line, mobj_t *thing)
                         door->direction = -1;   // start going down immediately
 
                         // [BH] play correct door sound
-                        S_StartSectorSound(&sec->soundorg, (door->type == doorBlazeRaise ? sfx_bdcls : sfx_dorcls));
+                        S_StartSectorSound(&sec->soundorg, (door->type == DoorBlazeRaise ? sfx_bdcls : sfx_dorcls));
                     }
                     else if (door->thinker.function == &T_PlatRaise)
                     {
@@ -696,7 +719,7 @@ void EV_VerticalDoor(line_t *line, mobj_t *thing)
     door->direction = 1;
     door->speed = VDOORSPEED;
     door->topwait = VDOORWAIT;
-    door->line = line;          // jff 1/31/98 remember line that triggered us
+    door->line = line;          // jff 01/31/98 remember line that triggered us
 
     // killough 10/98: use gradual lighting changes if nonzero tag given
     // [BH] check if tag is valid
@@ -709,29 +732,26 @@ void EV_VerticalDoor(line_t *line, mobj_t *thing)
         case DR_Door_Blue_OpenWaitClose:
         case DR_Door_Yellow_OpenWaitClose:
         case DR_Door_Red_OpenWaitClose:
-            door->type = doorNormal;
+            door->type = DoorNormal;
             break;
 
         case D1_Door_OpenStay:
         case D1_Door_Blue_OpenStay:
         case D1_Door_Red_OpenStay:
         case D1_Door_Yellow_OpenStay:
-            door->type = doorOpen;
+            door->type = DoorOpen;
             line->special = 0;
-
             break;
 
         case DR_Door_OpenWaitClose_Fast:
-            door->type = doorBlazeRaise;
+            door->type = DoorBlazeRaise;
             door->speed = VDOORSPEED * 4;
-
             break;
 
         case D1_Door_OpenStay_Fast:
-            door->type = doorBlazeOpen;
+            door->type = DoorBlazeOpen;
             line->special = 0;
             door->speed = VDOORSPEED * 4;
-
             break;
 
         default:
@@ -760,7 +780,7 @@ void P_SpawnDoorCloseIn30(sector_t *sec)
     sec->ceilingdata = door;
     sec->special = 0;
     door->sector = sec;
-    door->type = doorNormal;
+    door->type = DoorNormal;
     door->speed = VDOORSPEED;
     door->topcountdown = 30 * TICRATE;
     door->topheight = sec->ceilingheight;
@@ -780,7 +800,7 @@ void P_SpawnDoorRaiseIn5Mins(sector_t *sec)
     sec->special = 0;
     door->sector = sec;
     door->direction = 2;
-    door->type = doorRaiseIn5Mins;
+    door->type = DoorRaiseIn5Mins;
     door->speed = VDOORSPEED;
     door->topheight = P_FindLowestCeilingSurrounding(sec);
     door->topheight -= 4 * FRACUNIT;
