@@ -1,39 +1,36 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
 #include "doomstat.h"
@@ -45,12 +42,6 @@
 #include "s_sound.h"
 #include "z_zone.h"
 
-#define FLOOR   0
-#define CEILING 1
-
-#define DOWN   -1
-#define UP      1
-
 //
 // FLOORS
 //
@@ -58,18 +49,19 @@
 //
 // Move a plane (floor or ceiling) and check for crushing
 //
-result_e T_MovePlane(sector_t *sector, fixed_t speed, fixed_t dest, dboolean crush, int floororceiling, int direction)
+result_e T_MovePlane(sector_t *sector, const fixed_t speed, fixed_t dest,
+    const bool crush, const int floororceiling, const int direction)
 {
     sector->oldgametime = gametime;
 
     switch (floororceiling)
     {
-        case FLOOR:
+        case 0:
             sector->oldfloorheight = sector->floorheight;
 
             switch (direction)
             {
-                case DOWN:
+                case -1:
                     if (sector->floorheight - speed < dest)
                     {
                         sector->floorheight = dest;
@@ -90,8 +82,8 @@ result_e T_MovePlane(sector_t *sector, fixed_t speed, fixed_t dest, dboolean cru
 
                     break;
 
-                case UP:
-                    dest = MIN(dest, sector->ceilingheight);
+                case 1:
+                    dest = (compat_floormove || dest < sector->ceilingheight ? dest : sector->ceilingheight);
 
                     if (sector->floorheight + speed > dest)
                     {
@@ -123,13 +115,13 @@ result_e T_MovePlane(sector_t *sector, fixed_t speed, fixed_t dest, dboolean cru
 
             break;
 
-        case CEILING:
+        case 1:
             sector->oldceilingheight = sector->ceilingheight;
 
             switch (direction)
             {
-                case DOWN:
-                    dest = MAX(dest, sector->floorheight);
+                case -1:
+                    dest = (compat_floormove || dest > sector->floorheight ? dest : sector->floorheight);
 
                     if (sector->ceilingheight - speed < dest)
                     {
@@ -162,7 +154,7 @@ result_e T_MovePlane(sector_t *sector, fixed_t speed, fixed_t dest, dboolean cru
 
                     break;
 
-                case UP:
+                case 1:
                     if (sector->ceilingheight + speed > dest)
                     {
                         sector->ceilingheight = dest;
@@ -202,44 +194,42 @@ void T_MoveFloor(floormove_t *floor)
 
     if (res != pastdest)
     {
-        if (!(leveltime & 7))
+        if (!(maptime & 7))
             S_StartSectorSound(&sec->soundorg, sfx_stnmov);
 
         return;
     }
 
-    if (floor->direction == UP)
+    if (floor->direction == 1)
     {
         switch (floor->type)
         {
-            case donutRaise:
-            case genFloorChgT:
-            case genFloorChg0:
+            case DonutRaise:
+            case GenFloorChgT:
+            case GenFloorChg0:
                 sec->special = floor->newspecial;
 
-            case genFloorChg:
+            case GenFloorChg:
                 sec->floorpic = floor->texture;
                 P_CheckTerrainType(sec);
-
                 break;
 
             default:
                 break;
         }
     }
-    else if (floor->direction == DOWN)
+    else if (floor->direction == -1)
     {
         switch (floor->type)
         {
-            case lowerAndChange:
-            case genFloorChgT:
-            case genFloorChg0:
+            case LowerAndChange:
+            case GenFloorChgT:
+            case GenFloorChg0:
                 sec->special = floor->newspecial;
 
-            case genFloorChg:
+            case GenFloorChg:
                 sec->floorpic = floor->texture;
                 P_CheckTerrainType(sec);
-
                 break;
 
             default:
@@ -248,9 +238,9 @@ void T_MoveFloor(floormove_t *floor)
     }
 
     floor->sector->floordata = NULL;
-    P_RemoveThinker(&floor->thinker);
+    P_RemoveThinker2(&floor->thinker);
 
-    // jff 2/26/98 implement stair retrigger lockout while still building
+    // jff 02/26/98 implement stair retrigger lockout while still building
     // note this only applies to the retriggerable generalized stairs
     if (sec->stairlock == -2)               // if this sector is stairlocked
     {
@@ -301,34 +291,34 @@ void T_MoveElevator(elevator_t *elevator)
     result_e    res;
     sector_t    *sec = elevator->sector;
 
-    if (elevator->direction == DOWN)            // moving down
+    if (elevator->direction == -1)              // moving down
     {
-        // jff 4/7/98 reverse order of ceiling/floor
+        // jff 04/07/98 reverse order of ceiling/floor
         res = T_MovePlane(sec, elevator->speed, elevator->ceilingdestheight, false, 1, elevator->direction);
 
-        // jff 4/7/98 don't move ceiling if blocked
+        // jff 04/07/98 don't move ceiling if blocked
         if (res == ok || res == pastdest)
             T_MovePlane(sec, elevator->speed, elevator->floordestheight, false, 0, elevator->direction);
     }
     else                                        // up
     {
-        // jff 4/7/98 reverse order of ceiling/floor
+        // jff 04/07/98 reverse order of ceiling/floor
         res = T_MovePlane(sec, elevator->speed, elevator->floordestheight, false, 0, elevator->direction);
 
-        // jff 4/7/98 don't move floor if blocked
+        // jff 04/07/98 don't move floor if blocked
         if (res == ok || res == pastdest)
             T_MovePlane(sec, elevator->speed, elevator->ceilingdestheight, false, 1, elevator->direction);
     }
 
     // make floor move sound
-    if (!(leveltime & 7))
+    if (!(maptime & 7))
         S_StartSectorSound(&sec->soundorg, sfx_stnmov);
 
     if (res == pastdest)                        // if destination height achieved
     {
         sec->floordata = NULL;
         sec->ceilingdata = NULL;
-        P_RemoveThinker(&elevator->thinker);    // remove elevator from actives
+        P_RemoveThinker2(&elevator->thinker);   // remove elevator from actives
 
         // make floor stop sound
         // [BH] don't make stop sound if floor already at its destination height
@@ -340,10 +330,10 @@ void T_MoveElevator(elevator_t *elevator)
 //
 // HANDLE FLOOR TYPES
 //
-dboolean EV_DoFloor(line_t *line, floor_e floortype)
+bool EV_DoFloor(const line_t *line, const floor_e floortype)
 {
     int         secnum = -1;
-    dboolean    rtn = false;
+    bool        rtn = false;
     sector_t    *sec;
 
     if (P_ProcessNoTagLines(line, &sec, &secnum))
@@ -382,48 +372,43 @@ manual_floor:
 
         switch (floortype)
         {
-            case lowerFloor:
-                floor->direction = DOWN;
+            case LowerFloor:
+                floor->direction = -1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = P_FindHighestFloorSurrounding(sec);
-
                 break;
 
-            case lowerFloor24:
-                floor->direction = DOWN;
+            case LowerFloor24:
+                floor->direction = -1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = floor->sector->floorheight + 24 * FRACUNIT;
-
                 break;
 
-            case lowerFloor32Turbo:
-                floor->direction = DOWN;
+            case LowerFloor32Turbo:
+                floor->direction = -1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED * 4;
                 floor->floordestheight = floor->sector->floorheight + 32 * FRACUNIT;
-
                 break;
 
-            case lowerFloorToLowest:
-                floor->direction = DOWN;
+            case LowerFloorToLowest:
+                floor->direction = -1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = P_FindLowestFloorSurrounding(sec);
-
                 break;
 
-            case lowerFloorToNearest:
-                floor->direction = DOWN;
+            case LowerFloorToNearest:
+                floor->direction = -1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = P_FindNextLowestFloor(sec, sec->floorheight);
-
                 break;
 
-            case turboLower:
-                floor->direction = DOWN;
+            case TurboLower:
+                floor->direction = -1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED * 4;
                 floor->floordestheight = P_FindHighestFloorSurrounding(sec);
@@ -433,60 +418,54 @@ manual_floor:
 
                 break;
 
-            case raiseFloorCrush:
+            case RaiseFloorCrush:
                 floor->crush = true;
 
-            case raiseFloor:
-                floor->direction = UP;
+            case RaiseFloor:
+                floor->direction = 1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = MIN(P_FindLowestCeilingSurrounding(sec), sec->ceilingheight)
-                    - 8 * FRACUNIT * (floortype == raiseFloorCrush);
-
+                    - 8 * FRACUNIT * (floortype == RaiseFloorCrush);
                 break;
 
-            case raiseFloorTurbo:
-                floor->direction = UP;
+            case RaiseFloorTurbo:
+                floor->direction = 1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED * 4;
                 floor->floordestheight = P_FindNextHighestFloor(sec, sec->floorheight);
-
                 break;
 
-            case raiseFloorToNearest:
-                floor->direction = UP;
+            case RaiseFloorToNearest:
+                floor->direction = 1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = P_FindNextHighestFloor(sec, sec->floorheight);
-
                 break;
 
-            case raiseFloor24:
-                floor->direction = UP;
+            case RaiseFloor24:
+                floor->direction = 1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = sec->floorheight + 24 * FRACUNIT;
-
                 break;
 
-            case raiseFloor32Turbo:
-                floor->direction = UP;
+            case RaiseFloor32Turbo:
+                floor->direction = 1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED * 4;
                 floor->floordestheight = sec->floorheight + 32 * FRACUNIT;
-
                 break;
 
-            case raiseFloor512:
-                floor->direction = UP;
+            case RaiseFloor512:
+                floor->direction = 1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = sec->floorheight + 512 * FRACUNIT;
-
                 break;
 
-            case raiseFloor24AndChange:
-                floor->direction = UP;
+            case RaiseFloor24AndChange:
+                floor->direction = 1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = sec->floorheight + 24 * FRACUNIT;
@@ -512,23 +491,23 @@ manual_floor:
 
                 break;
 
-            case raiseToTexture:
+            case RaiseToTexture:
             {
                 int minsize = 32000 << FRACBITS;
 
-                floor->direction = UP;
+                floor->direction = 1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
 
                 for (int i = 0; i < sec->linecount; i++)
-                    if (twoSided(secnum, i))
+                    if (P_TwoSided(secnum, i))
                     {
-                        side_t  *side = getSide(secnum, i, 0);
+                        side_t  *side = P_GetSide(secnum, i, 0);
 
                         if (textureheight[side->bottomtexture] < minsize)
                             minsize = textureheight[side->bottomtexture];
 
-                        side = getSide(secnum, i, 1);
+                        side = P_GetSide(secnum, i, 1);
 
                         if (textureheight[side->bottomtexture] < minsize)
                             minsize = textureheight[side->bottomtexture];
@@ -538,8 +517,8 @@ manual_floor:
                 break;
             }
 
-            case lowerAndChange:
-                floor->direction = DOWN;
+            case LowerAndChange:
+                floor->direction = -1;
                 floor->sector = sec;
                 floor->speed = FLOORSPEED;
                 floor->floordestheight = P_FindLowestFloorSurrounding(sec);
@@ -547,25 +526,23 @@ manual_floor:
                 floor->newspecial = sec->special;
 
                 for (int i = 0; i < sec->linecount; i++)
-                    if (twoSided(secnum, i))
+                    if (P_TwoSided(secnum, i))
                     {
-                        if (getSide(secnum, i, 0)->sector->id == secnum)
+                        if (P_GetSide(secnum, i, 0)->sector->id == secnum)
                         {
-                            if ((sec = getSector(secnum, i, 1))->floorheight == floor->floordestheight)
+                            if ((sec = P_GetSector(secnum, i, 1))->floorheight == floor->floordestheight)
                             {
                                 floor->texture = sec->floorpic;
                                 floor->newspecial = sec->special;
-
                                 break;
                             }
                         }
                         else
                         {
-                            if ((sec = getSector(secnum, i, 0))->floorheight == floor->floordestheight)
+                            if ((sec = P_GetSector(secnum, i, 0))->floorheight == floor->floordestheight)
                             {
                                 floor->texture = sec->floorpic;
                                 floor->newspecial = sec->special;
-
                                 break;
                             }
                         }
@@ -582,47 +559,43 @@ manual_floor:
             sec->lines[i]->flags &= ~ML_SECRET;
 
         if (zerotag_manual)
-            return rtn; //e6y
+            return rtn; // e6y
     }
 
     return rtn;
 }
 
+//
+// P_CheckTerrainType()
+//
 void P_CheckTerrainType(sector_t *sector)
 {
-    terraintype_t   oldterraintype = sector->terraintype;
+    const terraintype_t oldterraintype = sector->terraintype;
 
     if ((sector->terraintype = terraintypes[sector->floorpic]) != oldterraintype)
     {
-        dboolean    isliquid = (sector->terraintype != SOLID);
-
-        if (isliquid)
+        if (sector->terraintype >= LIQUID)
         {
-            bloodsplat_t    *splat = sector->splatlist;
-
-            while (splat)
-            {
-                bloodsplat_t    *next = splat->next;
-
+            for (bloodsplat_t *splat = sector->splatlist; splat; splat = splat->next)
                 P_UnsetBloodSplatPosition(splat);
-                r_bloodsplats_total--;
-                splat = next;
+
+            for (msecnode_t *node = sector->touching_thinglist; node; node = node->m_snext)
+            {
+                mobj_t  *thing = node->m_thing;
+
+                if (!(thing->flags & MF_SPAWNCEILING) && (thing->flags2 & MF2_FOOTCLIP))
+                    thing->flags2 |= MF2_FEETARECLIPPED;
+                else
+                    thing->flags2 &= ~MF2_FEETARECLIPPED;
             }
         }
         else
         {
             sector->floorxoffset = 0;
             sector->flooryoffset = 0;
-        }
 
-        for (msecnode_t *node = sector->touching_thinglist; node; node = node->m_snext)
-        {
-            mobj_t  *thing = node->m_thing;
-
-            if (isliquid && !(thing->flags & MF_SPAWNCEILING) && (thing->flags2 & MF2_FOOTCLIP))
-                thing->flags2 |= MF2_FEETARECLIPPED;
-            else
-                thing->flags2 &= ~MF2_FEETARECLIPPED;
+            for (msecnode_t *node = sector->touching_thinglist; node; node = node->m_snext)
+                node->m_thing->flags2 &= ~MF2_FEETARECLIPPED;
         }
     }
 }
@@ -636,12 +609,12 @@ void P_CheckTerrainType(sector_t *sector)
 // The linedef causing the change and the type of change is passed
 // Returns true if any sector changes
 //
-// jff 3/15/98 added to better support generalized sector types
+// jff 03/15/98 added to better support generalized sector types
 //
-dboolean EV_DoChange(line_t *line, change_e changetype)
+bool EV_DoChange(const line_t *line, const change_e changetype)
 {
-    int         secnum = -1;
-    dboolean    rtn = false;
+    int     secnum = -1;
+    bool    rtn = false;
 
     // change all sectors with the same tag as the linedef
     while ((secnum = P_FindSectorFromLineTag(line, secnum)) >= 0)
@@ -654,14 +627,13 @@ dboolean EV_DoChange(line_t *line, change_e changetype)
         // handle trigger or numeric change type
         switch (changetype)
         {
-            case trigChangeOnly:
+            case TrigChangeOnly:
                 sec->floorpic = line->frontsector->floorpic;
                 P_CheckTerrainType(sec);
                 sec->special = line->frontsector->special;
-
                 break;
 
-            case numChangeOnly:
+            case NumChangeOnly:
                 if ((secm = P_FindModelFloorSector(sec->floorheight, secnum)))  // if no model, no change
                 {
                     sec->floorpic = secm->floorpic;
@@ -680,7 +652,7 @@ dboolean EV_DoChange(line_t *line, change_e changetype)
 // BUILD A STAIRCASE!
 //
 
-// cph 2001/09/21 - compatibility nightmares again
+// cph 09/21/01: compatibility nightmares again
 // There are three different ways this function has, during its history, stepped
 // through all the stairs to be triggered by the single switch
 // - original DOOM used a linear P_FindSectorFromLineTag, but failed to preserve
@@ -693,7 +665,7 @@ dboolean EV_DoChange(line_t *line, change_e changetype)
 // stairs
 // - BOOM fixed the bug, and MBF/PrBoom without comp_stairs work right
 //
-static int P_FindSectorFromLineTagWithLowerBound(line_t *l, int start, int min)
+static int P_FindSectorFromLineTagWithLowerBound(const line_t *l, int start, const int min)
 {
     do
     {
@@ -703,11 +675,11 @@ static int P_FindSectorFromLineTagWithLowerBound(line_t *l, int start, int min)
     return start;
 }
 
-dboolean EV_BuildStairs(line_t *line, fixed_t speed, fixed_t stairsize, dboolean crushing)
+bool EV_BuildStairs(const line_t *line, const fixed_t speed, const fixed_t stairsize, const bool crushing)
 {
     int         ssec = -1;
     int         minssec = -1;
-    dboolean    rtn = false;
+    bool        rtn = false;
     int         secnum = -1;
     sector_t    *sec;
 
@@ -722,7 +694,7 @@ dboolean EV_BuildStairs(line_t *line, fixed_t speed, fixed_t stairsize, dboolean
     while ((ssec = P_FindSectorFromLineTagWithLowerBound(line, ssec, minssec)) >= 0)
     {
         floormove_t *floor;
-        dboolean    okay;
+        bool        okay;
         int         height;
         int         texture;
 
@@ -730,7 +702,7 @@ dboolean EV_BuildStairs(line_t *line, fixed_t speed, fixed_t stairsize, dboolean
         sec = sectors + secnum;
 
 manual_stair:
-        // ALREADY MOVING? IF SO, KEEP GOING...
+        // already moving? if so, keep going...
         if (P_SectorActive(floor_special, sec))
             continue;
 
@@ -742,7 +714,7 @@ manual_stair:
         P_AddThinker(&floor->thinker);
 
         sec->floordata = floor;
-        floor->direction = UP;
+        floor->direction = 1;
         floor->sector = sec;
 
         floor->speed = speed;
@@ -765,23 +737,21 @@ manual_stair:
                 line_t      *li = sec->lines[i];
                 sector_t    *tsec;
 
-                if (!(li->flags & ML_TWOSIDED))
+                if (!(li->flags & ML_TWOSIDED)
+                    || secnum != li->frontsector->id
+                    || !(tsec = li->backsector)
+                    || tsec->floorpic != texture)
                     continue;
 
-                if (secnum != li->frontsector->id)
-                    continue;
-
-                if (!(tsec = li->backsector))
-                    continue;
-
-                if (tsec->floorpic != texture)
-                    continue;
-
-                height += stairsize;
+                if (compat_stairs)
+                    height += stairsize;
 
                 if (P_SectorActive(floor_special, tsec))
                     continue;
 
+                if (!compat_stairs)
+                    height += stairsize;
+
                 sec = tsec;
                 secnum = tsec->id;
                 floor = Z_Calloc(1, sizeof(*floor), PU_LEVSPEC, NULL);
@@ -790,17 +760,22 @@ manual_stair:
                 P_AddThinker(&floor->thinker);
 
                 sec->floordata = floor;
-                floor->direction = UP;
+                floor->direction = 1;
                 floor->sector = sec;
                 floor->speed = speed;
                 floor->floordestheight = height;
                 floor->crush = crushing;
                 floor->stopsound = (sec->floorheight != height);
                 okay = true;
-
                 break;
             }
         } while (okay);
+
+        if (compat_stairs)
+        {
+            ssec = -1;
+            minssec = secnum;
+        }
     }
 
     return rtn;
@@ -815,10 +790,10 @@ manual_stair:
 //
 // jff 02/22/98 new type to move floor and ceiling in parallel
 //
-dboolean EV_DoElevator(line_t *line, elevator_e elevtype)
+bool EV_DoElevator(const line_t *line, const elevator_e elevtype)
 {
-    int         secnum = -1;
-    dboolean    rtn = false;
+    int     secnum = -1;
+    bool    rtn = false;
 
     // act on all sectors with the same tag as the triggering linedef
     while ((secnum = P_FindSectorFromLineTag(line, secnum)) >= 0)
@@ -826,52 +801,46 @@ dboolean EV_DoElevator(line_t *line, elevator_e elevtype)
         sector_t    *sec = sectors + secnum;
         elevator_t  *elevator;
 
-        // If either floor or ceiling is already activated, skip it
+        // if either floor or ceiling is already activated, skip it
         if (sec->floordata || sec->ceilingdata)
             continue;
 
         // create and initialize new elevator thinker
-        rtn = true;
-        elevator = Z_Calloc(1, sizeof(*elevator), PU_LEVSPEC, NULL);
+        if (!(elevator = Z_Calloc(1, sizeof(*elevator), PU_LEVSPEC, NULL)))
+            return false;
 
+        rtn = true;
         elevator->thinker.function = &T_MoveElevator;
         P_AddThinker(&elevator->thinker);
 
         sec->floordata = elevator;
         sec->ceilingdata = elevator;
         elevator->type = elevtype;
+        elevator->sector = sec;
+        elevator->speed = ELEVATORSPEED;
 
         // set up the fields according to the type of elevator action
         switch (elevtype)
         {
             // elevator down to next floor
-            case elevateDown:
-                elevator->direction = DOWN;
-                elevator->sector = sec;
-                elevator->speed = ELEVATORSPEED;
+            case ElevateDown:
+                elevator->direction = -1;
                 elevator->floordestheight = P_FindNextLowestFloor(sec, sec->floorheight);
                 elevator->ceilingdestheight = elevator->floordestheight + sec->ceilingheight - sec->floorheight;
-
                 break;
 
             // elevator up to next floor
-            case elevateUp:
-                elevator->direction = UP;
-                elevator->sector = sec;
-                elevator->speed = ELEVATORSPEED;
+            case ElevateUp:
+                elevator->direction = 1;
                 elevator->floordestheight = P_FindNextHighestFloor(sec, sec->floorheight);
                 elevator->ceilingdestheight = elevator->floordestheight + sec->ceilingheight - sec->floorheight;
-
                 break;
 
             // elevator to floor height of activating switch's front sector
-            case elevateCurrent:
-                elevator->sector = sec;
-                elevator->speed = ELEVATORSPEED;
+            case ElevateCurrent:
                 elevator->floordestheight = line->frontsector->floorheight;
                 elevator->ceilingdestheight = elevator->floordestheight + sec->ceilingheight - sec->floorheight;
-                elevator->direction = (elevator->floordestheight > sec->floorheight ? UP : DOWN);
-
+                elevator->direction = (elevator->floordestheight > sec->floorheight ? 1 : -1);
                 break;
         }
 
