@@ -1,39 +1,36 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
 #include "i_system.h"
@@ -41,8 +38,6 @@
 #include "p_local.h"
 #include "p_setup.h"
 
-void P_CreateSecNodeList(mobj_t *thing, fixed_t x, fixed_t y);
-
 //
 // P_ApproxDistance
 // Gives an estimation of distance (not exact)
@@ -52,18 +47,18 @@ fixed_t P_ApproxDistance(fixed_t dx, fixed_t dy)
     dx = ABS(dx);
     dy = ABS(dy);
 
-    return (dx + dy - (MIN(dx, dy) >> 1));
+    return (dx + dy - MIN(dx, dy) / 2);
 }
 
 //
 // P_PointOnLineSide
 // Returns 0 or 1
 //
-int P_PointOnLineSide(fixed_t x, fixed_t y, line_t *line)
+int P_PointOnLineSide(const fixed_t x, const fixed_t y, const line_t *line)
 {
     return (!line->dx ? (x <= line->v1->x ? line->dy > 0 : line->dy < 0) :
         (!line->dy ? (y <= line->v1->y ? line->dx < 0 : line->dx > 0) :
-        ((int64_t)y - line->v1->y) * line->dx >= line->dy * ((int64_t)x - line->v1->x)));
+        FixedMul(y - line->v1->y, line->dx >> FRACBITS) >= FixedMul(line->dy >> FRACBITS, x - line->v1->x)));
 }
 
 //
@@ -72,7 +67,7 @@ int P_PointOnLineSide(fixed_t x, fixed_t y, line_t *line)
 // Returns side 0 or 1, -1 if box crosses the line.
 // killough 05/03/98: reformatted, cleaned up
 //
-int P_BoxOnLineSide(fixed_t *tmbox, line_t *ld)
+int P_BoxOnLineSide(const fixed_t *tmbox, const line_t *ld)
 {
     switch (ld->slopetype)
     {
@@ -99,9 +94,9 @@ int P_BoxOnLineSide(fixed_t *tmbox, line_t *ld)
 
 //
 // P_PointOnDivlineSide
-// Returns 0 or 1.
+// Returns 0 or 1
 //
-static int P_PointOnDivlineSide(fixed_t x, fixed_t y, divline_t *line)
+static int P_PointOnDivlineSide(fixed_t x, fixed_t y, const divline_t *line)
 {
     return (!line->dx ? (x <= line->x ? line->dy > 0 : line->dy < 0) :
         (!line->dy ? (y <= line->y ? line->dx < 0 : line->dx > 0) :
@@ -111,14 +106,12 @@ static int P_PointOnDivlineSide(fixed_t x, fixed_t y, divline_t *line)
 
 //
 // P_InterceptVector
-// Returns the fractional intercept point
-// along the first divline.
-// This is only called by the addthings
-// and addlines traversers.
+// Returns the fractional intercept point along the first divline.
+// This is only called by the addthings and addlines traversers.
 //
-fixed_t P_InterceptVector(divline_t *v2, divline_t *v1)
+fixed_t P_InterceptVector(const divline_t *v2, const divline_t *v1)
 {
-    int64_t den = ((int64_t)v1->dy * v2->dx - (int64_t)v1->dx * v2->dy) >> FRACBITS;
+    const int64_t   den = ((int64_t)v1->dy * v2->dx - (int64_t)v1->dx * v2->dy) >> FRACBITS;
 
     if (!den)
         return 0;
@@ -128,8 +121,7 @@ fixed_t P_InterceptVector(divline_t *v2, divline_t *v1)
 
 //
 // P_LineOpening
-// Sets opentop and openbottom to the window
-// through a two sided line.
+// Sets opentop and openbottom to the window through a two sided line.
 // OPTIMIZE: keep this precalculated
 //
 fixed_t opentop;
@@ -137,7 +129,7 @@ fixed_t openbottom;
 fixed_t openrange;
 fixed_t lowfloor;
 
-void P_LineOpening(line_t *line)
+void P_LineOpening(const line_t *line)
 {
     sector_t    *front;
     sector_t    *back;
@@ -177,10 +169,8 @@ void P_LineOpening(line_t *line)
 
 //
 // P_UnsetThingPosition
-// Unlinks a thing from block map and sectors.
-// On each position change, BLOCKMAP and other
-// lookups maintaining lists of things inside
-// these structures need to be updated.
+// Unlinks a thing from blockmap and sectors. On each position change, BLOCKMAP and other
+// lookups maintaining lists of things inside these structures need to be updated.
 //
 void P_UnsetThingPosition(mobj_t *thing)
 {
@@ -194,7 +184,7 @@ void P_UnsetThingPosition(mobj_t *thing)
         mobj_t  **sprev = thing->sprev;
         mobj_t  *snext = thing->snext;
 
-        if ((*sprev = snext))                           // unlink from sector list
+        if ((*sprev = snext))               // unlink from sector list
             snext->sprev = sprev;
 
         // phares 03/14/98
@@ -210,7 +200,7 @@ void P_UnsetThingPosition(mobj_t *thing)
         // If this Thing is being removed entirely, then the calling
         // routine will clear out the nodes in sector_list.
         sector_list = thing->touching_sectorlist;
-        thing->touching_sectorlist = NULL;              // to be restored by P_SetThingPosition
+        thing->touching_sectorlist = NULL;  // to be restored by P_SetThingPosition
     }
 
     if (!(thing->flags & MF_NOBLOCKMAP))
@@ -227,7 +217,7 @@ void P_UnsetThingPosition(mobj_t *thing)
         mobj_t  *bnext;
         mobj_t  **bprev = thing->bprev;
 
-        if (bprev && (*bprev = bnext = thing->bnext))   // unlink from block map
+        if (bprev && (*bprev = bnext = thing->bnext))   // unlink from blockmap
             bnext->bprev = bprev;
     }
 }
@@ -243,13 +233,12 @@ void P_UnsetBloodSplatPosition(bloodsplat_t *splat)
     if ((*prev = next))
         next->prev = prev;
 
-    free(splat);
+    r_bloodsplats_total--;
 }
 
 //
 // P_SetThingPosition
-// Links a thing into both a block and a subsector
-// based on its x y.
+// Links a thing into both a block and a subsector based on its x y.
 // Sets thing->subsector properly
 //
 void P_SetThingPosition(mobj_t *thing)
@@ -285,16 +274,16 @@ void P_SetThingPosition(mobj_t *thing)
         // at sector_t->touching_thinglist) are broken. When a node is
         // added, new sector links are created.
         P_CreateSecNodeList(thing, thing->x, thing->y);
-        thing->touching_sectorlist = sector_list;       // Attach to Thing's mobj_t
-        sector_list = NULL;                             // clear for next time
+        thing->touching_sectorlist = sector_list;   // attach to thing's mobj_t
+        sector_list = NULL;                         // clear for next time
     }
 
     // link into blockmap
     if (!(thing->flags & MF_NOBLOCKMAP))
     {
         // inert things don't need to be in blockmap
-        int blockx = P_GetSafeBlockX(thing->x - bmaporgx);
-        int blocky = P_GetSafeBlockY(thing->y - bmaporgy);
+        const int   blockx = P_GetSafeBlockX(thing->x - bmaporgx);
+        const int   blocky = P_GetSafeBlockY(thing->y - bmaporgy);
 
         if (blockx >= 0 && blockx < bmapwidth && blocky >= 0 && blocky < bmapheight)
         {
@@ -334,22 +323,17 @@ void P_SetBloodSplatPosition(bloodsplat_t *splat)
 }
 
 //
-// BLOCK MAP ITERATORS
-// For each line/thing in the given mapblock,
-// call the passed PIT_* function.
-// If the function returns false,
-// exit with false without checking anything else.
+// BLOCKMAP ITERATORS
+// For each line/thing in the given mapblock, call the passed PIT_* function.
+// If the function returns false, exit with false without checking anything else.
 //
 
 //
 // P_BlockLinesIterator
-// The validcount flags are used to avoid checking lines
-// that are marked in multiple mapblocks,
-// so increment validcount before the first call
-// to P_BlockLinesIterator, then make one or more calls
-// to it.
+// The validcount flags are used to avoid checking lines that are marked in multiple mapblocks, so
+// increment validcount before the first call to P_BlockLinesIterator, then make one or more calls to it.
 //
-dboolean P_BlockLinesIterator(int x, int y, dboolean func(line_t *))
+bool P_BlockLinesIterator(const int x, const int y, bool func(line_t *))
 {
     if (x < 0 || y < 0 || x >= bmapwidth || y >= bmapheight)
         return true;
@@ -365,7 +349,7 @@ dboolean P_BlockLinesIterator(int x, int y, dboolean func(line_t *))
             line_t  *ld = lines + *list;
 
             if (ld->validcount == validcount)
-                continue;       // line has already been checked
+                continue;   // line has already been checked
 
             ld->validcount = validcount;
 
@@ -373,14 +357,14 @@ dboolean P_BlockLinesIterator(int x, int y, dboolean func(line_t *))
                 return false;
         }
 
-        return true;            // everything was checked
+        return true;        // everything was checked
     }
 }
 
 //
 // P_BlockThingsIterator
 //
-dboolean P_BlockThingsIterator(int x, int y, dboolean func(mobj_t *))
+bool P_BlockThingsIterator(const int x, const int y, bool func(mobj_t *))
 {
     if (x < 0 || y < 0 || x >= bmapwidth || y >= bmapheight)
         return true;
@@ -465,12 +449,12 @@ static intercept_t  *intercept_p;
 void P_CheckIntercepts(void)
 {
     static size_t   num_intercepts;
-    size_t          offset = intercept_p - intercepts;
+    const size_t    offset = intercept_p - intercepts;
 
     if (offset >= num_intercepts)
     {
         num_intercepts = (num_intercepts ? num_intercepts * 2 : 128);
-        intercepts = I_Realloc(intercepts, sizeof(*intercepts) * num_intercepts);
+        intercepts = I_Realloc(intercepts, num_intercepts * sizeof(*intercepts));
         intercept_p = intercepts + offset;
     }
 }
@@ -479,21 +463,19 @@ divline_t   dltrace;
 
 //
 // PIT_AddLineIntercepts
-// Looks for lines in the given block
-// that intercept the given trace
-// to add to the intercepts list.
+// Looks for lines in the given block that intercept the given trace to add to the intercepts list.
+// A line is crossed if its endpoints are on opposite sides of the trace.
 //
-// A line is crossed if its endpoints
-// are on opposite sides of the trace.
-//
-static dboolean PIT_AddLineIntercepts(line_t *ld)
+static bool PIT_AddLineIntercepts(line_t *ld)
 {
-    int         s1, s2;
+    int         s1;
+    int         s2;
     fixed_t     frac;
-    divline_t   dl;
+    divline_t   dl = { 0 };
 
     // avoid precision problems with two routines
-    if (dltrace.dx > 16 * FRACUNIT || dltrace.dy > 16 * FRACUNIT || dltrace.dx < -16 * FRACUNIT || dltrace.dy < -16 * FRACUNIT)
+    if (dltrace.dx < -16 * FRACUNIT || dltrace.dx > 16 * FRACUNIT
+        || dltrace.dy < -16 * FRACUNIT || dltrace.dy > 16 * FRACUNIT)
     {
         s1 = P_PointOnDivlineSide(ld->v1->x, ld->v1->y, &dltrace);
         s2 = P_PointOnDivlineSide(ld->v2->x, ld->v2->y, &dltrace);
@@ -519,8 +501,8 @@ static dboolean PIT_AddLineIntercepts(line_t *ld)
     P_CheckIntercepts();    // killough
 
     intercept_p->frac = frac;
-    intercept_p->isaline = true;
-    intercept_p->d.line = ld;
+    intercept_p->thing = NULL;
+    intercept_p->line = ld;
     intercept_p++;
 
     return true;            // continue
@@ -529,13 +511,20 @@ static dboolean PIT_AddLineIntercepts(line_t *ld)
 //
 // PIT_AddThingIntercepts
 //
-static dboolean PIT_AddThingIntercepts(mobj_t *thing)
+static bool PIT_AddThingIntercepts(mobj_t *thing)
 {
     int         numfronts = 0;
-    divline_t   dl;
-    fixed_t     radius = thing->radius;
-    fixed_t     x = thing->x;
-    fixed_t     y = thing->y;
+    divline_t   dl = { 0 };
+    fixed_t     radius;
+    fixed_t     x, y;
+
+    // [BH] If the thing isn't shootable, exit early
+    if (!(thing->flags & MF_SHOOTABLE))
+        return true;
+
+    radius = thing->radius;
+    x = thing->x;
+    y = thing->y;
 
     // [RH] Don't check a corner to corner crosssection for hit.
     // Instead, check against the actual bounding box.
@@ -546,36 +535,32 @@ static dboolean PIT_AddThingIntercepts(mobj_t *thing)
     {
         switch (i)
         {
-            case 0:     // Top edge
+            case 0: // Top edge
                 dl.x = x + radius;
                 dl.y = y + radius;
                 dl.dx = -radius * 2;
                 dl.dy = 0;
-
                 break;
 
-            case 1:     // Right edge
+            case 1: // Right edge
                 dl.x = x + radius;
                 dl.y = y - radius;
                 dl.dx = 0;
                 dl.dy = radius * 2;
-
                 break;
 
-            case 2:     // Bottom edge
+            case 2: // Bottom edge
                 dl.x = x - radius;
                 dl.y = y - radius;
                 dl.dx = radius * 2;
                 dl.dy = 0;
-
                 break;
 
-            case 3:     // Left edge
+            case 3: // Left edge
                 dl.x = x - radius;
                 dl.y = y + radius;
                 dl.dx = 0;
                 dl.dy = radius * -2;
-
                 break;
         }
 
@@ -588,7 +573,7 @@ static dboolean PIT_AddThingIntercepts(mobj_t *thing)
             if (P_PointOnDivlineSide(dl.x, dl.y, &dltrace) != P_PointOnDivlineSide(dl.x + dl.dx, dl.y + dl.dy, &dltrace))
             {
                 // It's a hit
-                fixed_t frac = P_InterceptVector(&dltrace, &dl);
+                const fixed_t   frac = P_InterceptVector(&dltrace, &dl);
 
                 // behind source
                 if (frac < 0)
@@ -597,24 +582,23 @@ static dboolean PIT_AddThingIntercepts(mobj_t *thing)
                 P_CheckIntercepts();    // killough
 
                 intercept_p->frac = frac;
-                intercept_p->isaline = false;
-                intercept_p->d.thing = thing;
+                intercept_p->line = NULL;
+                intercept_p->thing = thing;
                 intercept_p++;
-
                 continue;
             }
         }
     }
 
-    // If none of the sides was facing the trace, then the trace
+    // If none of the sides were facing the trace, then the trace
     // must have started inside the box, so add it as an intercept.
     if (!numfronts)
     {
         P_CheckIntercepts();    // killough
 
         intercept_p->frac = 0;
-        intercept_p->isaline = false;
-        intercept_p->d.thing = thing;
+        intercept_p->line = NULL;
+        intercept_p->thing = thing;
         intercept_p++;
     }
 
@@ -623,10 +607,9 @@ static dboolean PIT_AddThingIntercepts(mobj_t *thing)
 
 //
 // P_TraverseIntercepts
-// Returns true if the traverser function returns true
-// for all lines.
+// Returns true if the traverser function returns true for all lines.
 //
-static dboolean P_TraverseIntercepts(traverser_t func, fixed_t maxfrac)
+static bool P_TraverseIntercepts(traverser_t func, const fixed_t maxfrac)
 {
     size_t      count = intercept_p - intercepts;
     intercept_t *in = NULL;
@@ -643,31 +626,32 @@ static dboolean P_TraverseIntercepts(traverser_t func, fixed_t maxfrac)
             }
 
         if (dist > maxfrac)
-            return true;        // checked everything in range
+            return true;    // checked everything in range
 
-        if (!func(in))
-            return false;       // don't bother going farther
+        if (!in || !func(in))
+            return false;   // don't bother going farther
 
         in->frac = FIXED_MAX;
     }
 
-    return true;                // everything was traversed
+    return true;            // everything was traversed
 }
 
 //
 // P_PathTraverse
-// Traces a line from (x1,y1) to (x2,y2),
-// calling the traverser function for each.
-// Returns true if the traverser function returns true
-// for all lines.
+// Traces a line from (x1,y1) to (x2,y2), calling the traverser function for each.
+// Returns true if the traverser function returns true for all lines.
 //
-dboolean P_PathTraverse(fixed_t x1, fixed_t y1, fixed_t x2, fixed_t y2, int flags, traverser_t trav)
+#define MAX_SIGHT_COUNT 64
+
+bool P_PathTraverse(fixed_t x1, fixed_t y1, fixed_t x2, fixed_t y2, const int flags, traverser_t trav)
 {
     fixed_t xt1, yt1;
     fixed_t xt2, yt2;
     int64_t _x1, _y1;
     int64_t _x2, _y2;
-    fixed_t xstep = 256 * FRACUNIT, ystep = 256 * FRACUNIT;
+    fixed_t xstep = 256 * FRACUNIT;
+    fixed_t ystep = 256 * FRACUNIT;
     fixed_t partial = FRACUNIT;
     fixed_t xintercept, yintercept;
     int     mapx, mapy;
@@ -715,7 +699,7 @@ dboolean P_PathTraverse(fixed_t x1, fixed_t y1, fixed_t x2, fixed_t y2, int flag
     else if (xt2 < xt1)
     {
         mapxstep = -1;
-        partial = mapx1 & (FRACUNIT - 1);
+        partial = (mapx1 & (FRACUNIT - 1));
         ystep = FixedDiv(y2 - y1, ABS(x2 - x1));
     }
 
@@ -730,19 +714,18 @@ dboolean P_PathTraverse(fixed_t x1, fixed_t y1, fixed_t x2, fixed_t y2, int flag
     else if (yt2 < yt1)
     {
         mapystep = -1;
-        partial = mapy1 & (FRACUNIT - 1);
+        partial = (mapy1 & (FRACUNIT - 1));
         xstep = FixedDiv(x2 - x1, ABS(y2 - y1));
     }
 
     xintercept = mapx1 + FixedMul(partial, xstep);
 
     // Step through map blocks.
-    // Count is present to prevent a round off error
-    // from skipping the break.
+    // Count is present to prevent a round off error from skipping the break.
     mapx = xt1;
     mapy = yt1;
 
-    for (int count = 0; count < 1000; count++)
+    for (int count = 0; count < MAX_SIGHT_COUNT; count++)
     {
         if (flags & PT_ADDLINES)
             if (!P_BlockLinesIterator(mapx, mapy, &PIT_AddLineIntercepts))
@@ -760,20 +743,19 @@ dboolean P_PathTraverse(fixed_t x1, fixed_t y1, fixed_t x2, fixed_t y2, int flag
         {
             case 0:
                 // neither xintercept nor yintercept match!
-                return false;
+                count = MAX_SIGHT_COUNT;
+                break;
 
             case 1:
                 // xintercept matches
                 xintercept += xstep;
                 mapy += mapystep;
-
                 break;
 
             case 2:
                 // yintercept matches
                 yintercept += ystep;
                 mapx += mapxstep;
-
                 break;
 
             case 3:
@@ -798,7 +780,6 @@ dboolean P_PathTraverse(fixed_t x1, fixed_t y1, fixed_t x2, fixed_t y2, int flag
                 yintercept += ystep;
                 mapx += mapxstep;
                 mapy += mapystep;
-
                 break;
         }
     }
@@ -828,3 +809,127 @@ int P_GetSafeBlockY(int coord)
 
     return coord;
 }
+
+//
+// MBF21: RoughBlockCheck
+// [XA] adapted from Hexen -- used by P_RoughTargetSearch
+//
+static mobj_t *RoughBlockCheck(mobj_t *mo, const int index, const angle_t fov)
+{
+    mobj_t  *link = blocklinks[index];
+
+    while (link)
+    {
+        // skip non-shootable actors
+        if (!(link->flags & MF_SHOOTABLE))
+        {
+            link = link->bnext;
+            continue;
+        }
+
+        // skip the projectile's owner
+        if (link == mo->target)
+        {
+            link = link->bnext;
+            continue;
+        }
+
+        // skip actors on the same "team", unless infighting
+        if (mo->target && !((link->flags ^ mo->target->flags) & MF_FRIEND)
+            && mo->target->target != link && !(link->player && mo->target->player))
+        {
+            link = link->bnext;
+            continue;
+        }
+
+        // skip actors outside of specified FOV
+        if (fov > 0 && !P_CheckFOV(mo, link, fov))
+        {
+            link = link->bnext;
+            continue;
+        }
+
+        // skip actors not in line of sight
+        if (!P_CheckSight(mo, link))
+        {
+            link = link->bnext;
+            continue;
+        }
+
+        // all good! return it.
+        return link;
+    }
+
+    // couldn't find a valid target
+    return NULL;
+}
+
+//
+// MBF21: P_RoughTargetSearch
+// Searches though the surrounding mapblocks for monsters/players
+// based on Hexen's P_RoughMonsterSearch
+//
+// distance is in MAPBLOCKUNITS
+mobj_t *P_RoughTargetSearch(mobj_t *mo, const angle_t fov, const int distance)
+{
+    const int   startx = (mo->x - bmaporgx) >> MAPBLOCKSHIFT;
+    const int   starty = (mo->y - bmaporgy) >> MAPBLOCKSHIFT;
+    mobj_t      *target;
+
+    if (startx >= 0 && startx < bmapwidth && starty >= 0 && starty < bmapheight
+        && (target = RoughBlockCheck(mo, starty * bmapwidth + startx, fov)))
+        return target;  // found a target right away
+
+    for (int count = 1; count <= distance; count++)
+    {
+        int blockx, blocky;
+        int blockindex;
+        int firststop = startx + count;
+        int secondstop;
+        int thirdstop;
+        int finalstop;
+
+        if (firststop < 0)
+            continue;
+
+        if ((secondstop = starty + count) < 0)
+            continue;
+
+        if (firststop >= bmapwidth)
+            firststop = bmapwidth - 1;
+
+        if (secondstop >= bmapheight)
+            secondstop = bmapheight - 1;
+
+        blockx = BETWEEN(0, startx - count, bmapwidth - 1);
+        blocky = BETWEEN(0, starty - count, bmapheight - 1);
+        blockindex = blocky * bmapwidth + blockx;
+
+        thirdstop = secondstop * bmapwidth + blockx;
+        secondstop = secondstop * bmapwidth + firststop;
+        firststop += blocky * bmapwidth;
+        finalstop = blockindex;
+
+        // Trace the first block section (along the top)
+        for (; blockindex <= firststop; blockindex++)
+            if ((target = RoughBlockCheck(mo, blockindex, fov)))
+                return target;
+
+        // Trace the second block section (right edge)
+        for (blockindex--; blockindex <= secondstop; blockindex += bmapwidth)
+            if ((target = RoughBlockCheck(mo, blockindex, fov)))
+                return target;
+
+        // Trace the third block section (bottom edge)
+        for (blockindex -= bmapwidth; blockindex >= thirdstop; blockindex--)
+            if ((target = RoughBlockCheck(mo, blockindex, fov)))
+                return target;
+
+        // Trace the final block section (left edge)
+        for (blockindex++; blockindex > finalstop; blockindex -= bmapwidth)
+            if ((target = RoughBlockCheck(mo, blockindex, fov)))
+                return target;
+    }
+
+    return NULL;
+}
