@@ -1,45 +1,44 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
+#include "c_cmds.h"
 #include "c_console.h"
 #include "doomstat.h"
 #include "g_game.h"
-#include "i_gamepad.h"
+#include "hu_stuff.h"
+#include "i_timer.h"
 #include "m_config.h"
 #include "m_menu.h"
 #include "p_inter.h"
@@ -54,16 +53,13 @@
 #define STEP1DISTANCE   24
 #define STEP2DISTANCE   32
 
-dboolean        autotilt = autotilt_default;
-dboolean        autouse = autouse_default;
-dboolean        infighting = infighting_default;
-int             movebob = movebob_default;
-dboolean        r_liquid_lowerview = r_liquid_lowerview_default;
-int             r_shake_damage = r_shake_damage_default;
-int             stillbob = stillbob_default;
+bool        autousing = false;
+int         deadlookdir = -1;
 
-dboolean        autousing = false;
-int             deadlookdir = -1;
+static int  ammodiffspeed[NUMAMMO] = { 0 };
+static int  maxammodiffspeed[NUMAMMO] = { 0 };
+static int  armordiffspeed = 0;
+static int  healthdiffspeed = 0;
 
 //
 // Movement
@@ -95,25 +91,6 @@ static void P_Bob(angle_t angle, fixed_t move)
     viewplayer->momy += FixedMul(move, finesine[angle]);
 }
 
-//
-// P_IsSelfReferencingSector
-//
-static dboolean P_IsSelfReferencingSector(sector_t *sec)
-{
-    const int   linecount = sec->linecount;
-    int         count = 0;
-
-    for (int i = 0; i < linecount; i++)
-    {
-        line_t  *line = sec->lines[i];
-
-        if (line->backsector && line->frontsector == line->backsector && !line->frontsector->tag)
-            count++;
-    }
-
-    return (count >= 2);
-}
-
 //
 // P_CalcHeight
 // Calculate the walking/running height adjustment
@@ -128,16 +105,16 @@ void P_CalcHeight(void)
     {
         // Regular movement bobbing
         // (needs to be calculated for gun swing even if not on ground)
-        fixed_t momx = viewplayer->momx;
-        fixed_t momy = viewplayer->momy;
-        fixed_t bob = (MAXBOB * stillbob / 400) / 2;
+        const fixed_t   momx = viewplayer->momx;
+        const fixed_t   momy = viewplayer->momy;
+        fixed_t         bob = (MAXBOB * stillbob / 400) / 2;
 
         if (momx | momy)
             bob = MAX(MIN((FixedMul(momx, momx) + FixedMul(momy, momy)) >> 2, MAXBOB) * movebob / 200, bob);
 
         if (viewplayer->bouncemax)
         {
-            viewplayer->bounce -= FRACUNIT;
+            viewplayer->bounce -= FRACUNIT * 3 / 2;
 
             if (viewplayer->bounce < viewplayer->bouncemax)
                 viewplayer->bounce = -viewplayer->bounce;
@@ -171,22 +148,22 @@ void P_CalcHeight(void)
                 viewplayer->deltaviewheight = 1;
         }
 
-        viewplayer->viewz += FixedMul(bob, finesine[(FINEANGLES / 20 * leveltime) & FINEMASK]);
+        viewplayer->viewz += FixedMul(bob, finesine[((FINEANGLES / 20 * maptime) & FINEMASK)]);
     }
 
     if (mo->flags2 & MF2_FEETARECLIPPED)
     {
         if (viewplayer->playerstate == PST_DEAD)
         {
-            if (r_liquid_bob)
+            if (r_liquid_bobsprites)
                 viewplayer->viewz += animatedliquiddiff;
         }
         else if (r_liquid_lowerview)
         {
             sector_t    *sector = mo->subsector->sector;
 
-            if (!P_IsSelfReferencingSector(sector)
-                && (!sector->heightsec || mo->z + viewplayer->viewheight - FOOTCLIPSIZE >= sector->heightsec->floorheight))
+            if (!sector->heightsec
+                || mo->z + viewplayer->viewheight - FOOTCLIPSIZE >= sector->heightsec->floorheight)
                 viewplayer->viewz -= FOOTCLIPSIZE;
         }
     }
@@ -197,15 +174,15 @@ void P_CalcHeight(void)
 //
 // P_CheckForSteps
 //
-static dboolean P_CheckForSteps(fixed_t width)
+static bool P_CheckForSteps(const fixed_t width)
 {
-    sector_t    *sector1 = R_PointInSubsector(viewx + width * viewcos, viewy + width * viewsin)->sector;
-    sector_t    *sector2 = R_PointInSubsector(viewx + width * 2 * viewcos, viewy + width * 2 * viewsin)->sector;
+    const sector_t  *sector1 = R_PointInSubsector(viewx + width * viewcos, viewy + width * viewsin)->sector;
+    const sector_t  *sector2 = R_PointInSubsector(viewx + width * 2 * viewcos, viewy + width * 2 * viewsin)->sector;
 
     if (sector1->terraintype == sector2->terraintype)
     {
-        fixed_t step = sector1->floorheight;
-        int     delta = step - viewplayer->mo->floorz;
+        const fixed_t   step = sector1->floorheight;
+        const int       delta = step - viewplayer->mo->floorz;
 
         if (delta == sector2->floorheight - step)
         {
@@ -230,12 +207,12 @@ static dboolean P_CheckForSteps(fixed_t width)
 //
 void P_MovePlayer(void)
 {
-    mobj_t      *mo = viewplayer->mo;
-    ticcmd_t    *cmd = &viewplayer->cmd;
-    signed char forward = cmd->forwardmove;
-    signed char side = cmd->sidemove;
+    mobj_t              *mo = viewplayer->mo;
+    const ticcmd_t      *cmd = &viewplayer->cmd;
+    const signed char   forward = cmd->forwardmove;
+    const signed char   side = cmd->sidemove;
 
-    mo->angle += (cmd->angleturn * turbo / 100) << FRACBITS;
+    mo->angle += ((cmd->angleturn * turbo / 100) << FRACBITS);
 
     // killough 10/98:
     //
@@ -245,14 +222,14 @@ void P_MovePlayer(void)
     // thrust applied to the movement varies with 'movefactor'.
     if ((forward | side) && (mo->z <= mo->floorz || (mo->flags2 & MF2_ONMOBJ)))
     {
-        int     friction;
-        int     movefactor = P_GetMoveFactor(mo, &friction);
-        angle_t angle = mo->angle;
+        int         friction;
+        const int   movefactor = P_GetMoveFactor(mo, &friction);
+        angle_t     angle = mo->angle;
 
         // killough 11/98:
         // On sludge, make bobbing depend on efficiency.
         // On ice, make it depend on effort.
-        int     bobfactor = (friction < ORIG_FRICTION ? movefactor : ORIG_FRICTION_FACTOR);
+        const int   bobfactor = (friction < ORIG_FRICTION ? movefactor : ORIG_FRICTION_FACTOR);
 
         if (forward)
         {
@@ -265,9 +242,12 @@ void P_MovePlayer(void)
             P_Bob((angle -= ANG90), side * bobfactor);
             P_Thrust(angle, side * movefactor);
         }
+
+        if (mo->state == &states[S_PLAY])
+            P_SetMobjState(mo, S_PLAY_RUN1);
     }
 
-    if (autotilt && !(mouselook || freeze || (viewplayer->cheats & MF_NOCLIP)))
+    if (autotilt && !(freelook || freeze || (viewplayer->cheats & MF_NOCLIP)))
     {
         if (!P_CheckForSteps(STEP1DISTANCE) && !P_CheckForSteps(STEP2DISTANCE))
         {
@@ -280,12 +260,13 @@ void P_MovePlayer(void)
                 viewplayer->lookdir = 0;
         }
     }
-    else if (canmouselook)
+    else if (canfreelook)
     {
         if (cmd->lookdir)
-            viewplayer->lookdir = BETWEEN(-LOOKDIRMAX * MLOOKUNIT, viewplayer->lookdir + cmd->lookdir, LOOKDIRMAX * MLOOKUNIT);
+            viewplayer->lookdir = BETWEEN(-LOOKDIRMAX * MLOOKUNIT, viewplayer->lookdir + cmd->lookdir,
+                LOOKDIRMAX * MLOOKUNIT);
 
-        if (viewplayer->lookdir && !usemouselook)
+        if (viewplayer->lookdir && !usefreelook)
         {
             if (viewplayer->lookdir > 0)
             {
@@ -310,6 +291,8 @@ static void P_ReduceDamageCount(void)
         if (r_shake_damage)
             I_UpdateBlitFunc(viewplayer->damagecount);
     }
+    else
+        healthcvar = false;
 }
 
 //
@@ -319,28 +302,28 @@ static void P_ReduceDamageCount(void)
 //
 static void P_DeathThink(void)
 {
-    static dboolean facingkiller;
-    static int      deathcount = 0;
+    static bool     facingkiller;
+    static int      deathcount;
     mobj_t          *mo = viewplayer->mo;
-    mobj_t          *attacker = viewplayer->attacker;
+    const mobj_t    *attacker = viewplayer->attacker;
 
-    weaponvibrationtics = 1;
-    idlevibrationstrength = 0;
+    weaponrumbletics = 1;
+    idlechainsawrumblestrength = 0;
     freeze = false;
-    infight = (infighting && !(viewplayer->cheats & CF_NOTARGET));
+    infight = (infighting && !solonet && !(viewplayer->cheats & CF_NOTARGET));
 
-    P_MovePsprites();
+    P_MovePlayerSprites();
 
     // fall to the ground
     if (mo->z <= mo->floorz || (mo->flags2 & MF2_ONMOBJ))
     {
-        if (canmouselook)
+        if (canfreelook)
         {
             static int  inc;
 
             if (deadlookdir == -1)
             {
-                double  viewheightrange = (double)(viewplayer->viewheight - DEADVIEWHEIGHT) / FRACUNIT;
+                const double    viewheightrange = ((double)viewplayer->viewheight - DEADVIEWHEIGHT) / FRACUNIT;
 
                 inc = MAX(1, ABS(DEADLOOKDIR - viewplayer->lookdir));
 
@@ -372,10 +355,10 @@ static void P_DeathThink(void)
 
     if (attacker && attacker != mo && !facingkiller)
     {
-        angle_t angle = R_PointToAngle2(mo->x, mo->y, attacker->x, attacker->y);
-        angle_t delta = angle - mo->angle;
+        const angle_t   angle = R_PointToAngle2(mo->x, mo->y, attacker->x, attacker->y);
+        const angle_t   delta = angle - mo->angle;
 
-        if (delta < ANG5 || delta > (unsigned int)(-ANG5))
+        if (delta < ANG5 || delta > (angle_t)(-ANG5))
         {
             // Looking at killer, so fade damage flash down.
             mo->angle = angle;
@@ -403,12 +386,13 @@ static void P_DeathThink(void)
         || ((viewplayer->cmd.buttons & BT_ATTACK) && !viewplayer->damagecount && deathcount > TICRATE * 2))
     {
         deathcount = 0;
-        damagevibrationtics = 1;
+        damagerumbletics = 1;
         viewplayer->playerstate = PST_REBORN;
         facingkiller = false;
-        skipaction = true;
         gamekeydown[' '] = false;
         gamekeydown[KEY_ENTER] = false;
+
+        S_StopSounds();
     }
     else
         deathcount++;
@@ -417,7 +401,7 @@ static void P_DeathThink(void)
 //
 // P_ResurrectPlayer
 //
-void P_ResurrectPlayer(int health)
+void P_ResurrectPlayer(const int health)
 {
     mobj_t  *mo = viewplayer->mo;
     mobj_t  *thing;
@@ -442,6 +426,7 @@ void P_ResurrectPlayer(int health)
     viewplayer->mo = thing;
     viewplayer->playerstate = PST_LIVE;
     viewplayer->viewheight = VIEWHEIGHT;
+    P_AnimateHealth(viewplayer->health - health);
     viewplayer->health = health;
     viewplayer->lookdir = 0;
     viewplayer->oldlookdir = 0;
@@ -449,15 +434,19 @@ void P_ResurrectPlayer(int health)
     viewplayer->oldrecoil = 0;
     infight = false;
 
-    P_SetupPsprites();
+    P_SetupPlayerSprites();
     P_MapEnd();
 
+    viewplayer->deaths--;
+    stat_deaths--;
+    M_SaveCVARs();
+
     C_HideConsole();
 }
 
 void P_ChangeWeapon(weapontype_t newweapon)
 {
-    weapontype_t    readyweapon = viewplayer->readyweapon;
+    const weapontype_t  readyweapon = viewplayer->readyweapon;
 
     if (newweapon == wp_fist)
     {
@@ -484,9 +473,9 @@ void P_ChangeWeapon(weapontype_t newweapon)
     else
     {
         // Don't switch to a weapon without any or enough ammo.
-        ammotype_t  ammotype = weaponinfo[newweapon].ammotype;
+        const ammotype_t    ammotype = weaponinfo[newweapon].ammotype;
 
-        if (ammotype != am_noammo && viewplayer->ammo[ammotype] < weaponinfo[newweapon].minammo)
+        if (ammotype != am_noammo && viewplayer->ammo[ammotype] < weaponinfo[newweapon].ammopershot && !infiniteammo)
             newweapon = wp_nochange;
 
         // Select the preferred shotgun.
@@ -505,7 +494,7 @@ void P_ChangeWeapon(weapontype_t newweapon)
 
     if (newweapon != wp_nochange && newweapon != readyweapon && viewplayer->weaponowned[newweapon])
     {
-        P_EquipWeapon(newweapon);
+        viewplayer->pendingweapon = newweapon;
 
         if (newweapon == wp_fist && viewplayer->powers[pw_strength])
             S_StartSound(NULL, sfx_getpow);
@@ -515,6 +504,56 @@ void P_ChangeWeapon(weapontype_t newweapon)
     }
 }
 
+void P_AnimateAllStatsFromStart(void)
+{
+    const ammotype_t    ammotype = weaponinfo[viewplayer->readyweapon].ammotype;
+
+    healthdiffspeed = MIN(ABS(-viewplayer->health) / 20 + 1, 20);
+    healthdiff = -viewplayer->health - healthdiffspeed;
+
+    armordiffspeed = MIN(ABS(-viewplayer->armor) / 20 + 1, 20);
+    armordiff = -viewplayer->armor - armordiffspeed;
+
+    ammodiffspeed[ammotype] = MIN(ABS(-viewplayer->ammo[ammotype]) / 20 + 1, 20);
+    ammodiff[ammotype] = -viewplayer->ammo[ammotype] - ammodiffspeed[ammotype];
+}
+
+void P_AnimateHealth(const int diff)
+{
+    if (animatedstats)
+    {
+        healthdiff = BETWEEN(-maxhealth, diff, maxhealth);
+        healthdiffspeed = MIN(ABS(healthdiff) / 20 + 1, 20);
+    }
+}
+
+void P_AnimateArmor(const int diff)
+{
+    if (animatedstats)
+    {
+        armordiff = diff;
+        armordiffspeed = MIN(ABS(diff) / 20 + 1, 20);
+    }
+}
+
+void P_AnimateAmmo(const int diff, const ammotype_t type)
+{
+    if (animatedstats)
+    {
+        ammodiff[type] = diff;
+        ammodiffspeed[type] = MIN(ABS(diff) / 20 + 1, 20);
+    }
+}
+
+void P_AnimateMaxAmmo(const int diff, const ammotype_t type)
+{
+    if (animatedstats)
+    {
+        maxammodiff[type] = diff;
+        maxammodiffspeed[type] = MIN(ABS(diff) / 20 + 1, 20);
+    }
+}
+
 //
 // P_PlayerThink
 //
@@ -524,21 +563,66 @@ void P_PlayerThink(void)
     mobj_t      *mo = viewplayer->mo;
     static int  motionblur;
 
+    if (viewplayer->bonuscount)
+        viewplayer->bonuscount--;
+
     if (menuactive)
     {
-        if (!inhelpscreens && ((messagetoprint && !consoleactive) || !messagetoprint))
-            mo->angle += ANG1 / (spinspeed = MIN(spinspeed + 1, 512)) * 8 * spindirection;
+        if (viewplayer->damagecount)
+            viewplayer->damagecount = MAX(0, viewplayer->damagecount - 5);
+
+        if (menuspin && !helpscreen && ((messagetoprint && !consoleactive) || !messagetoprint))
+            mo->angle += ANG1 / (menuspinspeed = MIN(menuspinspeed + 1, 512)) * 8 * menuspindirection;
 
         return;
     }
 
-    if (viewplayer->bonuscount)
-        viewplayer->bonuscount--;
+    if (animatedstats)
+    {
+        if (healthdiff < 0)
+            healthdiff = MIN(healthdiff + healthdiffspeed, 0);
+        else if (healthdiff > 0)
+            healthdiff = MAX(0, healthdiff - healthdiffspeed);
 
-    P_ReduceDamageCount();
+        if (healthdiff)
+            healthhighlight = I_GetTimeMS() + TICRATE;
+
+        if (armordiff < 0)
+            armordiff = MIN(armordiff + armordiffspeed, 0);
+        else if (armordiff > 0)
+            armordiff = MAX(0, armordiff - armordiffspeed);
+
+        if (armordiff)
+            armorhighlight = I_GetTimeMS() + TICRATE;
+
+        for (ammotype_t i = 0; i < NUMAMMO; i++)
+        {
+            if (ammodiff[i] < 0)
+                ammodiff[i] = MIN(ammodiff[i] + ammodiffspeed[i], 0);
+            else if (ammodiff[i] > 0)
+                ammodiff[i] = MAX(0, ammodiff[i] - ammodiffspeed[i]);
+
+            if (maxammodiff[i] < 0)
+                maxammodiff[i] = MIN(maxammodiff[i] + maxammodiffspeed[i], 0);
+            else if (maxammodiff[i] > 0)
+                maxammodiff[i] = MAX(0, maxammodiff[i] - maxammodiffspeed[i]);
+        }
+    }
 
     if (consoleactive)
+    {
+        if (viewplayer->damagecount)
+        {
+            if (!healthcvar)
+                viewplayer->damagecount = MAX(0, viewplayer->damagecount - 5);
+
+            P_ReduceDamageCount();
+        }
+
         return;
+    }
+
+    P_ReduceDamageCount();
 
     // [AM] Assume we can interpolate at the beginning of the tic.
     mo->interpolate = 1;
@@ -613,21 +697,30 @@ void P_PlayerThink(void)
         return;
 
     // cycle psprites
-    P_MovePsprites();
+    P_MovePlayerSprites();
 
     // [BH] regenerate health by 1% every second up to 100%
-    if (regenhealth && mo->health < initial_health && !(leveltime % TICRATE) && !viewplayer->damagecount)
-        P_GiveBody(1, MAXHEALTH, false);
+    if (regenhealth && mo->health < initial_health && !(maptime % TICRATE) && !viewplayer->damagecount)
+    {
+        P_GiveHealth(1, MAXHEALTH, true);
+        S_StartSound(viewplayer->mo, sfx_itemup);
+    }
 
     // [BH] Check all sectors player is touching are special
     for (const struct msecnode_s *seclist = mo->touching_sectorlist; seclist; seclist = seclist->m_tnext)
-        if (seclist->m_sector->special && mo->z == seclist->m_sector->floorheight)
+    {
+        sector_t    *sector = seclist->m_sector;
+
+        if (sector->special && mo->z == sector->floorheight)
         {
-            P_PlayerInSpecialSector(seclist->m_sector);
+            P_PlayerInSpecialSector(sector);
             break;
         }
+    }
 
-    if ((cmd->buttons & BT_JUMP) && (mo->z <= mo->floorz || (mo->flags2 & MF2_ONMOBJ)) && !viewplayer->jumptics)
+    if ((cmd->buttons & BT_JUMP)
+        && (mo->z <= mo->floorz || (mo->flags2 & MF2_ONMOBJ))
+        && !viewplayer->jumptics)
     {
         mo->momz = JUMPHEIGHT;
         viewplayer->jumptics = 18;
@@ -640,16 +733,14 @@ void P_PlayerThink(void)
         P_ChangeWeapon((cmd->buttons & BT_WEAPONMASK) >> BT_WEAPONSHIFT);
 
     // check for use
-    if (autouse)
+    if (autouse && !(viewplayer->cheats & CF_NOCLIP))
     {
-        if (!(leveltime % TICRATE))
-        {
-            autousing = true;
-            P_UseLines();
-            autousing = false;
-        }
+        autousing = true;
+        P_UseLines();
+        autousing = false;
     }
-    else if (cmd->buttons & BT_USE)
+
+    if (cmd->buttons & BT_USE)
     {
         if (!viewplayer->usedown)
         {
@@ -675,8 +766,10 @@ void P_PlayerThink(void)
         viewplayer->powers[pw_ironfeet]--;
 
     // Handling colormaps.
-    if (viewplayer->powers[pw_invulnerability] > STARTFLASHING || (viewplayer->powers[pw_invulnerability] & 8))
+    if (viewplayer->powers[pw_invulnerability] > STARTFLASHING
+        || (viewplayer->powers[pw_invulnerability] & FLASHONTIC))
         viewplayer->fixedcolormap = INVERSECOLORMAP;
     else
-        viewplayer->fixedcolormap = (viewplayer->powers[pw_infrared] > STARTFLASHING || (viewplayer->powers[pw_infrared] & 8));
+        viewplayer->fixedcolormap = (viewplayer->powers[pw_infrared] > STARTFLASHING
+            || (viewplayer->powers[pw_infrared] & FLASHONTIC));
 }
