@@ -1,39 +1,36 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
 #include <stdlib.h>
@@ -42,7 +39,6 @@
 #include "doomstat.h"
 #include "m_bbox.h"
 #include "m_config.h"
-#include "r_main.h"
 #include "r_plane.h"
 #include "r_segs.h"
 #include "r_things.h"
@@ -74,12 +70,12 @@ byte        *solidcol;
 //
 // Replaces the old R_Clip*WallSegment functions. It draws bits of walls in those
 // columns which aren't solid, and updates the solidcol[] array appropriately
-static void R_ClipWallSegment(int first, int last, dboolean solid)
+static void R_ClipWallSegment(int first, const int last, const bool solid)
 {
     while (first < last)
         if (solidcol[first])
         {
-            byte    *p = memchr(solidcol + first, 0, (size_t)last - first);
+            const byte  *p = memchr(solidcol + first, 0, (size_t)last - first);
 
             if (!p)
                 return;
@@ -88,8 +84,8 @@ static void R_ClipWallSegment(int first, int last, dboolean solid)
         }
         else
         {
-            byte    *p = memchr(solidcol + first, 1, (size_t)last - first);
-            int     to = (p ? (int)(p - solidcol) : last);
+            const byte  *p = memchr(solidcol + first, 1, (size_t)last - first);
+            const int   to = (p ? (int)(p - solidcol) : last);
 
             R_StoreWallRange(first, to - 1);
 
@@ -110,7 +106,7 @@ void R_InitClipSegs(void)
         + sizeof(*frontsector->floorlightsec) + sizeof(*frontsector->ceilinglightsec)
         + sizeof(frontsector->floorpic) + sizeof(frontsector->ceilingpic)
         + sizeof(frontsector->lightlevel);
-    solidcol = calloc(SCREENWIDTH, sizeof(*solidcol));
+    solidcol = calloc(MAXWIDTH, sizeof(*solidcol));
 }
 
 //
@@ -118,7 +114,7 @@ void R_InitClipSegs(void)
 //
 void R_ClearClipSegs(void)
 {
-    memset(solidcol, 0, SCREENWIDTH);
+    memset(solidcol, 0, MAXWIDTH);
 }
 
 // killough 01/18/98 -- This function is used to fix the automap bug which
@@ -242,7 +238,8 @@ static void R_InterpolateSector(sector_t *sector)
 //
 // killough 04/11/98, 04/13/98: fix bugs, add 'back' parameter
 //
-sector_t *R_FakeFlat(sector_t *sec, sector_t *tempsec, int *floorlightlevel, int *ceilinglightlevel, dboolean back)
+sector_t *R_FakeFlat(sector_t *sec, sector_t *tempsec,
+    int *floorlightlevel, int *ceilinglightlevel, const bool back)
 {
     const sector_t  *s = sec->heightsec;
 
@@ -254,8 +251,8 @@ sector_t *R_FakeFlat(sector_t *sec, sector_t *tempsec, int *floorlightlevel, int
 
     if (s)
     {
-        sector_t    *heightsec = viewplayer->mo->subsector->sector->heightsec;
-        dboolean    underwater = (heightsec && viewz <= heightsec->interpfloorheight);
+        const sector_t  *heightsec = viewplayer->mo->subsector->sector->heightsec;
+        const bool      underwater = (heightsec && viewz <= heightsec->interpfloorheight);
 
         // Replace sector being drawn, with a copy to be hacked
         *tempsec = *sec;
@@ -361,28 +358,28 @@ static void R_AddLine(seg_t *line)
     {
         // Either angle1 or angle2 is behind us, so it doesn't matter if we change it to the correct sign
         if (angle1 >= ANG180 && angle1 < ANG270)
-            angle1 = INT_MAX;           // which is ANG180 - 1
+            angle1 = INT_MAX;   // which is ANG180 - 1
         else
             angle2 = INT_MIN;
     }
 
     if ((int)angle2 >= (int)clipangle)
-        return;                         // Both off left edge
+        return;                 // Both off left edge
 
     if ((int)angle1 <= -(int)clipangle)
-        return;                         // Both off right edge
+        return;                 // Both off right edge
 
     if ((int)angle1 >= (int)clipangle)
-        angle1 = clipangle;             // Clip at left edge
+        angle1 = clipangle;     // Clip at left edge
 
     if ((int)angle2 <= -(int)clipangle)
-        angle2 = 0 - clipangle;         // Clip at right edge
+        angle2 = 0 - clipangle; // Clip at right edge
 
     // The seg is in the view range, but not necessarily visible.
 
     // killough 01/31/98: Here is where "slime trails" can SOMETIMES occur:
-    x1 = viewangletox[(angle1 + ANG90) >> ANGLETOFINESHIFT];
-    x2 = viewangletox[(angle2 + ANG90) >> ANGLETOFINESHIFT];
+    x1 = viewangletox[MIN((angle1 + ANG90) >> ANGLETOFINESHIFT, FINEANGLES / 2 - 1)];
+    x2 = viewangletox[MIN((angle2 + ANG90) >> ANGLETOFINESHIFT, FINEANGLES / 2 - 1)];
 
     // Does not cross a pixel?
     if (x1 >= x2)
@@ -391,9 +388,10 @@ static void R_AddLine(seg_t *line)
     // Single sided line?
     if ((backsector = line->backsector))
     {
-        sector_t    tempsec;            // killough 03/08/98: ceiling/water hack
+        sector_t    tempsec;    // killough 03/08/98: ceiling/water hack
 
-        // [AM] Interpolate sector movement before running clipping tests. Frontsector should already be interpolated.
+        // [AM] Interpolate sector movement before running clipping tests.
+        // Frontsector should already be interpolated.
         R_InterpolateSector(backsector);
 
         // killough 03/08/98, 04/04/98: hack for invisible ceilings/deep water
@@ -412,10 +410,9 @@ static void R_AddLine(seg_t *line)
 //
 // R_CheckBBox
 // Checks BSP node/subtree bounding box.
-// Returns true
-//  if some part of the bbox might be visible.
+// Returns true if some part of the bbox might be visible.
 //
-static dboolean R_CheckBBox(const fixed_t *bspcoord)
+static bool R_CheckBBox(const fixed_t *bspcoord)
 {
     const byte checkcoord[12][4] =
     {
@@ -478,8 +475,8 @@ static dboolean R_CheckBBox(const fixed_t *bspcoord)
         angle2 = 0 - clipangle;         // Clip at right edge
 
     // Find the first clippost that touches the source post (adjacent pixels are touching).
-    sx1 = viewangletox[(angle1 + ANG90) >> ANGLETOFINESHIFT];
-    sx2 = viewangletox[(angle2 + ANG90) >> ANGLETOFINESHIFT];
+    sx1 = viewangletox[MIN((angle1 + ANG90) >> ANGLETOFINESHIFT, FINEANGLES / 2 - 1)];
+    sx2 = viewangletox[MIN((angle2 + ANG90) >> ANGLETOFINESHIFT, FINEANGLES / 2 - 1)];
 
     // Does not cross a pixel.
     if (sx1 == sx2)
@@ -534,7 +531,7 @@ static void R_Subsector(int num)
 
     // killough 09/18/98: Fix underwater slowdown, by passing real sector
     // instead of fake one. Improve sprite lighting by basing sprite
-    // lightlevels on floor & ceiling lightlevels in the surrounding area.
+    // lightlevels on floor and ceiling lightlevels in the surrounding area.
     //
     // 10/98 killough:
     //
@@ -548,6 +545,7 @@ static void R_Subsector(int num)
     {
         sector->validcount = validcount;
         R_AddSprites(sector, (sector->heightsec ? (ceilinglightlevel + floorlightlevel) / 2 : floorlightlevel));
+        R_AddNearbySprites(sector);
     }
 
     while (count--)
@@ -563,8 +561,8 @@ static void R_Subsector(int num)
 
 void R_RenderBSPNode(int bspnum)
 {
-    int bspstack[MAX_BSP_DEPTH];
-    int sidestack[MAX_BSP_DEPTH];
+    int bspstack[MAX_BSP_DEPTH] = { 0 };
+    int sidestack[MAX_BSP_DEPTH] = { 0 };
     int sp = 0;
 
     while (true)
