@@ -1,39 +1,36 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
 #include <string.h>
@@ -43,19 +40,19 @@
 #include "m_config.h"
 #include "p_local.h"
 
-static dboolean     segtextured;        // True if any of the segs textures might be visible.
+static bool         segtextured;        // True if any of the segs textures might be visible.
 
-static dboolean     markfloor;          // False if the back side is the same plane.
-static dboolean     markceiling;
+static bool         markfloor;          // False if the back side is the same plane.
+static bool         markceiling;
 
-static dboolean     maskedtexture;
+static bool         maskedtexture;
 static int          toptexture;
 static int          midtexture;
 static int          bottomtexture;
 
-static dboolean     missingtoptexture;
-static dboolean     missingmidtexture;
-static dboolean     missingbottomtexture;
+static bool         missingtoptexture;
+static bool         missingmidtexture;
+static bool         missingbottomtexture;
 
 static fixed_t      toptexheight;
 static fixed_t      midtexheight;
@@ -92,14 +89,12 @@ static fixed_t      topstep;
 static int64_t      bottomfrac;
 static fixed_t      bottomstep;
 
-lighttable_t        **walllights;
-lighttable_t        **walllightsnext;
+static lighttable_t **walllights;
+static lighttable_t **walllightsnext;
 
 static int          *maskedtexturecol;  // dropoff overflow
 
-dboolean            r_brightmaps = r_brightmaps_default;
-
-extern dboolean     usebrightmaps;
+unsigned int        maxdrawsegs;
 
 //
 // R_FixWiggle()
@@ -114,7 +109,7 @@ extern dboolean     usebrightmaps;
 //   causing full-screen HOM, and possible program crashes.
 //
 //  Therefore, Vanilla DOOM clamps this scale calculation, preventing it
-//   from becoming larger than 0x400000 (64*FRACUNIT). This number was
+//   from becoming larger than 0x400000 (64 * FRACUNIT). This number was
 //   chosen carefully, to allow reasonably-tight angles, with reasonably
 //   tall sectors to be rendered, within the limits of the fixed-point
 //   math system being used. When the scale gets clamped, DOOM cannot
@@ -206,7 +201,7 @@ static void R_BlastMaskedSegColumn(const rcolumn_t *column)
         const int64_t   topscreen = sprtopscreen + (int64_t)spryscale * topdelta + 1;
 
         if ((dc_yh = MIN((int)((topscreen + (int64_t)spryscale * post->length) >> FRACBITS), dc_floorclip)) >= 0)
-            if ((dc_yl = MAX(dc_ceilingclip, (int)((topscreen + FRACUNIT) >> FRACBITS))) <= dc_yh)
+            if ((dc_yl = MAX(dc_ceilingclip, (int)((topscreen + FRACUNIT - 1) >> FRACBITS))) <= dc_yh)
             {
                 dc_texturefrac = dc_texturemid - (topdelta << FRACBITS) + FixedMul((dc_yl - centery) << FRACBITS, dc_iscale);
                 dc_source = pixels + topdelta;
@@ -218,14 +213,13 @@ static void R_BlastMaskedSegColumn(const rcolumn_t *column)
 //
 // R_RenderMaskedSegRange
 //
-void R_RenderMaskedSegRange(drawseg_t *ds, const int x1, const int x2)
+void R_RenderMaskedSegRange(const drawseg_t *ds, const int x1, const int x2)
 {
     int             texnum;
     fixed_t         texheight;
     const rpatch_t  *patch;
 
     curline = ds->curline;
-    colfunc = (curline->linedef->tranlump >= 0 ? tl50segcolfunc : segcolfunc);
     frontsector = curline->frontsector;
     backsector = curline->backsector;
     texnum = texturetranslation[curline->sidedef->midtexture];
@@ -238,14 +232,23 @@ void R_RenderMaskedSegRange(drawseg_t *ds, const int x1, const int x2)
     {
         dc_colormap[0] = fixedcolormap;
         dc_nextcolormap[0] = fixedcolormap;
+        colfunc = (curline->linedef->tranlump >= 0 ? tl50segcolfunc : segcolfunc);
     }
     else
     {
         sector_t    tempsec;
-        short       lightlevel = R_FakeFlat(frontsector, &tempsec, NULL, NULL, false)->lightlevel;
+        const short lightlevel = R_FakeFlat(frontsector, &tempsec, NULL, NULL, false)->lightlevel;
 
         walllights = GetLightTable(lightlevel);
-        walllightsnext = (r_ditheredlighting ? GetLightTable(lightlevel + 4) : walllights);
+        walllightsnext = GetLightTable(lightlevel + 4);
+
+        if (usebrightmaps && !nobrightmap[texnum] && brightmap[texnum])
+        {
+            dc_brightmap = brightmap[texnum];
+            colfunc = bmapsegcolfunc;
+        }
+        else
+            colfunc = (curline->linedef->tranlump >= 0 ? tl50segcolfunc : segcolfunc);
     }
 
     maskedtexturecol = ds->maskedtexturecol;
@@ -269,42 +272,42 @@ void R_RenderMaskedSegRange(drawseg_t *ds, const int x1, const int x2)
         if (maskedtexturecol[dc_x] != INT_MAX)
         {
             const rcolumn_t *column = R_GetPatchColumnWrapped(patch, maskedtexturecol[dc_x]);
-            int64_t         t;
 
-            if (!(dc_numposts = column->numposts))
-                continue;
-
-            // killough 03/02/98:
-            //
-            // This calculation used to overflow and cause crashes in DOOM:
-            //
-            // sprtopscreen = centeryfrac - FixedMul(dc_texturemid, spryscale);
-            //
-            // This code fixes it, by using double-precision intermediate
-            // arithmetic and by skipping the drawing of 2s normals whose
-            // mapping to screen coordinates is totally out of range:
-            t = ((int64_t)centeryfrac << FRACBITS) - (int64_t)dc_texturemid * spryscale;
-
-            if (t + (int64_t)texheight * spryscale < 0 || t > (int64_t)SCREENHEIGHT << FRACBITS * 2)
-                continue;                       // skip if the texture is out of screen's range
-
-            sprtopscreen = (int64_t)(t >> FRACBITS);
-
-            // calculate lighting
-            if (!fixedcolormap)
+            if ((dc_numposts = column->numposts))
             {
-                int index = MIN(spryscale >> LIGHTSCALESHIFT, MAXLIGHTSCALE - 1);
+                // killough 03/02/98:
+                //
+                // This calculation used to overflow and cause crashes in DOOM:
+                //
+                // sprtopscreen = centeryfrac - FixedMul(dc_texturemid, spryscale);
+                //
+                // This code fixes it, by using double-precision intermediate
+                // arithmetic and by skipping the drawing of 2s normals whose
+                // mapping to screen coordinates is totally out of range:
+                const int64_t   t = ((int64_t)centeryfrac << FRACBITS) - (int64_t)dc_texturemid * spryscale;
 
-                dc_colormap[0] = walllights[index];
-                dc_nextcolormap[0] = walllightsnext[index];
-                dc_z = spryscale;
+                // skip if the texture is out of screen's range
+                if (t + (int64_t)texheight * spryscale < 0 || t > (int64_t)SCREENHEIGHT << FRACBITS * 2)
+                    continue;
+
+                sprtopscreen = (int64_t)(t >> FRACBITS);
+
+                // calculate lighting
+                if (!fixedcolormap)
+                {
+                    const int   index = MIN(spryscale >> LIGHTSCALESHIFT, MAXLIGHTSCALE - 1);
+
+                    dc_colormap[0] = walllights[index];
+                    dc_nextcolormap[0] = walllightsnext[index];
+                    dc_z = ((spryscale >> 5) & 255);
+                }
+
+                dc_iscale = UINT_MAX / (unsigned int)spryscale;
+
+                // draw the texture
+                R_BlastMaskedSegColumn(column);
+                maskedtexturecol[dc_x] = INT_MAX;   // dropoff overflow
             }
-
-            dc_iscale = UINT_MAX / (unsigned int)spryscale;
-
-            // draw the texture
-            R_BlastMaskedSegColumn(column);
-            maskedtexturecol[dc_x] = INT_MAX;   // dropoff overflow
         }
 }
 
@@ -314,7 +317,7 @@ void R_RenderMaskedSegRange(drawseg_t *ds, const int x1, const int x2)
 // Can draw or mark the starting pixel of floor and ceiling textures.
 // CALLED: CORE LOOPING ROUTINE.
 //
-static dboolean didsolidcol;
+static bool didsolidcol;
 
 static void R_RenderSegLoop(void)
 {
@@ -326,15 +329,15 @@ static void R_RenderSegLoop(void)
 
     for (; rw_x < rw_stopx; rw_x++)
     {
-        fixed_t texturecolumn = 0;
+        fixed_t     texturecolumn = 0;
 
         // no space above wall?
-        int     bottom;
-        int     top = ceilingclip[rw_x] + 1;
+        int         bottom;
+        int         top = ceilingclip[rw_x] + 1;
 
         // mark floor/ceiling areas
-        int     yl = MAX((int)((topfrac + heightunit - 1) >> heightbits), top);
-        int     yh = (int)(bottomfrac >> heightbits);
+        const int   yl = MAX((int)((topfrac + heightunit - 1) >> heightbits), top);
+        int         yh = (int)(bottomfrac >> heightbits);
 
         if (markceiling)
         {
@@ -373,11 +376,11 @@ static void R_RenderSegLoop(void)
 
             if (!fixedcolormap)
             {
-                int index = MIN(rw_scale >> LIGHTSCALESHIFT, MAXLIGHTSCALE - 1);
+                const int   index = MIN(rw_scale >> LIGHTSCALESHIFT, MAXLIGHTSCALE - 1);
 
                 dc_colormap[0] = walllights[index];
                 dc_nextcolormap[0] = walllightsnext[index];
-                dc_z = rw_scale;
+                dc_z = ((rw_scale >> 5) & 255);
             }
 
             dc_x = rw_x;
@@ -392,7 +395,7 @@ static void R_RenderSegLoop(void)
             dc_yh = yh;
 
             if (missingmidtexture)
-                R_DrawColorColumn();
+                missingcolfunc();
             else
             {
                 dc_source = R_GetTextureColumn(R_CacheTextureCompositePatchNum(midtexture), texturecolumn);
@@ -405,7 +408,7 @@ static void R_RenderSegLoop(void)
 
                     if (r_ditheredlighting)
                     {
-                        if (dc_colormap == dc_nextcolormap)
+                        if (!memcmp(dc_colormap, dc_nextcolormap, sizeof(dc_colormap)))
                             altbmapwallcolfunc();
                         else
                             bmapwallcolfunc();
@@ -415,7 +418,7 @@ static void R_RenderSegLoop(void)
                 }
                 else if (r_ditheredlighting)
                 {
-                    if (dc_colormap == dc_nextcolormap)
+                    if (!memcmp(dc_colormap, dc_nextcolormap, sizeof(dc_colormap)))
                         altwallcolfunc();
                     else
                         wallcolfunc();
@@ -443,7 +446,7 @@ static void R_RenderSegLoop(void)
                     dc_yh = mid;
 
                     if (missingtoptexture)
-                        R_DrawColorColumn();
+                        missingcolfunc();
                     else
                     {
                         dc_source = R_GetTextureColumn(R_CacheTextureCompositePatchNum(toptexture), texturecolumn);
@@ -457,7 +460,7 @@ static void R_RenderSegLoop(void)
 
                             if (r_ditheredlighting)
                             {
-                                if (dc_colormap == dc_nextcolormap)
+                                if (!memcmp(dc_colormap, dc_nextcolormap, sizeof(dc_colormap)))
                                     altbmapwallcolfunc();
                                 else
                                     bmapwallcolfunc();
@@ -467,7 +470,7 @@ static void R_RenderSegLoop(void)
                         }
                         else if (r_ditheredlighting)
                         {
-                            if (dc_colormap == dc_nextcolormap)
+                            if (!memcmp(dc_colormap, dc_nextcolormap, sizeof(dc_colormap)))
                                 altwallcolfunc();
                             else
                                 wallcolfunc();
@@ -499,7 +502,7 @@ static void R_RenderSegLoop(void)
                     dc_yh = yh;
 
                     if (missingbottomtexture)
-                        R_DrawColorColumn();
+                        missingcolfunc();
                     else
                     {
                         dc_source = R_GetTextureColumn(R_CacheTextureCompositePatchNum(bottomtexture), texturecolumn);
@@ -512,7 +515,7 @@ static void R_RenderSegLoop(void)
 
                             if (r_ditheredlighting)
                             {
-                                if (dc_colormap == dc_nextcolormap)
+                                if (!memcmp(dc_colormap, dc_nextcolormap, sizeof(dc_colormap)))
                                     altbmapwallcolfunc();
                                 else
                                     bmapwallcolfunc();
@@ -522,7 +525,7 @@ static void R_RenderSegLoop(void)
                         }
                         else if (r_ditheredlighting)
                         {
-                            if (dc_colormap == dc_nextcolormap)
+                            if (!memcmp(dc_colormap, dc_nextcolormap, sizeof(dc_colormap)))
                                 altwallcolfunc();
                             else
                                 wallcolfunc();
@@ -580,20 +583,25 @@ static fixed_t R_ScaleFromGlobalAngle(angle_t visangle)
 //
 void R_StoreWallRange(const int start, const int stop)
 {
-    int64_t             dx, dy;
-    int64_t             dx1, dy1;
-    int64_t             len;
-    int                 worldtop;
-    int                 worldbottom;
-    int                 worldhigh = 0;
-    int                 worldlow = 0;
-    side_t              *sidedef;
-    static unsigned int maxdrawsegs;
+    int64_t         dx, dy;
+    int64_t         dx1, dy1;
+    int64_t         len;
+    int             worldtop;
+    int             worldbottom;
+    int             worldhigh = 0;
+    int             worldlow = 0;
+    side_t          *sidedef;
+    unsigned short  flags;
 
     linedef = curline->linedef;
+    flags = linedef->flags;
 
     // mark the segment as visible for automap
-    linedef->flags |= ML_MAPPED;
+    if (!menuactive && !(flags & ML_MAPPED) && !(flags & ML_DONTDRAW))
+    {
+        nummappedlines++;
+        linedef->flags |= ML_MAPPED;
+    }
 
     // [BH] if in automap, we're done now that line is mapped
     if (automapactive)
@@ -646,7 +654,7 @@ void R_StoreWallRange(const int start, const int stop)
             openings = I_Realloc(openings, maxopenings * sizeof(*openings));
             lastopening = openings + pos;
 
-            // jff 8/9/98 borrowed fix for openings from ZDOOM1.14
+            // jff 08/09/98 borrowed fix for openings from ZDOOM 1.14
             // [RH] We also need to adjust the openings pointers that
             //    were already stored in drawsegs.
             for (drawseg_t *ds = drawsegs; ds < ds_p; ds++)
@@ -667,8 +675,9 @@ void R_StoreWallRange(const int start, const int stop)
     worldbottom = frontsector->interpfloorheight - viewz;
 
     // [BH] animate liquid sectors
-    if (frontsector->terraintype != SOLID
-        && (!frontsector->heightsec || viewz > frontsector->heightsec->interpfloorheight)
+    if (frontsector->terraintype >= LIQUID
+        && (viewz > frontsector->interpfloorheight + FRACUNIT * 2 || !P_IsInLiquid(viewplayer->mo))
+        && (!frontsector->heightsec || viewz > frontsector->heightsec->interpfloorheight + FRACUNIT * 2)
         && r_liquid_bob)
         worldbottom += animatedliquiddiff;
 
@@ -680,7 +689,7 @@ void R_StoreWallRange(const int start, const int stop)
 
     if (stop > start)
     {
-        fixed_t scale = R_ScaleFromGlobalAngle(xtoviewangle[stop]);
+        const fixed_t   scale = R_ScaleFromGlobalAngle(xtoviewangle[stop]);
 
         rw_scalestep = (scale - rw_scale) / (stop - start);
         ds_p->scalestep = rw_scalestep;
@@ -768,9 +777,10 @@ void R_StoreWallRange(const int start, const int stop)
             worldtop = worldhigh;
 
         // [BH] animate liquid sectors
-        if (backsector->terraintype != SOLID
+        if (backsector->terraintype >= LIQUID
             && backsector->interpfloorheight >= frontsector->interpfloorheight
-            && (!backsector->heightsec || viewz > backsector->heightsec->interpfloorheight)
+            && (viewz > backsector->interpfloorheight + FRACUNIT * 2 || !P_IsInLiquid(viewplayer->mo))
+            && (!backsector->heightsec || viewz > backsector->heightsec->interpfloorheight + FRACUNIT * 2)
             && r_liquid_bob)
         {
             liquidoffset = animatedliquiddiff;
@@ -781,27 +791,28 @@ void R_StoreWallRange(const int start, const int stop)
             || backsector->floorpic != frontsector->floorpic
             || backsector->lightlevel != frontsector->lightlevel
 
-            // killough 03/07/98: Add checks for (x,y) offsets
+            // killough 03/07/98: add checks for (x,y) offsets
             || backsector->floorxoffset != frontsector->floorxoffset
             || backsector->flooryoffset != frontsector->flooryoffset
 
-            // killough 04/15/98: prevent 2s normals
-            // from bleeding through deep water
+            // killough 04/15/98: prevent 2s normals from bleeding through deep water
             || frontsector->heightsec
 
             // killough 04/17/98: draw floors if different light levels
-            || backsector->floorlightsec != frontsector->floorlightsec);
+            || backsector->floorlightsec != frontsector->floorlightsec
+
+            // draw floors if different specials
+            || backsector->special != frontsector->special);
 
         markceiling = (worldhigh != worldtop
             || backsector->ceilingpic != frontsector->ceilingpic
             || backsector->lightlevel != frontsector->lightlevel
 
-            // killough 03/07/98: Add checks for (x,y) offsets
+            // killough 03/07/98: add checks for (x,y) offsets
             || backsector->ceilingxoffset != frontsector->ceilingxoffset
             || backsector->ceilingyoffset != frontsector->ceilingyoffset
 
-            // killough 04/15/98: prevent 2s normals
-            // from bleeding through fake ceilings
+            // killough 04/15/98: prevent 2s normals from bleeding through fake ceilings
             || (frontsector->heightsec && frontsector->ceilingpic != skyflatnum)
 
             // killough 04/17/98: draw ceilings if different light levels
@@ -865,7 +876,7 @@ void R_StoreWallRange(const int start, const int stop)
     }
 
     // calculate rw_offset (only needed for textured lines)
-    if ((segtextured = (midtexture | toptexture | bottomtexture | maskedtexture)))
+    if ((segtextured = (midtexture || toptexture || bottomtexture || maskedtexture)))
     {
         rw_offset = (fixed_t)(((dx * dx1 + dy * dy1) / len) << 1) + sidedef->textureoffset + curline->offset;
         rw_centerangle = ANG90 + viewangle - rw_normalangle;
@@ -874,10 +885,10 @@ void R_StoreWallRange(const int start, const int stop)
         //  use different light tables for horizontal/vertical
         if (!fixedcolormap)
         {
-            short   lightlevel = frontsector->lightlevel;
+            const short lightlevel = frontsector->lightlevel;
 
             walllights = GetLightTable(lightlevel);
-            walllightsnext = (r_ditheredlighting ? GetLightTable(lightlevel + 4) : walllights);
+            walllightsnext = GetLightTable(lightlevel + 4);
         }
     }
 
@@ -931,7 +942,7 @@ void R_StoreWallRange(const int start, const int stop)
         // killough 04/11/98: add NULL ptr checks
         if (floorplane)
         {
-            // cph 2003/04/18  - ceilingplane and floorplane might be the same
+            // cph 04/18/03: ceilingplane and floorplane might be the same
             // visplane (e.g. if both skies); R_CheckPlane() doesn't know about
             // modifications to the plane that might happen in parallel with the check
             // being made, so we have to override it and split them anyway if that is
