@@ -1,39 +1,36 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
 #include <ctype.h>
@@ -68,7 +65,7 @@
 // Stereo separation
 #define S_STEREO_SWING  96
 
-#define NORM_SEP        128
+#define NORM_SEP        127
 
 #define TIDNUM(x)       (int)(x->musicid & 0xFFFF)  // thing identifier
 
@@ -95,37 +92,53 @@ typedef struct
 static channel_t    *channels;
 static sobj_t       *sobjs;
 
-int                 s_channels = s_channels_default;
-int                 s_musicvolume = s_musicvolume_default;
-dboolean            s_randommusic = s_randommusic_default;
-dboolean            s_randompitch = s_randompitch_default;
-int                 s_sfxvolume = s_sfxvolume_default;
-dboolean            s_stereo = s_stereo_default;
-
 // Maximum volume of a sound effect.
 // Internal default is max out of 0-31.
-int                 sfxVolume;
+int                 sfxvolume;
 
 // Maximum volume of music.
-int                 musicVolume;
+int                 musicvolume;
 
-// Internal volume level, ranging from 0-MIX_MAX_VOLUME
-static int          snd_SfxVolume;
+// Internal volume level, ranging from 0 to 127
+static int          snd_sfxvolume;
 
 // Whether songs are mus_paused
-static dboolean     mus_paused;
+static bool         mus_paused;
 
 // Music currently being played
 musicinfo_t         *mus_playing;
 
-dboolean            nosfx;
-dboolean            nomusic;
+bool                nosfx;
+bool                nomusic;
 
 musinfo_t           musinfo;
 
-#if defined(_WIN32)
-extern dboolean     serverMidiPlaying;
-#endif
+const int spmus[] =
+{
+    // Song - Who? - Where?
+    mus_e3m4,   // American     E4M1
+    mus_e3m2,   // Romero       E4M2
+    mus_e3m3,   // Shawn        E4M3
+    mus_e1m5,   // American     E4M4
+    mus_e2m7,   // Tim          E4M5
+    mus_e2m4,   // Romero       E4M6
+    mus_e2m6,   // J.Anderson   E4M7 CHIRON.WAD
+    mus_e2m5,   // Shawn        E4M8
+    mus_e1m9    // Tim          E4M9
+};
+
+const int nmus[] =
+{
+    mus_messag,
+    mus_ddtblu,
+    mus_doom,
+    mus_shawn,
+    mus_in_cit,
+    mus_the_da,
+    mus_in_cit,
+    mus_shawn,
+    mus_ddtblu
+};
 
 // Initialize sound effects.
 static void InitSfxModule(void)
@@ -134,13 +147,14 @@ static void InitSfxModule(void)
     {
         const char  *audiodriver = SDL_GetCurrentAudioDriver();
 
-        C_Output("Sound effects are playing at a sample rate of %.1fkHz over %i channels%s", SAMPLERATE / 1000.0f, s_channels,
-            (M_StringCompare(audiodriver, "wasapi") ? " using " ITALICS("WASAPI.") :
+        C_Output("Sound effects are playing at %i%% volume and a sample rate of %.1fkHz over %i channels%s",
+            s_sfxvolume, SAMPLERATE / 1000.0f, s_channels,
+            (M_StringCompare(audiodriver, "wasapi") ? " using the " ITALICS("WASAPI.") :
             (M_StringCompare(audiodriver, "directsound") ? " using the " ITALICS("DirectSound") " API." : "")));
     }
     else
     {
-        C_Warning(1, "Sound effects couldn't be initialized.");
+        C_Warning(1, "Sound effects couldn't be %s.", (english == english_american ? "initialized" : "initialised"));
         nosfx = true;
     }
 }
@@ -150,20 +164,20 @@ static void InitMusicModule(void)
 {
     if (!I_InitMusic())
     {
-        C_Warning(1, "Music couldn't be initialized.");
+        C_Warning(1, "Music couldn't be %s.", (english == english_american ? "initialized" : "initialised"));
         nomusic = true;
     }
 }
 
 //
 // Initializes sound stuff, including volume
-// Sets channels, SFX and music volume, allocates channel buffer, sets S_sfx lookup.
+// Sets channels, SFX and music volume, allocates channel buffer, sets s_sfx lookup.
 //
 void S_Init(void)
 {
     if (M_CheckParm("-nosound"))
     {
-        C_Warning(1, "A " BOLD("-nosound") " parameter was found on the command-line. Both sound effects and music have been disabled.");
+        C_Warning(1, "A " BOLD("-nosound") " parameter was found on the command-line. No sound effects or music will be played.");
         nomusic = true;
         nosfx = true;
     }
@@ -171,13 +185,13 @@ void S_Init(void)
     {
         if (M_CheckParm("-nomusic"))
         {
-            C_Warning(1, "A " BOLD("-nomusic") " parameter was found on the command-line. Music has been disabled.");
+            C_Warning(1, "A " BOLD("-nomusic") " parameter was found on the command-line. No music will be played.");
             nomusic = true;
         }
 
         if (M_CheckParm("-nosfx"))
         {
-            C_Warning(1, "A " BOLD("-nosfx") " parameter was found on the command-line. Sound effects have been disabled.");
+            C_Warning(1, "A " BOLD("-nosfx") " parameter was found on the command-line. No sound effects will be played.");
             nosfx = true;
         }
     }
@@ -195,43 +209,30 @@ void S_Init(void)
 #endif
 
         InitSfxModule();
-        sfxVolume = (s_sfxvolume * 31 + 50) / 100;
-        S_SetSfxVolume(sfxVolume * MIX_MAX_VOLUME / 31);
+        sfxvolume = (s_sfxvolume * 31 + 50) / 100;
+        S_SetSfxVolume(sfxvolume * (MIX_MAX_VOLUME - 1) / 31);
 
-        // Allocating the internal channels for mixing (the maximum number of sounds rendered simultaneously) within zone memory.
+        // Allocating the internal channels for mixing (the maximum number of sounds played simultaneously) within zone memory.
         channels = Z_Calloc(s_channels_max, sizeof(channel_t), PU_STATIC, NULL);
         sobjs = Z_Malloc(s_channels_max * sizeof(sobj_t), PU_STATIC, NULL);
 
         // [BH] precache all SFX
-        for (int i = 1; i < NUMSFX; i++)
+        for (int i = 1; i < numsfx; i++)
         {
-            sfxinfo_t   *sfx = &S_sfx[i];
-            char        namebuf[9];
+            sfxinfo_t   *sfx = &s_sfx[i];
 
-            M_snprintf(namebuf, sizeof(namebuf), "ds%s", sfx->name1);
-
-            if ((sfx->lumpnum = W_CheckNumForName(namebuf)) >= 0)
+            if (*s_sfx[i].name1)
             {
-                if (!CacheSFX(sfx) && W_CheckMultipleLumps(namebuf) > 1)
-                {
-                    sfx->lumpnum = W_GetLastNumForName(namebuf);
+                char    namebuf[9];
 
-                    if (!CacheSFX(sfx))
-                        sfx->lumpnum = -1;
-                    else
-                    {
-                        char    *temp = uppercase(namebuf);
+                M_snprintf(namebuf, sizeof(namebuf), "ds%s", sfx->name1);
 
-                        C_Warning(1, "The " BOLD("%s") " sound lump is in an unknown format.", temp);
-                        free(temp);
-                    }
-                }
-
-                if (sfx->lumpnum == -1)
+                if ((sfx->lumpnum = W_CheckNumForName(namebuf)) >= 0 && !CacheSFX(sfx))
                 {
                     char    *temp = uppercase(namebuf);
 
-                    C_Warning(1, "The " BOLD("%s") " sound lump is in an unknown format and won't be played.", temp);
+                    sfx->lumpnum = -1;
+                    C_Warning(1, "The " BOLD("%s") " sound effect lump won't be played.", temp);
                     free(temp);
                 }
             }
@@ -241,13 +242,27 @@ void S_Init(void)
     if (!nomusic)
     {
         InitMusicModule();
-        musicVolume = (s_musicvolume * 31 + 50) / 100;
-        S_SetMusicVolume(musicVolume * MIX_MAX_VOLUME / 31);
+        musicvolume = (s_musicvolume * 31 + 50) / 100;
+        S_RestoreMusicVolume();
 
         // no sounds are playing, and they are not mus_paused
         mus_paused = false;
 
         musinfo.mapthing = NULL;
+
+        for (int i = mus_e4m1, j = 0; i <= mus_e4m9; i++, j++)
+        {
+            musicinfo_t *music = &s_music[i];
+            char        namebuf[9];
+
+            M_snprintf(namebuf, sizeof(namebuf), "d_%s", music->name1);
+
+            if (W_CheckNumForName(namebuf) == -1)
+            {
+                M_StringCopy(music->name1, s_music[spmus[j]].name1, sizeof(music->name1));
+                M_StringCopy(music->title1, s_music[spmus[j]].title1, sizeof(music->title1));
+            }
+        }
     }
 }
 
@@ -277,60 +292,51 @@ static void S_StopChannel(int cnum)
     }
 }
 
+void S_StopSound(const sfxnum_t sfxnum)
+{
+    sfxinfo_t   *sfx;
+
+    if (nosfx)
+        return;
+
+    sfx = &s_sfx[sfxnum];
+
+    for (int cnum = 0; cnum < s_channels; cnum++)
+        if (channels[cnum].sfxinfo == sfx)
+        {
+            S_StopChannel(cnum);
+            break;
+        }
+}
+
+void S_StopSounds(void)
+{
+    if (nosfx)
+        return;
+
+    for (int cnum = 0; cnum < s_channels; cnum++)
+        S_StopChannel(cnum);
+}
+
 static int S_GetMusicNum(void)
 {
-    int mnum;
-
     if (gamemode == commercial)
     {
         if (gamemission == pack_nerve)
-        {
-            int nmus[] =
-            {
-                mus_messag,
-                mus_ddtblu,
-                mus_doom,
-                mus_shawn,
-                mus_in_cit,
-                mus_the_da,
-                mus_in_cit,
-                mus_shawn,
-                mus_ddtblu
-            };
-
-            mnum = nmus[(s_randommusic ? M_RandomIntNoRepeat(1, 9, gamemap) : gamemap) - 1];
-        }
+            return nmus[(s_randommusic ? M_RandomIntNoRepeat(1, 9, gamemap) : gamemap) - 1];
         else
-            mnum = mus_runnin + (s_randommusic ? M_RandomIntNoRepeat(1, 32, gamemap) : gamemap) - 1;
+            return (mus_runnin + (s_randommusic ? M_RandomIntNoRepeat(1, 32, gamemap) : gamemap) - 1);
     }
     else
     {
-        if (gameepisode < 4)
-            mnum = mus_e1m1 + (s_randommusic ? M_RandomIntNoRepeat(1, 21, (gameepisode - 1) * 9 + gamemap) :
-                (gameepisode - 1) * 9 + gamemap) - 1;
-        else if (gameepisode == 5 && sigil)
-            mnum = mus_e5m1 + (s_randommusic ? M_RandomIntNoRepeat(1, 9, gamemap) : gamemap) - 1;
+        if (gameepisode == 5 && sigil)
+            return (mus_e5m1 + (s_randommusic ? M_RandomIntNoRepeat(1, 9, gamemap) : gamemap) - 1);
+        else if (gameepisode == 6 && sigil2)
+            return (mus_e6m1 + (s_randommusic ? M_RandomIntNoRepeat(1, 9, gamemap) : gamemap) - 1);
         else
-        {
-            int spmus[] =
-            {
-                // Song - Who? - Where?
-                mus_e3m4,   // American     E4M1
-                mus_e3m2,   // Romero       E4M2
-                mus_e3m3,   // Shawn        E4M3
-                mus_e1m5,   // American     E4M4
-                mus_e2m7,   // Tim          E4M5
-                mus_e2m4,   // Romero       E4M6
-                mus_e2m6,   // J.Anderson   E4M7 CHIRON.WAD
-                mus_e2m5,   // Shawn        E4M8
-                mus_e1m9    // Tim          E4M9
-            };
-
-            mnum = spmus[(s_randommusic ? M_RandomIntNoRepeat(1, 9, gamemap) : gamemap) - 1];
-        }
+            return (mus_e1m1 + (s_randommusic ? M_RandomIntNoRepeat(1, 4 * 9, (gameepisode - 1) * 9 + gamemap) :
+                (gameepisode - 1) * 9 + gamemap) - 1);
     }
-
-    return mnum;
 }
 
 //
@@ -352,8 +358,8 @@ void S_Start(void)
 // so stereo positioning and distance calculations continue to work even after
 // the corresponding map object has already disappeared.
 // Thanks to jeff-d and kb1 for discussing this feature and the former for the
-// original implementation idea: <https://www.doomworld.com/forum/topic/1585325>
-void S_UnlinkSound(mobj_t *origin)
+// original implementation idea: <https://www.doomworld.com/forum/post/1585325>
+void S_UnlinkSound(const mobj_t *origin)
 {
     if (!origin->madesound || nosfx)
         return;
@@ -367,14 +373,13 @@ void S_UnlinkSound(mobj_t *origin)
             sobj->y = origin->y;
             sobj->z = origin->z;
             channels[cnum].origin = (mobj_t *)sobj;
-
             break;
         }
 }
 
 //
-// S_GetChannel :
-//   If none available, return -1. Otherwise channel #.
+// S_GetChannel
+// If none available, return -1. Otherwise channel #.
 //
 static int S_GetChannel(mobj_t *origin, sfxinfo_t *sfxinfo)
 {
@@ -401,9 +406,9 @@ static int S_GetChannel(mobj_t *origin, sfxinfo_t *sfxinfo)
                 break;
 
         if (cnum == s_channels)
-            return -1;                  // FUCK! No lower priority. Sorry, Charlie.
+            return -1;              // FUCK! No lower priority. Sorry, Charlie.
         else
-            S_StopChannel(cnum);        // Otherwise, kick out lower priority.
+            S_StopChannel(cnum);    // Otherwise, kick out lower priority.
     }
 
     c = &channels[cnum];
@@ -418,19 +423,18 @@ static int S_GetChannel(mobj_t *origin, sfxinfo_t *sfxinfo)
 // Changes volume and stereo-separation variables from the norm of a sound
 // effect to be played. If the sound is not audible, returns false. Otherwise,
 // modifies parameters and returns true.
-static dboolean S_AdjustSoundParms(mobj_t *origin, int *vol, int *sep)
+static bool S_AdjustSoundParms(const mobj_t *origin, int *vol, int *sep)
 {
-    fixed_t     dist = 0;
-    mobj_t      *listener = viewplayer->mo;
-    dboolean    boss = (origin->flags2 & MF2_BOSS);
-    fixed_t     x = origin->x;
-    fixed_t     y = origin->y;
+    fixed_t         dist = 0;
+    const mobj_t    *listener = viewplayer->mo;
+    const fixed_t   x = origin->x;
+    const fixed_t   y = origin->y;
 
     // calculate the distance to sound origin and clip it if necessary
     // killough 11/98: scale coordinates down before calculations start
     // killough 12/98: use exact distance formula instead of approximation
-    fixed_t     adx = ABS((listener->x >> FRACBITS) - (x >> FRACBITS));
-    fixed_t     ady = ABS((listener->y >> FRACBITS) - (y >> FRACBITS));
+    fixed_t adx = ABS((listener->x >> FRACBITS) - (x >> FRACBITS));
+    fixed_t ady = ABS((listener->y >> FRACBITS) - (y >> FRACBITS));
 
     if (ady > adx)
         SWAP(adx, ady);
@@ -442,12 +446,12 @@ static dboolean S_AdjustSoundParms(mobj_t *origin, int *vol, int *sep)
     if (!dist)
     {
         *sep = NORM_SEP;
-        *vol = snd_SfxVolume;
+        *vol = snd_sfxvolume;
 
         return (*vol > 0);
     }
 
-    if (!boss && dist > S_CLIPPING_DIST)
+    if (dist > S_CLIPPING_DIST)
         return false;
 
     // stereo separation
@@ -463,18 +467,18 @@ static dboolean S_AdjustSoundParms(mobj_t *origin, int *vol, int *sep)
     }
 
     // volume calculation
-    *vol = (dist < S_CLOSE_DIST || boss ? snd_SfxVolume : snd_SfxVolume * (S_CLIPPING_DIST - dist) / S_ATTENUATOR);
+    *vol = (dist < S_CLOSE_DIST ? snd_sfxvolume : snd_sfxvolume * (S_CLIPPING_DIST - dist) / S_ATTENUATOR);
 
     return (*vol > 0);
 }
 
-static void S_StartSoundAtVolume(mobj_t *origin, int sfx_id, int pitch)
+static void S_StartSoundAtVolume(mobj_t *origin, sfxnum_t sfxnum, int pitch)
 {
-    sfxinfo_t   *sfx = &S_sfx[sfx_id];
+    sfxinfo_t   *sfx = &s_sfx[sfxnum];
     int         sep = NORM_SEP;
     int         cnum;
     int         handle;
-    int         volume = snd_SfxVolume;
+    int         volume = snd_sfxvolume;
 
     if (sfx->lumpnum == -1 || nosfx)
         return;
@@ -483,17 +487,6 @@ static void S_StartSoundAtVolume(mobj_t *origin, int sfx_id, int pitch)
     if (origin && origin != viewplayer->mo && !S_AdjustSoundParms(origin, &volume, &sep))
         return;
 
-    // kill old sound
-    if (origin)
-        for (cnum = 0; cnum < s_channels; cnum++)
-            if (channels[cnum].sfxinfo
-                && channels[cnum].sfxinfo->singularity == sfx->singularity
-                && channels[cnum].origin == origin)
-            {
-                S_StopChannel(cnum);
-                break;
-            }
-
     // try to find a channel
     if ((cnum = S_GetChannel(origin, sfx)) < 0)
         return;
@@ -504,20 +497,20 @@ static void S_StartSoundAtVolume(mobj_t *origin, int sfx_id, int pitch)
         channels[cnum].handle = handle;
 }
 
-void S_StartSound(mobj_t *mobj, int sfx_id)
+void S_StartSound(mobj_t *mobj, const sfxnum_t sfxnum)
 {
     if (mobj)
     {
         mobj->madesound = true;
-        S_StartSoundAtVolume(mobj, sfx_id, mobj->pitch);
+        S_StartSoundAtVolume(mobj, sfxnum, mobj->pitch);
     }
     else
-        S_StartSoundAtVolume(NULL, sfx_id, NORM_PITCH);
+        S_StartSoundAtVolume(NULL, sfxnum, NORM_PITCH);
 }
 
-void S_StartSectorSound(degenmobj_t *degenmobj, int sfx_id)
+void S_StartSectorSound(degenmobj_t *degenmobj, const sfxnum_t sfxnum)
 {
-    S_StartSoundAtVolume((mobj_t *)degenmobj, sfx_id, NORM_PITCH);
+    S_StartSoundAtVolume((mobj_t *)degenmobj, sfxnum, NORM_PITCH);
 }
 
 //
@@ -556,14 +549,14 @@ void S_UpdateSounds(void)
         if (I_SoundIsPlaying(c->handle))
         {
             // initialize parameters
-            mobj_t  *origin = c->origin;
+            const mobj_t    *origin = c->origin;
 
             // check non-local sounds for distance clipping or modify their parms
             if (origin && origin != viewplayer->mo)
             {
-                int         sep = NORM_SEP;
-                int         volume = snd_SfxVolume;
-                sfxinfo_t   *sfx = c->sfxinfo;
+                int             sep = NORM_SEP;
+                int             volume = snd_sfxvolume;
+                const sfxinfo_t *sfx = c->sfxinfo;
 
                 if (!sfx)
                     continue;
@@ -580,40 +573,38 @@ void S_UpdateSounds(void)
     }
 }
 
-void S_SetMusicVolume(int volume)
-{
-    I_SetMusicVolume(volume);
-}
-
 void S_LowerMusicVolume(void)
 {
-#if defined(_WIN32)
-    if (!serverMidiPlaying && (musmusictype || midimusictype || Mix_GetMusicType(NULL) == MUS_MID))
-        return;
-#endif
-
-    S_SetMusicVolume(musicVolume * MIX_MAX_VOLUME / 31 / LOWER_MUSIC_VOLUME_FACTOR);
+    I_SetMusicVolume((int)(musicvolume * (MIX_MAX_VOLUME - 1) / 31.0f
+        / (s_lowermenumusic ? LOWER_MUSIC_VOLUME_FACTOR : 1.0f)));
 }
 
-void S_SetSfxVolume(int volume)
+void S_RestoreMusicVolume(void)
 {
-    snd_SfxVolume = volume;
+    I_SetMusicVolume(musicvolume * (MIX_MAX_VOLUME - 1) / 31);
 }
 
-void S_StartMusic(int music_id)
+void S_SetSfxVolume(const int volume)
 {
-    S_ChangeMusic(music_id, false, false, false);
+    snd_sfxvolume = volume;
 }
 
-void S_ChangeMusic(int music_id, dboolean looping, dboolean allowrestart, dboolean mapstart)
+void S_StartMusic(const musicnum_t musicnum)
 {
-    musicinfo_t *music = &S_music[music_id];
+    S_ChangeMusic(musicnum, false, false, false);
+}
+
+void S_ChangeMusic(const musicnum_t musicnum, const bool looping,
+    const bool allowrestart, const bool mapstart)
+{
+    musicinfo_t *music = &s_music[musicnum];
     char        namebuf[9];
     void        *handle;
     int         mapinfomusic;
 
     // current music which should play
-    musinfo.current_item = -1;
+    musinfo.currentitem = -1;
+    s_music[mus_musinfo].lumpnum = -1;
 
     if (nomusic || (mus_playing == music && !allowrestart))
         return;
@@ -621,17 +612,34 @@ void S_ChangeMusic(int music_id, dboolean looping, dboolean allowrestart, dboole
     // shutdown old music
     S_StopMusic();
 
-    M_snprintf(namebuf, sizeof(namebuf), "d_%s", music->name1);
+    if (M_StringStartsWith(music->name1, "d_"))
+        M_StringCopy(namebuf, music->name1, sizeof(namebuf));
+    else
+    {
+        M_snprintf(namebuf, sizeof(namebuf), "h_%s", music->name1);
+
+        if (W_CheckNumForName(namebuf) == -1)
+            M_snprintf(namebuf, sizeof(namebuf), "d_%s", music->name1);
+    }
 
     // get lumpnum if necessary
     if (autosigil)
     {
-        if (music_id == mus_intro || (music_id == mus_inter && gameepisode != 5))
+        if (musicnum == mus_intro)
             music->lumpnum = W_GetLastNumForName(namebuf);
+        else if (musicnum == mus_inter && sigil && sigil2)
+        {
+            if (gameepisode == 5)
+                music->lumpnum = W_GetXNumForName(namebuf, (buckethead ? 4 : 2));
+            else if (gameepisode == 6)
+                music->lumpnum = W_GetXNumForName(namebuf, (thorr && buckethead ? 5 : 3));
+            else
+                music->lumpnum = W_GetLastNumForName(namebuf);
+        }
         else
             music->lumpnum = W_CheckNumForName(namebuf);
     }
-    else if (mapstart && (mapinfomusic = P_GetMapMusic((gameepisode - 1) * 10 + gamemap)) > 0)
+    else if (mapstart && (mapinfomusic = P_GetMapMusic(gameepisode, gamemap)) > 0)
         music->lumpnum = mapinfomusic;
     else if (!music->lumpnum)
         music->lumpnum = W_CheckNumForName(namebuf);
@@ -646,22 +654,20 @@ void S_ChangeMusic(int music_id, dboolean looping, dboolean allowrestart, dboole
         return;
     }
 
-    // load & register it
+    // load and register it
     music->data = W_CacheLumpNum(music->lumpnum);
 
     if (!(handle = I_RegisterSong(music->data, W_LumpLength(music->lumpnum))))
 #if defined(_WIN32)
-        if (!serverMidiPlaying)
+        if (!midimusictype || !windowsmidi)
 #endif
         {
-            char    *filename = M_StringJoin(namebuf, ".mp3", NULL);
-            char    *path = M_TempFile(filename);
+            char    *filename = M_TempFile(DOOMRETRO ".mp3");
 
-            if (W_WriteFile(path, music->data, W_LumpLength(music->lumpnum)))
-                handle = Mix_LoadMUS(path);
+            if (W_WriteFile(filename, music->data, W_LumpLength(music->lumpnum)))
+                handle = Mix_LoadMUS(filename);
 
             free(filename);
-            free(path);
 
             if (!handle)
             {
@@ -679,13 +685,15 @@ void S_ChangeMusic(int music_id, dboolean looping, dboolean allowrestart, dboole
     // Play it
     I_PlaySong(handle, looping);
 
+    S_RestoreMusicVolume();
+
     mus_playing = music;
 
     // [crispy] musinfo.items[0] is reserved for the map's default music
     if (!musinfo.items[0])
     {
         musinfo.items[0] = music->lumpnum;
-        S_music[mus_musinfo].lumpnum = -1;
+        s_music[mus_musinfo].lumpnum = -1;
     }
 }
 
@@ -698,23 +706,24 @@ void S_StopMusic(void)
         I_ResumeSong();
 
     I_StopSong();
-    I_UnRegisterSong(mus_playing->handle);
-    W_ReleaseLumpNum(mus_playing->lumpnum);
+    I_UnregisterSong(mus_playing->handle);
+
+    if (mus_playing->lumpnum >= 0)
+        W_ReleaseLumpNum(mus_playing->lumpnum);
+
     mus_playing->data = NULL;
     mus_playing = NULL;
 }
 
-void S_ChangeMusInfoMusic(int lumpnum, int looping)
+void S_ChangeMusInfoMusic(const int lumpnum, const bool looping)
 {
     musicinfo_t *music;
+    void        *handle;
 
-    if (nomusic)
+    if (nomusic || (mus_playing && mus_playing->lumpnum == lumpnum))
         return;
 
-    if (mus_playing && mus_playing->lumpnum == lumpnum)
-        return;
-
-    music = &S_music[mus_musinfo];
+    music = &s_music[mus_musinfo];
 
     if (music->lumpnum == lumpnum)
         return;
@@ -725,53 +734,73 @@ void S_ChangeMusInfoMusic(int lumpnum, int looping)
     // save lumpnum
     music->lumpnum = lumpnum;
 
-    // load & register it
-    music->data = W_CacheLumpNum(music->lumpnum);
-    music->handle = I_RegisterSong(music->data, W_LumpLength(music->lumpnum));
+    // load and register it
+    music->data = W_CacheLumpNum(lumpnum);
+
+    if (!(handle = I_RegisterSong(music->data, W_LumpLength(lumpnum))))
+#if defined(_WIN32)
+        if (!midimusictype || !windowsmidi)
+#endif
+        {
+            char    *filename = M_TempFile(DOOMRETRO ".mp3");
+
+            if (W_WriteFile(filename, music->data, W_LumpLength(lumpnum)))
+                handle = Mix_LoadMUS(filename);
+
+            free(filename);
+
+            if (!handle)
+                return;
+        }
+
+    music->handle = handle;
+
+    S_RestoreMusicVolume();
 
     // play it
-    I_PlaySong(music->handle, looping);
+    I_PlaySong(handle, looping);
 
     mus_playing = music;
+    M_StringCopy(mus_playing->name1, lumpinfo[lumpnum]->name, sizeof(mus_playing->name1));
 
-    musinfo.current_item = lumpnum;
+    musinfo.currentitem = lumpnum;
 }
 
 //
 // S_ParseMusInfo
 // Parses MUSINFO lump.
 //
-void S_ParseMusInfo(char *mapid)
+void S_ParseMusInfo(const char *lumpname)
 {
     memset(&musinfo, 0, sizeof(musinfo));
-    musinfo.current_item = -1;
+    musinfo.currentitem = -1;
 
-    S_music[NUMMUSIC].lumpnum = -1;
+    s_music[NUMMUSIC].lumpnum = -1;
 
     if (W_CheckNumForName("MUSINFO") >= 0)
     {
-        int inMap = false;
+        bool    inmap = false;
 
-        SC_Open("MUSINFO");
+        SC_Open(W_CheckNumForName("MUSINFO"));
 
         while (SC_GetString())
-            if (inMap || SC_Compare(mapid))
+            if (inmap || SC_Compare(lumpname))
             {
-                unsigned int    num;
+                unsigned int    num = 0;
 
-                if (!inMap)
+                if (!inmap)
                 {
                     SC_GetString();
-                    inMap = true;
+                    inmap = true;
                 }
 
                 if (toupper(sc_String[0]) == 'E' || toupper(sc_String[0]) == 'M')
                     break;
 
                 // Check number in range
-                if (M_StrToInt(sc_String, &num) && num > 0 && num < MAX_MUS_ENTRIES && SC_GetString())
+                if (M_StrToInt(sc_String, (int *)&num) && num > 0 && num < MAX_MUS_ENTRIES && SC_GetString())
                 {
-                    int lumpnum = W_CheckNumForName(sc_String);
+                    const int   lumpnum = W_CheckNumForName(sc_String);
 
                     if (lumpnum >= 0)
                     {
@@ -791,7 +820,7 @@ void MusInfoThinker(mobj_t *thing)
     {
         musinfo.lastmapthing = musinfo.mapthing;
         musinfo.mapthing = thing;
-        musinfo.tics = (leveltime ? 30 : 0);
+        musinfo.tics = (maptime ? 30 : 0);
     }
 }
 
@@ -804,11 +833,11 @@ void T_MAPMusic(void)
         musinfo.tics--;
     else if (musinfo.lastmapthing != musinfo.mapthing)
     {
-        int arraypt = TIDNUM(musinfo.mapthing);
+        const int   arraypt = TIDNUM(musinfo.mapthing);
 
         if (arraypt >= 0 && arraypt < MAX_MUS_ENTRIES)
         {
-            int lumpnum = musinfo.items[arraypt];
+            const int   lumpnum = musinfo.items[arraypt];
 
             if (lumpnum > 0 && lumpnum < numlumps)
                 S_ChangeMusInfoMusic(lumpnum, true);
