@@ -1,137 +1,159 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "doomdef.h"
+#include "i_system.h"
+#include "m_misc.h"
 #include "sounds.h"
 
 //
 // Information about all the music
 //
-musicinfo_t S_music[] =
+musicinfo_t s_music[] =
 {
-    { "",        "",        "",                                "",                                0, 0, 0 },
-    { "e1m1",    "e1m1",    "At Doom's Gate",                  "At Doom's Gate",                  0, 0, 0 },
-    { "e1m2",    "e1m2",    "The Imp's Song",                  "The Imp's Song",                  0, 0, 0 },
-    { "e1m3",    "e1m3",    "Dark Halls",                      "Dark Halls",                      0, 0, 0 },
-    { "e1m4",    "e1m4",    "Kitchen Ace (And Taking Names)",  "Kitchen Ace (And Taking Names)",  0, 0, 0 },
-    { "e1m5",    "e1m5",    "Suspense",                        "Suspense",                        0, 0, 0 },
-    { "e1m6",    "e1m6",    "On The Hunt",                     "On The Hunt",                     0, 0, 0 },
-    { "e1m7",    "e1m7",    "Demons On The Prey",              "Demons On The Prey",              0, 0, 0 },
-    { "e1m8",    "e1m8",    "Sign Of Evil",                    "Sign Of Evil",                    0, 0, 0 },
-    { "e1m9",    "e1m9",    "Hiding The Secrets",              "Hiding The Secrets",              0, 0, 0 },
-    { "e2m1",    "e2m1",    "I Sawed The Demons",              "I Sawed The Demons",              0, 0, 0 },
-    { "e2m2",    "e2m2",    "The Demons From Adrian's Pen",    "The Demons From Adrian's Pen",    0, 0, 0 },
-    { "e2m3",    "e2m3",    "Intermission From DOOM",          "Intermission From DOOM",          0, 0, 0 },
-    { "e2m4",    "e2m4",    "They're Going To Get You",        "They're Going To Get You",        0, 0, 0 },
-    { "e2m5",    "e2m5",    "Demons On The Prey",              "Demons On The Prey",              0, 0, 0 },
-    { "e2m6",    "e2m6",    "Sinister",                        "Sinister",                        0, 0, 0 },
-    { "e2m7",    "e2m7",    "Waltz Of The Demons",             "Waltz Of The Demons",             0, 0, 0 },
-    { "e2m8",    "e2m8",    "Nobody Told Me About id",         "Nobody Told Me About id",         0, 0, 0 },
-    { "e2m9",    "e2m9",    "n/a",                             "n/a",                             0, 0, 0 },
-    { "e3m1",    "e3m1",    "n/a",                             "n/a",                             0, 0, 0 },
-    { "e3m2",    "e3m2",    "Donna To The Rescue",             "Donna To The Rescue",             0, 0, 0 },
-    { "e3m3",    "e3m3",    "Deep Into The Code",              "Deep Into The Code",              0, 0, 0 },
-    { "e3m4",    "e3m4",    "Sign Of Evil",                    "Sign Of Evil",                    0, 0, 0 },
-    { "e3m5",    "e3m5",    "Demons On The Prey",              "Demons On The Prey",              0, 0, 0 },
-    { "e3m6",    "e3m6",    "Suspense",                        "Suspense",                        0, 0, 0 },
-    { "e3m7",    "e3m7",    "Waltz Of The Demons",             "Waltz Of The Demons",             0, 0, 0 },
-    { "e3m8",    "e3m8",    "Facing The Spider",               "Facing The Spider",               0, 0, 0 },
-    { "e3m9",    "e3m9",    "Hiding The Secrets",              "Hiding The Secrets",              0, 0, 0 },
-    { "e5m1",    "e5m1",    "Hate Machine",                    "Romero One Mind Any Weapon",      0, 0, 0 },
-    { "e5m2",    "e5m2",    "You Ain't The Boss Of Me",        "13th Floor",                      0, 0, 0 },
-    { "e5m3",    "e5m3",    "Quell The Beast",                 "Buildor Part 2",                  0, 0, 0 },
-    { "e5m4",    "e5m4",    "Riot Squadron",                   "The Patrolman",                   0, 0, 0 },
-    { "e5m5",    "e5m5",    "Alice",                           "Cold Frost Part 6",               0, 0, 0 },
-    { "e5m6",    "e5m6",    "Besieged City",                   "Melting Man Part 2",              0, 0, 0 },
-    { "e5m7",    "e5m7",    "Watching You",                    "Far 5",                           0, 0, 0 },
-    { "e5m8",    "e5m8",    "Easel",                           "Poseidon 4-6",                    0, 0, 0 },
-    { "e5m9",    "e5m9",    "Adrenaline In The Blood",         "Fastpass",                        0, 0, 0 },
-    { "inter",   "inter",   "Intermission From DOOM",          "Triceratoptron",                  0, 0, 0 },
-    { "intro",   "intro",   "n/a",                             "Eye On Spiral Part 1",            0, 0, 0 },
-    { "bunny",   "bunny",   "Sweet Little Dead Bunny",         "Sweet Little Dead Bunny",         0, 0, 0 },
-    { "victor",  "victor",  "The End Of DOOM",                 "The End Of DOOM",                 0, 0, 0 },
-    { "introa",  "introa",  "n/a",                             "n/a",                             0, 0, 0 },
-    { "runnin",  "runnin",  "Running From Evil",               "Running From Evil",               0, 0, 0 },
-    { "stalks",  "stalks",  "The Healer Stalks",               "The Healer Stalks",               0, 0, 0 },
-    { "countd",  "countd",  "Countdown To Death",              "Countdown To Death",              0, 0, 0 },
-    { "betwee",  "betwee",  "Between Levels",                  "Between Levels",                  0, 0, 0 },
-    { "doom",    "doom",    "DOOM",                            "DOOM",                            0, 0, 0 },
-    { "the_da",  "the_da",  "In The Dark",                     "In The Dark",                     0, 0, 0 },
-    { "shawn",   "shawn",   "Shawn's Got The Shotgun",         "Shawn's Got The Shotgun",         0, 0, 0 },
-    { "ddtblu",  "ddtblu",  "The Dave D. Taylor Blues",        "The Dave D. Taylor Blues",        0, 0, 0 },
-    { "in_cit",  "in_cit",  "Into Sandy's City",               "Into Sandy's City",               0, 0, 0 },
-    { "dead",    "dead",    "The Demon's Dead",                "The Demon's Dead",                0, 0, 0 },
-    { "stlks2",  "stlks2",  "The Healer Stalks",               "The Healer Stalks",               0, 0, 0 },
-    { "theda2",  "theda2",  "In The Dark",                     "In The Dark",                     0, 0, 0 },
-    { "doom2",   "doom2",   "DOOM",                            "DOOM",                            0, 0, 0 },
-    { "ddtbl2",  "ddtbl2",  "The Dave D. Taylor Blues",        "The Dave D. Taylor Blues",        0, 0, 0 },
-    { "runni2",  "runni2",  "Running From Evil",               "Running From Evil",               0, 0, 0 },
-    { "dead2",   "dead2",   "The Demon's Dead",                "The Demon's Dead",                0, 0, 0 },
-    { "stlks3",  "stlks3",  "The Healer Stalks",               "The Healer Stalks",               0, 0, 0 },
-    { "romero",  "romero",  "Waiting For Romero To Play",      "Waiting For Romero To Play",      0, 0, 0 },
-    { "shawn2",  "shawn2",  "Shawn's Got The Shotgun",         "Shawn's Got The Shotgun",         0, 0, 0 },
-    { "messag",  "messag",  "Message For The Arch-vile",       "Message For The Arch-vile",       0, 0, 0 },
-    { "count2",  "count2",  "Countdown To Death",              "Countdown To Death",              0, 0, 0 },
-    { "ddtbl3",  "ddtbl3",  "The Dave D. Taylor Blues",        "The Dave D. Taylor Blues",        0, 0, 0 },
-    { "ampie",   "ampie",   "Bye Bye American Pie",            "Bye Bye American Pie",            0, 0, 0 },
-    { "theda3",  "theda3",  "In The Dark",                     "In The Dark",                     0, 0, 0 },
-    { "adrian",  "adrian",  "Adrian's Asleep",                 "Adrian's Asleep",                 0, 0, 0 },
-    { "messg2",  "messg2",  "Message For The Arch-vile",       "Message For The Arch-vile",       0, 0, 0 },
-    { "romer2",  "romer2",  "Waiting For Romero To Play",      "Waiting For Romero To Play",      0, 0, 0 },
-    { "tense",   "tense",   "Getting Too Tense",               "Getting Too Tense",               0, 0, 0 },
-    { "shawn3",  "shawn3",  "Shawn's Got The Shotgun",         "Shawn's Got The Shotgun",         0, 0, 0 },
-    { "openin",  "openin",  "Opening To Hell",                 "Opening To Hell",                 0, 0, 0 },
-    { "evil",    "evil",    "Evil Incarnate",                  "Evil Incarnate",                  0, 0, 0 },
-    { "ultima",  "ultima",  "The Ultimate Challenge/Conquest", "The Ultimate Challenge/Conquest", 0, 0, 0 },
-    { "read_m",  "read_m",  "n/a",                             "n/a",                             0, 0, 0 },
-    { "dm2ttl",  "dm2ttl",  "n/a",                             "n/a",                             0, 0, 0 },
-    { "dm2int",  "dm2int",  "n/a",                             "n/a",                             0, 0, 0 },
-    { "",        "",        "",                                "",                                0, 0, 0 },
+    { "",        "",        "",                                "",                                ""            },
+    { "e1m1",    "e1m1",    "At DOOM's Gate",                  "At DOOM's Gate",                  ""            },
+    { "e1m2",    "e1m2",    "The Imp's Song",                  "The Imp's Song",                  ""            },
+    { "e1m3",    "e1m3",    "Dark Halls",                      "Dark Halls",                      ""            },
+    { "e1m4",    "e1m4",    "Kitchen Ace (And Taking Names)",  "Kitchen Ace (And Taking Names)",  ""            },
+    { "e1m5",    "e1m5",    "Suspense",                        "Suspense",                        ""            },
+    { "e1m6",    "e1m6",    "On The Hunt",                     "On The Hunt",                     ""            },
+    { "e1m7",    "e1m7",    "Demons On The Prey",              "Demons On The Prey",              ""            },
+    { "e1m8",    "e1m8",    "Sign Of Evil",                    "Sign Of Evil",                    ""            },
+    { "e1m9",    "e1m9",    "Hiding The Secrets",              "Hiding The Secrets",              ""            },
+    { "e2m1",    "e2m1",    "I Sawed The Demons",              "I Sawed The Demons",              ""            },
+    { "e2m2",    "e2m2",    "The Demons From Adrian's Pen",    "The Demons From Adrian's Pen",    ""            },
+    { "e2m3",    "e2m3",    "Intermission From DOOM",          "Intermission From DOOM",          ""            },
+    { "e2m4",    "e2m4",    "They're Going To Get You",        "They're Going To Get You",        ""            },
+    { "e2m5",    "e2m5",    "Demons On The Prey",              "Demons On The Prey",              ""            },
+    { "e2m6",    "e2m6",    "Sinister",                        "Sinister",                        ""            },
+    { "e2m7",    "e2m7",    "Waltz Of The Demons",             "Waltz Of The Demons",             ""            },
+    { "e2m8",    "e2m8",    "Nobody Told Me About id",         "Nobody Told Me About id",         ""            },
+    { "e2m9",    "e2m9",    "n/a",                             "n/a",                             ""            },
+    { "e3m1",    "e3m1",    "n/a",                             "n/a",                             ""            },
+    { "e3m2",    "e3m2",    "Donna To The Rescue",             "Donna To The Rescue",             ""            },
+    { "e3m3",    "e3m3",    "Deep Into The Code",              "Deep Into The Code",              ""            },
+    { "e3m4",    "e3m4",    "Sign Of Evil",                    "Sign Of Evil",                    ""            },
+    { "e3m5",    "e3m5",    "Demons On The Prey",              "Demons On The Prey",              ""            },
+    { "e3m6",    "e3m6",    "Suspense",                        "Suspense",                        ""            },
+    { "e3m7",    "e3m7",    "Waltz Of The Demons",             "Waltz Of The Demons",             ""            },
+    { "e3m8",    "e3m8",    "Facing The Spider",               "Facing The Spider",               ""            },
+    { "e3m9",    "e3m9",    "Hiding The Secrets",              "Hiding The Secrets",              ""            },
+    { "e4m1",    "e4m1",    "n/a",                             "n/a",                             ""            },
+    { "e4m2",    "e4m2",    "n/a",                             "n/a",                             ""            },
+    { "e4m3",    "e4m3",    "n/a",                             "n/a",                             ""            },
+    { "e4m4",    "e4m4",    "n/a",                             "n/a",                             ""            },
+    { "e4m5",    "e4m5",    "n/a",                             "n/a",                             ""            },
+    { "e4m6",    "e4m6",    "n/a",                             "n/a",                             ""            },
+    { "e4m7",    "e4m7",    "n/a",                             "n/a",                             ""            },
+    { "e4m8",    "e4m8",    "n/a",                             "n/a",                             ""            },
+    { "e4m9",    "e4m9",    "n/a",                             "n/a",                             ""            },
+    { "e5m1",    "e5m1",    "Hate Machine",                    "Romero One Mind Any Weapon",      ""            },
+    { "e5m2",    "e5m2",    "You Ain't The Boss Of Me",        "13th Floor",                      ""            },
+    { "e5m3",    "e5m3",    "Quell The Beast",                 "Buildor Part 2",                  ""            },
+    { "e5m4",    "e5m4",    "Riot Squadron",                   "The Patrolman",                   ""            },
+    { "e5m5",    "e5m5",    "Alice",                           "Cold Frost Part 6",               ""            },
+    { "e5m6",    "e5m6",    "Besieged City",                   "Melting Man Part 2",              ""            },
+    { "e5m7",    "e5m7",    "Watching You",                    "Far 5",                           ""            },
+    { "e5m8",    "e5m8",    "Easel",                           "Poseidon 4-6",                    ""            },
+    { "e5m9",    "e5m9",    "Adrenaline In The Blood",         "Fastpass",                        ""            },
+    { "e6m1",    "e6m1",    "Nightmare Overture",              "Umbral Uprising",                 ""            },
+    { "e6m2",    "e6m2",    "Sleep Of Reason",                 "Hellspawn Havoc",                 ""            },
+    { "e6m3",    "e6m3",    "Cathedral Rock",                  "Evil Dead",                       ""            },
+    { "e6m4",    "e6m4",    "Fractures",                       "Pain",                            ""            },
+    { "e6m5",    "e6m5",    "Hexaphobia",                      "Chainsaw Chorus",                 ""            },
+    { "e6m6",    "e6m6",    "Walls Of The Minotaur",           "Incineration Anthem",             ""            },
+    { "e6m7",    "e6m7",    "The Impenetrable Dark",           "Wicked Warpath",                  ""            },
+    { "e6m8",    "e6m8",    "Final Impact",                    "Vitriolic Vendetta",              ""            },
+    { "e6m9",    "e6m9",    "I'm The Doomguy With The Gun",    "Raging Revenant",                 ""            },
+    { "inter",   "inter",   "Intermission From DOOM",          "Triceratoptron",                  "Extirpation" },
+    { "intro",   "intro",   "n/a",                             "Eye On Spiral Part 1",            "Immolator"   },
+    { "bunny",   "bunny",   "Sweet Little Dead Bunny",         "Sweet Little Dead Bunny",         ""            },
+    { "victor",  "victor",  "The End Of DOOM",                 "The End Of DOOM",                 ""            },
+    { "introa",  "introa",  "n/a",                             "n/a",                             ""            },
+    { "runnin",  "runnin",  "Running From Evil",               "Running From Evil",               ""            },
+    { "stalks",  "stalks",  "The Healer Stalks",               "The Healer Stalks",               ""            },
+    { "countd",  "countd",  "Countdown To Death",              "Countdown To Death",              ""            },
+    { "betwee",  "betwee",  "Between Levels",                  "Between Levels",                  ""            },
+    { "doom",    "doom",    "DOOM",                            "DOOM",                            ""            },
+    { "the_da",  "the_da",  "In The Dark",                     "In The Dark",                     ""            },
+    { "shawn",   "shawn",   "Shawn's Got The Shotgun",         "Shawn's Got The Shotgun",         ""            },
+    { "ddtblu",  "ddtblu",  "The Dave D. Taylor Blues",        "The Dave D. Taylor Blues",        ""            },
+    { "in_cit",  "in_cit",  "Into Sandy's City",               "Into Sandy's City",               ""            },
+    { "dead",    "dead",    "The Demon's Dead",                "The Demon's Dead",                ""            },
+    { "stlks2",  "stlks2",  "The Healer Stalks",               "The Healer Stalks",               ""            },
+    { "theda2",  "theda2",  "In The Dark",                     "In The Dark",                     ""            },
+    { "doom2",   "doom2",   "DOOM",                            "DOOM",                            ""            },
+    { "ddtbl2",  "ddtbl2",  "The Dave D. Taylor Blues",        "The Dave D. Taylor Blues",        ""            },
+    { "runni2",  "runni2",  "Running From Evil",               "Running From Evil",               ""            },
+    { "dead2",   "dead2",   "The Demon's Dead",                "The Demon's Dead",                ""            },
+    { "stlks3",  "stlks3",  "The Healer Stalks",               "The Healer Stalks",               ""            },
+    { "romero",  "romero",  "Waiting For Romero To Play",      "Waiting For Romero To Play",      ""            },
+    { "shawn2",  "shawn2",  "Shawn's Got The Shotgun",         "Shawn's Got The Shotgun",         ""            },
+    { "messag",  "messag",  "Message For The Arch-vile",       "Message For The Arch-vile",       ""            },
+    { "count2",  "count2",  "Countdown To Death",              "Countdown To Death",              ""            },
+    { "ddtbl3",  "ddtbl3",  "The Dave D. Taylor Blues",        "The Dave D. Taylor Blues",        ""            },
+    { "ampie",   "ampie",   "Bye Bye American Pie",            "Bye Bye American Pie",            ""            },
+    { "theda3",  "theda3",  "In The Dark",                     "In The Dark",                     ""            },
+    { "adrian",  "adrian",  "Adrian's Asleep",                 "Adrian's Asleep",                 ""            },
+    { "messg2",  "messg2",  "Message For The Arch-vile",       "Message For The Arch-vile",       ""            },
+    { "romer2",  "romer2",  "Waiting For Romero To Play",      "Waiting For Romero To Play",      ""            },
+    { "tense",   "tense",   "Getting Too Tense",               "Getting Too Tense",               ""            },
+    { "shawn3",  "shawn3",  "Shawn's Got The Shotgun",         "Shawn's Got The Shotgun",         ""            },
+    { "openin",  "openin",  "Opening To Hell",                 "Opening To Hell",                 ""            },
+    { "evil",    "evil",    "Evil Incarnate",                  "Evil Incarnate",                  ""            },
+    { "ultima",  "ultima",  "The Ultimate Challenge/Conquest", "The Ultimate Challenge/Conquest", ""            },
+    { "read_m",  "read_m",  "n/a",                             "n/a",                             ""            },
+    { "dm2ttl",  "dm2ttl",  "n/a",                             "n/a",                             ""            },
+    { "dm2int",  "dm2int",  "n/a",                             "n/a",                             ""            },
+    { "",        "",        "",                                "",                                ""            },
 
     // custom music from MUSINFO lump
-    { "musinfo", "musinfo", "n/a",                             "n/a",                             0, 0, 0 }
+    { "musinfo", "musinfo", "n/a",                             "n/a",                             ""            }
 };
 
 //
 // Information about all the SFX
 //
-sfxinfo_t S_sfx[] =
+sfxinfo_t original_s_sfx[NUMSFX] =
 {
-    // S_sfx[0] needs to be a dummy for odd reasons.
+    // s_sfx[0] needs to be a dummy for odd reasons.
     { "none",   "none",   sg_none,     0, -1 },
     { "pistol", "pistol", sg_none,    64, -1 },
     { "shotgn", "shotgn", sg_none,    64, -1 },
@@ -254,6 +276,7 @@ sfxinfo_t S_sfx[] =
     { "gibdth", "gibdth", sg_none,    60, -1 },
 
     { "scrsht", "scrsht", sg_none,     0, -1 },
+    { "consol", "consol", sg_none,     0, -1 },
 
     [500] = { "fre000", "fre000", sg_none, 127, -1 },
     [501] = { "fre001", "fre001", sg_none, 127, -1 },
@@ -456,3 +479,102 @@ sfxinfo_t S_sfx[] =
     [698] = { "fre198", "fre198", sg_none, 127, -1 },
     [699] = { "fre199", "fre199", sg_none, 127, -1 }
 };
+
+// DSDHacked
+sfxinfo_t   *s_sfx;
+int         numsfx;
+static char **deh_soundnames;
+static int  deh_soundnames_size;
+static byte *sfx_state;
+
+void InitSFX(void)
+{
+    s_sfx = original_s_sfx;
+    numsfx = NUMSFX;
+    deh_soundnames_size = numsfx + 1;
+    deh_soundnames = I_Malloc(deh_soundnames_size * sizeof(*deh_soundnames));
+
+    for (int i = 1; i < numsfx; i++)
+        deh_soundnames[i] = (*s_sfx[i].name1 ? M_StringDuplicate(s_sfx[i].name1) : NULL);
+
+    deh_soundnames[0] = NULL;
+    deh_soundnames[numsfx] = NULL;
+    sfx_state = calloc(numsfx, sizeof(*sfx_state));
+}
+
+void FreeSFX(void)
+{
+    for (int i = 1; i < deh_soundnames_size; i++)
+        if (deh_soundnames[i])
+            free(deh_soundnames[i]);
+
+    free(deh_soundnames);
+    free(sfx_state);
+}
+
+void dsdh_EnsureSFXCapacity(const int limit)
+{
+    static int  first_allocation = true;
+
+    while (limit >= numsfx)
+    {
+        const int   old_numsfx = numsfx;
+
+        numsfx *= 2;
+
+        if (first_allocation)
+        {
+            first_allocation = false;
+            s_sfx = I_Malloc(numsfx * sizeof(*s_sfx));
+            memcpy(s_sfx, original_s_sfx, old_numsfx * sizeof(*s_sfx));
+        }
+        else
+            s_sfx = I_Realloc(s_sfx, numsfx * sizeof(*s_sfx));
+
+        memset(s_sfx + old_numsfx, 0, (numsfx - old_numsfx) * sizeof(*s_sfx));
+
+        sfx_state = I_Realloc(sfx_state, numsfx * sizeof(*sfx_state));
+        memset(sfx_state + old_numsfx, 0,
+            (numsfx - old_numsfx) * sizeof(*sfx_state));
+
+        for (int i = old_numsfx; i < numsfx; i++)
+        {
+            s_sfx[i].priority = 127;
+            s_sfx[i].lumpnum = -1;
+        }
+    }
+}
+
+int dsdh_GetDehSFXIndex(const char *key, size_t length)
+{
+    for (int i = 1; i < numsfx; i++)
+        if (*s_sfx[i].name1
+            && strlen(s_sfx[i].name1) == length
+            && !strncasecmp(s_sfx[i].name1, key, length)
+            && !sfx_state[i])
+        {
+            sfx_state[i] = true;    // sfx has been edited
+            return i;
+        }
+
+    return -1;
+}
+
+int dsdh_GetOriginalSFXIndex(const char *key)
+{
+    int limit;
+
+    for (int i = 1; deh_soundnames[i]; i++)
+        if (!strncasecmp(deh_soundnames[i], key, 6))
+            return i;
+
+    // is it a number?
+    for (const char *c = key; *c; c++)
+        if (!isdigit(*c))
+            return -1;
+
+    limit = strtol(key, NULL, 10);
+    dsdh_EnsureSFXCapacity(limit);
+
+    return limit;
+}
