@@ -1,42 +1,40 @@
 /*
-========================================================================
+==============================================================================
 
-                           D O O M  R e t r o
-         The classic, refined DOOM source port. For Windows PC.
+                                 DOOM Retro
+           The classic, refined DOOM source port. For Windows PC.
 
-========================================================================
+==============================================================================
 
-  Copyright © 1993-2021 by id Software LLC, a ZeniMax Media company.
-  Copyright © 2013-2021 by Brad Harding <mailto:brad@doomretro.com>.
+    Copyright © 1993-2024 by id Software LLC, a ZeniMax Media company.
+    Copyright © 2013-2024 by Brad Harding <mailto:brad@doomretro.com>.
 
-  DOOM Retro is a fork of Chocolate DOOM. For a list of credits, see
-  <https://github.com/bradharding/doomretro/wiki/CREDITS>.
+    This file is a part of DOOM Retro.
 
-  This file is a part of DOOM Retro.
+    DOOM Retro is free software: you can redistribute it and/or modify it
+    under the terms of the GNU General Public License as published by the
+    Free Software Foundation, either version 3 of the license, or (at your
+    option) any later version.
 
-  DOOM Retro is free software: you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation, either version 3 of the License, or (at your
-  option) any later version.
+    DOOM Retro is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+    General Public License for more details.
 
-  DOOM Retro is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
+    You should have received a copy of the GNU General Public License
+    along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public License
-  along with DOOM Retro. If not, see <https://www.gnu.org/licenses/>.
+    DOOM is a registered trademark of id Software LLC, a ZeniMax Media
+    company, in the US and/or other countries, and is used without
+    permission. All other trademarks are the property of their respective
+    holders. DOOM Retro is in no way affiliated with nor endorsed by
+    id Software.
 
-  DOOM is a registered trademark of id Software LLC, a ZeniMax Media
-  company, in the US and/or other countries, and is used without
-  permission. All other trademarks are the property of their respective
-  holders. DOOM Retro is in no way affiliated with nor endorsed by
-  id Software.
-
-========================================================================
+==============================================================================
 */
 
 #include "c_console.h"
+#include "d_deh.h"
 #include "d_main.h"
 #include "doomstat.h"
 #include "g_game.h"
@@ -69,14 +67,14 @@
 #define NUMMAPS     9
 
 // GLOBAL LOCATIONS
-#define WI_TITLEY   2
+#define WI_TITLEY   12
 
 // SINGLE-PLAYER STUFF
 #define SP_STATSX   50
-#define SP_STATSY   50
+#define SP_STATSY   58
 
 #define SP_TIMEX    16
-#define SP_TIMEY    (VANILLAHEIGHT - 32)
+#define SP_TIMEY    (VANILLAHEIGHT - 25)
 
 typedef enum
 {
@@ -169,46 +167,44 @@ static point_t lnodes[][NUMMAPS] =
 // Using patches saves a lot of space,
 //  as they replace 320x200 full screen frames.
 //
-#define ANIM(type, period, nanims, x, y, nexttic)   { (type), (period), (nanims), { (x), (y) }, (nexttic) }
-
 static anim_t epsd0animinfo[] =
 {
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3, 224, 104, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3, 184, 160, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3, 112, 136, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3,  72, 112, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3,  88,  96, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3,  64,  48, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3, 192,  40, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3, 136,  16, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3,  80,  16, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3,  64,  24, 0)
+    { ANIM_ALWAYS, TICRATE / 3, 3, { 224, 104 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, { 184, 160 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, { 112, 136 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, {  72, 112 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, {  88,  96 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, {  64,  48 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, { 192,  40 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, { 136,  16 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, {  80,  16 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, {  64,  24 }, 0 }
 };
 
 static anim_t epsd1animinfo[] =
 {
-    ANIM(ANIM_LEVEL, TICRATE / 3, 1, 128, 136, 1),
-    ANIM(ANIM_LEVEL, TICRATE / 3, 1, 128, 136, 2),
-    ANIM(ANIM_LEVEL, TICRATE / 3, 1, 128, 136, 3),
-    ANIM(ANIM_LEVEL, TICRATE / 3, 1, 128, 136, 4),
-    ANIM(ANIM_LEVEL, TICRATE / 3, 1, 128, 136, 5),
-    ANIM(ANIM_LEVEL, TICRATE / 3, 1, 128, 136, 6),
-    ANIM(ANIM_LEVEL, TICRATE / 3, 1, 128, 136, 7),
-    ANIM(ANIM_LEVEL, TICRATE / 3, 3, 192, 144, 8),
-    ANIM(ANIM_LEVEL, TICRATE / 3, 1, 128, 136, 8)
+    { ANIM_LEVEL, TICRATE / 3, 1, { 128, 136 }, 1 },
+    { ANIM_LEVEL, TICRATE / 3, 1, { 128, 136 }, 2 },
+    { ANIM_LEVEL, TICRATE / 3, 1, { 128, 136 }, 3 },
+    { ANIM_LEVEL, TICRATE / 3, 1, { 128, 136 }, 4 },
+    { ANIM_LEVEL, TICRATE / 3, 1, { 128, 136 }, 5 },
+    { ANIM_LEVEL, TICRATE / 3, 1, { 128, 136 }, 6 },
+    { ANIM_LEVEL, TICRATE / 3, 1, { 128, 136 }, 7 },
+    { ANIM_LEVEL, TICRATE / 3, 3, { 192, 144 }, 8 },
+    { ANIM_LEVEL, TICRATE / 3, 1, { 128, 136 }, 8 }
 };
 
 static anim_t epsd2animinfo[] =
 {
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3, 104, 168, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3,  40, 136, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3, 160,  96, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3, 104,  80, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 3, 3, 120,  32, 0),
-    ANIM(ANIM_ALWAYS, TICRATE / 4, 3,  40,   0, 0)
+    { ANIM_ALWAYS, TICRATE / 3, 3, { 104, 168 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, {  40, 136 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, { 160,  96 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, { 104,  80 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 3, 3, { 120,  32 }, 0 },
+    { ANIM_ALWAYS, TICRATE / 4, 3, {  40,   0 }, 0 }
 };
 
-static int NUMANIMS[] =
+static int numanims[] =
 {
     arrlen(epsd0animinfo),
     arrlen(epsd1animinfo),
@@ -234,7 +230,7 @@ static anim_t *anims[] =
 #define SHOWNEXTLOCDELAY    4
 
 // used to accelerate or skip a stage
-dboolean                acceleratestage;
+bool                    acceleratestage;
 
 // specifies current state
 static stateenum_t      state;
@@ -256,7 +252,7 @@ static int              cnt_par;
 static int              cnt_pause;
 
 // # of commercial levels
-static int              NUMCMAPS;
+static int              numcmaps;
 
 //
 // GRAPHICS
@@ -296,9 +292,8 @@ static patch_t          *sucks;
 // Name graphics of each level (centered)
 static patch_t          **lnames;
 
-//
-// CODE
-//
+static int              enterpic;
+static int              exitpic;
 
 // slam background
 static void WI_SlamBackground(void)
@@ -309,11 +304,11 @@ static void WI_SlamBackground(void)
 // [BH] Draws character of "<Levelname>"
 static void WI_DrawWILVchar(int x, int y, int i)
 {
-    int w = (int)strlen(wilv[i]) / 13;
+    const int   width = (int)strlen(wilv[i]) / 13;
 
     for (int y1 = 0; y1 < 13; y1++)
-        for (int x1 = 0; x1 < w; x1++)
-            V_DrawPixel(x + x1, y + y1, (int)wilv[i][y1 * w + x1], true);
+        for (int x1 = 0; x1 < width; x1++)
+            V_DrawPixel(x + x1, y + y1, (int)wilv[i][y1 * width + x1], false, true);
 }
 
 static char mapname[128];
@@ -321,42 +316,45 @@ static char nextmapname[128];
 
 static const int chartoi[130] =
 {
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 000 - 009
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 010 - 019
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 020 - 029
-    -1, -1, -1, 27, -1, -1, -1, -1, -1, 28, // 030 - 039
-    -1, -1, -1, -1, -1, 29, 26, 30, 31, 32, // 040 - 049
-    33, 34, 35, 36, 37, 38, 39, 40, -1, -1, // 050 - 059
-    -1, -1, -1, -1, -1,  0,  1,  2,  3,  4, // 060 - 069
-     5,  6,  7,  8,  9, 10, 11, 12, 13, 14, // 070 - 079
-    15, 16, 17, 18, 19, 20, 21, 22, 23, 24, // 080 - 089
-    25, -1, -1, -1, -1, -1, -1,  0,  1,  2, // 090 - 099
-     3,  4,  5,  6,  7,  8,  9, 10, 11, 12, // 100 - 109
-    13, 14, 15, 16, 17, 18, 19, 20, 21, 22, // 110 - 119
-    23, 24, 25, -1, -1, -1, -1, -1, -1, -1  // 120 - 129
+    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 000 to 009
+    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 010 to 019
+    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 020 to 029
+    -1, -1, -1, 27, -1, -1, -1, -1, -1, 28, // 030 to 039
+    -1, -1, -1, -1, -1, 29, 26, 30, 31, 32, // 040 to 049
+    33, 34, 35, 36, 37, 38, 39, 40, -1, -1, // 050 to 059
+    -1, -1, -1, -1, -1,  0,  1,  2,  3,  4, // 060 to 069
+     5,  6,  7,  8,  9, 10, 11, 12, 13, 14, // 070 to 079
+    15, 16, 17, 18, 19, 20, 21, 22, 23, 24, // 080 to 089
+    25, -1, -1, -1, -1, -1, -1,  0,  1,  2, // 090 to 099
+     3,  4,  5,  6,  7,  8,  9, 10, 11, 12, // 100 to 109
+    13, 14, 15, 16, 17, 18, 19, 20, 21, 22, // 110 to 119
+    23, 24, 25, -1, -1, -1, -1, -1, -1, -1  // 120 to 129
 };
 
 static void WI_DrawWILV(int y, char *str)
 {
-    int len = (int)strlen(str);
-    int w = 0;
-    int x;
+    const int   len = (int)strlen(str);
+    int         width = 0;
+    int         x;
 
-    for (int i = 0; i < len; i++)
+    for (int i = 0, j; i < len; i++)
     {
-        int j = chartoi[(int)str[i]];
-
-        w += (j == -1 ? 6 : ((int)strlen(wilv[j]) / 13 - 2));
-    }
-
-    x = (VANILLAWIDTH - w - 1) / 2;
-
-    for (int i = 0; i < len; i++)
-    {
-        int j = chartoi[(int)str[i]];
-
         if (str[i] == '\'' && (!i || str[i - 1] == ' '))
             j = 41;
+        else
+            j = chartoi[(int)str[i]];
+
+        width += (j == -1 ? 6 : ((int)strlen(wilv[j]) / 13 - 2));
+    }
+
+    x = (VANILLAWIDTH - width) / 2 + 1;
+
+    for (int i = 0, j; i < len; i++)
+    {
+        if (str[i] == '\'' && (!i || str[i - 1] == ' '))
+            j = 41;
+        else
+            j = chartoi[(int)str[i]];
 
         if (j == -1)
             x += 6;
@@ -368,23 +366,23 @@ static void WI_DrawWILV(int y, char *str)
     }
 }
 
-// Draws "<Levelname> Finished!"
+// Draws "<LevelName> Finished!"
 static void WI_DrawLF(void)
 {
-    int x = (VANILLAWIDTH - SHORT(finished->width)) / 2;
-    int y = WI_TITLEY;
-    int titlepatch = P_GetMapTitlePatch(wbs->epsd * 10 + wbs->last + 1);
+    const int   x = (VANILLAWIDTH - SHORT(finished->width)) / 2 + 1;
+    int         y = WI_TITLEY;
+    const int   titlepatch = P_GetMapTitlePatch(wbs->epsd + 1, wbs->last + 1);
 
     // draw <LevelName>
     if (titlepatch > 0)
     {
-        patch_t *patch = W_CacheLumpNum(titlepatch);
-        short   height = SHORT(patch->height);
+        patch_t     *patch = W_CacheLumpNum(titlepatch);
+        const short height = SHORT(patch->height);
 
         if (height < VANILLAHEIGHT)
-            V_DrawPatchWithShadow((VANILLAWIDTH - SHORT(patch->width)) / 2 + 1, y + 1, patch, false);
+            V_DrawMenuPatch((VANILLAWIDTH - SHORT(patch->width)) / 2 + 1, y + 1, patch, false, VANILLAWIDTH);
         else
-            V_DrawPagePatch(patch);
+            V_DrawPagePatch(0, patch);
 
         y += height + 2;
     }
@@ -397,15 +395,15 @@ static void WI_DrawLF(void)
         else
             M_snprintf(name, sizeof(name), "WILV%i%i", wbs->epsd, wbs->last);
 
-        if (W_CheckMultipleLumps(name) > 1 && !nerve)
+        if (W_GetNumLumps(name) > 1 && !nerve)
         {
-            patch_t *patch = lnames[wbs->last];
-            short   height = SHORT(patch->height);
+            patch_t     *patch = lnames[wbs->last];
+            const short height = SHORT(patch->height);
 
             if (height < VANILLAHEIGHT)
-                V_DrawPatchWithShadow((VANILLAWIDTH - SHORT(patch->width)) / 2 + 1, y + 1, patch, false);
+                V_DrawMenuPatch((VANILLAWIDTH - SHORT(patch->width)) / 2 + 1, y + 1, patch, false, VANILLAWIDTH);
             else
-                V_DrawPagePatch(patch);
+                V_DrawPagePatch(0, patch);
 
             y += height + 2;
         }
@@ -420,31 +418,36 @@ static void WI_DrawLF(void)
         y = WI_TITLEY + 24;
 
     // draw "Finished!"
-    V_DrawPatchWithShadow(x + 1, y + 1, finished, false);
+    if (SHORT(finished->height) < VANILLAHEIGHT)
+        V_DrawMenuPatch(x + 1, y + 1, finished, false, VANILLAWIDTH);
+    else
+        V_DrawPagePatch(0, finished);
 }
 
 // Draws "Entering <LevelName>"
 static void WI_DrawEL(void)
 {
-    int x = (VANILLAWIDTH - SHORT(entering->width)) / 2;
-    int y = WI_TITLEY;
-    int titlepatch = P_GetMapTitlePatch(wbs->epsd * 10 + wbs->next + 1);
+    const int   x = (VANILLAWIDTH - SHORT(entering->width)) / 2 + 1;
+    int         y = WI_TITLEY;
+    const int   titlepatch = P_GetMapTitlePatch(wbs->epsd + 1, wbs->next + 1);
 
     // draw "Entering"
-    V_DrawPatchWithShadow(x + 1, y + 1, entering, false);
+    if (SHORT(entering->height) < VANILLAHEIGHT)
+        V_DrawMenuPatch(x + 1, y + 1, entering, false, VANILLAWIDTH);
+    else
+        V_DrawPagePatch(0, entering);
 
-    // draw level
-    y += SHORT(entering->height) + 2;
+    // draw "<LevelName>"
+    y += SHORT(entering->height) + 4;
 
     if (titlepatch > 0)
     {
         patch_t *patch = W_CacheLumpNum(titlepatch);
-        short   height = SHORT(patch->height);
 
-        if (height < VANILLAHEIGHT)
-            V_DrawPatchWithShadow((VANILLAWIDTH - SHORT(patch->width)) / 2 + 1, y + 1, patch, false);
+        if (SHORT(patch->height) < VANILLAHEIGHT)
+            V_DrawMenuPatch((VANILLAWIDTH - SHORT(patch->width)) / 2 + 1, y + 1, patch, false, VANILLAWIDTH);
         else
-            V_DrawPagePatch(patch);
+            V_DrawPagePatch(0, patch);
     }
     else
     {
@@ -455,15 +458,15 @@ static void WI_DrawEL(void)
         else
             M_snprintf(name, sizeof(name), "WILV%i%i", wbs->epsd, wbs->next);
 
-        if (W_CheckMultipleLumps(name) > 1 && !nerve)
+        if (W_GetNumLumps(name) > 1 && !nerve)
         {
-            patch_t *patch = lnames[wbs->next];
-            short   height = SHORT(patch->height);
+            patch_t     *patch = lnames[wbs->next];
+            const short height = SHORT(patch->height);
 
             if (height < VANILLAHEIGHT)
-                V_DrawPatchWithShadow((VANILLAWIDTH - SHORT(patch->width)) / 2 + 1, y + 1, patch, false);
+                V_DrawMenuPatch((VANILLAWIDTH - SHORT(patch->width)) / 2 + 1, y + 1, patch, false, VANILLAWIDTH);
             else
-                V_DrawPagePatch(patch);
+                V_DrawPagePatch(0, patch);
         }
         else
             WI_DrawWILV(y, nextmapname);
@@ -472,15 +475,15 @@ static void WI_DrawEL(void)
 
 static void WI_DrawOnLnode(int n, patch_t *c[])
 {
-    dboolean    fits = false;
-    int         i = 0;
+    bool    fits = false;
+    int     i = 0;
 
     do
     {
-        int left = lnodes[wbs->epsd][n].x - SHORT(c[i]->leftoffset);
-        int top = lnodes[wbs->epsd][n].y - SHORT(c[i]->topoffset);
-        int right = left + SHORT(c[i]->width);
-        int bottom = top + SHORT(c[i]->height);
+        const int   left = lnodes[wbs->epsd][n].x - SHORT(c[i]->leftoffset);
+        const int   top = lnodes[wbs->epsd][n].y - SHORT(c[i]->topoffset);
+        const int   right = left + SHORT(c[i]->width);
+        const int   bottom = top + SHORT(c[i]->height);
 
         if (left >= 0 && right < VANILLAWIDTH && top >= 0 && bottom < VANILLAHEIGHT)
             fits = true;
@@ -490,33 +493,39 @@ static void WI_DrawOnLnode(int n, patch_t *c[])
 
     if (fits && i < 2)
     {
-        if (c[i] == *splat)
+        if (c[i] == splat[0] || c[i] == splat[1])
             V_DrawTranslucentNoGreenPatch(lnodes[wbs->epsd][n].x, lnodes[wbs->epsd][n].y, c[i]);
-        else if (c[i] == *yah)
+        else if (c[i] == yah[0] || c[i] == yah[1])
             V_DrawNoGreenPatchWithShadow(lnodes[wbs->epsd][n].x + 1, lnodes[wbs->epsd][n].y + 1, c[i]);
         else
             V_DrawPatch(lnodes[wbs->epsd][n].x, lnodes[wbs->epsd][n].y, 0, c[i]);
     }
 }
 
-static void WI_InitAnimatedBack(void)
+static void WI_InitAnimatedBack(bool firstcall)
 {
+    if (exitpic > 0 || (enterpic > 0 && entering))
+        return;
+
     if (gamemode == commercial)
         return;
 
     if (wbs->epsd > 2)
         return;
 
-    for (int i = 0; i < NUMANIMS[wbs->epsd]; i++)
+    for (int i = 0; i < numanims[wbs->epsd]; i++)
     {
         anim_t  *a = &anims[wbs->epsd][i];
 
         // init variables
-        a->ctr = -1;
+        // [JN] Do not reset animation timers upon switching to "Entering" state
+        // (WI_initShowNextLoc). Fixes notable blinking of Tower of Babel drawing.
+        if (firstcall)
+            a->ctr = -1;
 
         // specify the next time to draw it
         if (a->type == ANIM_ALWAYS)
-            a->nexttic = bcnt + 1 + (M_Random() % a->period);
+            a->nexttic = bcnt + 1 + M_Random() % a->period;
         else if (a->type == ANIM_LEVEL)
             a->nexttic = bcnt + 1;
     }
@@ -524,13 +533,16 @@ static void WI_InitAnimatedBack(void)
 
 static void WI_UpdateAnimatedBack(void)
 {
+    if (exitpic > 0 || (enterpic > 0 && state != StatCount))
+        return;
+
     if (gamemode == commercial)
         return;
 
     if (wbs->epsd > 2)
         return;
 
-    for (int i = 0; i < NUMANIMS[wbs->epsd]; i++)
+    for (int i = 0; i < numanims[wbs->epsd]; i++)
     {
         anim_t  *a = &anims[wbs->epsd][i];
 
@@ -565,13 +577,16 @@ static void WI_DrawAnimatedBack(void)
 {
     anim_t  *a;
 
+    if (exitpic > 0 || (enterpic > 0 && state != StatCount))
+        return;
+
     if (gamemode == commercial)
         return;
 
     if (wbs->epsd > 2)
         return;
 
-    for (int i = 0; i < NUMANIMS[wbs->epsd]; i++)
+    for (int i = 0; i < numanims[wbs->epsd]; i++)
     {
         a = &anims[wbs->epsd][i];
 
@@ -595,45 +610,39 @@ static void WI_DrawAnimatedBack(void)
 //
 static int WI_DrawNum(int x, int y, int n, int digits)
 {
-    // if non-number, do not draw it
-    if (n == 1994)
-        return 0;
-    else
+    const int   fontwidth = SHORT(num[0]->width);
+
+    if (digits < 0)
     {
-        int fontwidth = SHORT(num[0]->width);
-
-        if (digits < 0)
+        if (!n)
+            // make variable-length zeros 1 digit long
+            digits = 1;
+        else
         {
-            if (!n)
-                // make variable-length zeros 1 digit long
-                digits = 1;
-            else
+            int temp = n;
+
+            // figure out # of digits in #
+            digits = 0;
+
+            while (temp)
             {
-                int temp = n;
-
-                // figure out # of digits in #
-                digits = 0;
-
-                while (temp)
-                {
-                    temp /= 10;
-                    digits++;
-                }
+                temp /= 10;
+                digits++;
             }
         }
-
-        // draw the new number
-        while (digits--)
-        {
-            x -= fontwidth;
-            x += 2 * (n % 10 == 1);
-            V_DrawPatchWithShadow(x + 1, y + 1, num[n % 10], true);
-            x -= 2 * (n % 10 == 1);
-            n /= 10;
-        }
-
-        return x;
     }
+
+    // draw the new number
+    while (digits--)
+    {
+        x -= fontwidth;
+        x += 2 * (n % 10 == 1);
+        V_DrawMenuPatch(x + 1, y + 1, num[n % 10], false, VANILLAWIDTH);
+        x -= 2 * (n % 10 == 1);
+        n /= 10;
+    }
+
+    return x;
 }
 
 static void WI_DrawPercent(int x, int y, int p)
@@ -641,7 +650,7 @@ static void WI_DrawPercent(int x, int y, int p)
     if (p < 0)
         return;
 
-    V_DrawPatchWithShadow(x + 1, y + 1, percent, false);
+    V_DrawMenuPatch(x + 1, y + 1, percent, false, VANILLAWIDTH);
     WI_DrawNum(x, y, p, -1);
 }
 
@@ -656,7 +665,9 @@ static void WI_DrawTime(int x, int y, int t)
 
     x += (SHORT(num[0]->width) - 11) * 4;
 
-    if (t <= 61 * 59)
+    if (sucktime && t > sucktime * 61 * 59)
+        V_DrawMenuPatch(SP_TIMEX + SHORT(timepatch->width) + 12, y + 1, sucks, false, VANILLAWIDTH);
+    else
     {
         int div = 1;
 
@@ -667,15 +678,12 @@ static void WI_DrawTime(int x, int y, int t)
 
             // draw
             if (div == 60 || t / div)
-                V_DrawPatchWithShadow(x + 1, y + 1, colon, true);
+                V_DrawMenuPatch(x + 1, y + 1, colon, false, VANILLAWIDTH);
         } while (t / div);
 
         if (t < 60)
             WI_DrawNum(x, y, 0, 2);
     }
-    else
-        // "sucks"
-        V_DrawPatchWithShadow(x + 13 - SHORT(sucks->width), y + 1, sucks, false);
 }
 
 static void WI_UnloadData(void);
@@ -702,7 +710,7 @@ static void WI_UpdateNoState(void)
         G_WorldDone();
 }
 
-static dboolean snl_pointeron;
+static bool snl_pointeron;
 
 static void WI_InitShowNextLoc(void)
 {
@@ -718,7 +726,7 @@ static void WI_InitShowNextLoc(void)
 
     D_FadeScreen(false);
 
-    WI_InitAnimatedBack();
+    WI_InitAnimatedBack(false);
 }
 
 static void WI_UpdateShowNextLoc(void)
@@ -733,7 +741,7 @@ static void WI_UpdateShowNextLoc(void)
 
 static void WI_DrawShowNextLoc(void)
 {
-    if (P_GetMapEndGame(gamemap) || P_GetMapEndCast(gamemap))
+    if (P_GetMapEndGame(gameepisode, gamemap) || P_GetMapEndCast(gameepisode, gamemap))
         return;
 
     WI_SlamBackground();
@@ -741,6 +749,12 @@ static void WI_DrawShowNextLoc(void)
     // draw animated background
     WI_DrawAnimatedBack();
 
+    if (exitpic > 0 || (enterpic > 0 && state != StatCount))
+    {
+        WI_DrawEL();
+        return;
+    }
+
     if (gamemode != commercial)
     {
         int last;
@@ -784,7 +798,9 @@ static int  sp_state;
 
 static void WI_InitStats(void)
 {
-    const int   tabs[3] = { 100, 0, 0 };
+    const int   tabs[MAXTABS] = { 85 };
+    char        *temp1;
+    char        *temp2;
 
     state = StatCount;
     acceleratestage = false;
@@ -796,31 +812,67 @@ static void WI_InitStats(void)
     cnt_par = -1;
     cnt_pause = TICRATE;
 
-    if (M_StringCompare(playername, playername_default))
-        C_PlayerMessage("You finished " ITALICS("%s%s"),
-            mapname, (ispunctuation(mapname[strlen(mapname) - 1]) ? "" : "!"));
+    if (M_StringCompare(maptitle, mapnumandtitle))
+        C_PlayerMessage("%s finished %s!",
+            (M_StringCompare(playername, playername_default) ? "You" : playername), mapname);
     else
         C_PlayerMessage("%s finished " ITALICS("%s%s"),
-            playername, mapname, (ispunctuation(mapname[strlen(mapname) - 1]) ? "" : "!"));
+            (M_StringCompare(playername, playername_default) ? "you" : playername), mapname,
+            (ispunctuation(mapname[strlen(mapname) - 1]) ? "" : "!"));
 
-    C_TabbedOutput(tabs, "Kills\t" BOLD("%i%%"), (wbs->skills * 100) / wbs->maxkills);
-    C_TabbedOutput(tabs, "Items\t" BOLD("%i%%"), (wbs->sitems * 100) / wbs->maxitems);
+    temp1 = commify(wbs->skills);
+    temp2 = commify(wbs->maxkills);
+    C_TabbedOutput(tabs, "Kills\t%s of %s (%i%%)",
+        temp1, temp2, (wbs->skills * 100) / wbs->maxkills);
+    free(temp1);
+    free(temp2);
 
-    if (totalsecret)
-        C_TabbedOutput(tabs, "Secrets\t" BOLD("%i%%"), (wbs->ssecret * 100) / wbs->maxsecret);
+    temp1 = commify(wbs->sitems);
+    temp2 = commify(wbs->maxitems);
+    C_TabbedOutput(tabs, "Items\t%s of %s (%i%%)",
+        temp1, temp2, (wbs->sitems * 100) / wbs->maxitems);
+    free(temp1);
+    free(temp2);
 
-    C_TabbedOutput(tabs, "Time\t" BOLD("%02i:%02i"), wbs->stime / TICRATE / 60, wbs->stime / TICRATE % 60);
+    if (totalsecrets)
+    {
+        temp1 = commify(wbs->ssecret);
+        temp2 = commify(wbs->maxsecret);
+        C_TabbedOutput(tabs, "Secrets\t%s of %s (%i%%)",
+            temp1, temp2, (wbs->ssecret * 100) / wbs->maxsecret);
+        free(temp1);
+        free(temp2);
+    }
+
+    if (sucktime && wbs->stime / TICRATE > sucktime * 61 * 59)
+        C_TabbedOutput(tabs, "Time\t%s", s_STSTR_SUCKS);
+    else
+        C_TabbedOutput(tabs, "Time\t" MONOSPACED("%02i") ":" MONOSPACED("%02i"),
+            wbs->stime / TICRATE / 60, wbs->stime / TICRATE % 60);
 
     if (wbs->partime)
-        C_TabbedOutput(tabs, "Par time\t" BOLD("%02i:%02i"), wbs->partime / TICRATE / 60, wbs->partime / TICRATE % 60);
+        C_TabbedOutput(tabs, "Par time\t" MONOSPACED("%02i") ":" MONOSPACED("%02i"),
+            wbs->partime / TICRATE / 60, wbs->partime / TICRATE % 60);
 
-    WI_InitAnimatedBack();
+    if (totaltime > maptime)
+    {
+        const int   hours = totaltime / TICRATE / 3600;
+
+        if (hours)
+            C_TabbedOutput(tabs, "Total time\t" MONOSPACED("%i") ":" MONOSPACED("%02i") ":" MONOSPACED("%02i"),
+                hours, hours / TICRATE / 60, hours / TICRATE % 60);
+        else
+            C_TabbedOutput(tabs, "Total time\t" MONOSPACED("%02i") ":" MONOSPACED("%02i"),
+                totaltime / TICRATE / 60, totaltime / TICRATE % 60);
+    }
+
+    WI_InitAnimatedBack(true);
 }
 
 static void WI_UpdateStats(void)
 {
     // e6y
-    static dboolean play_early_explosion = true;
+    static bool play_early_explosion = true;
 
     WI_UpdateAnimatedBack();
 
@@ -866,19 +918,27 @@ static void WI_UpdateStats(void)
     }
     else if (sp_state == 6)
     {
-        cnt_secret += 2;
-
-        if (!(bcnt & 3))
-            S_StartSound(NULL, sfx_pistol);
-
-        if (cnt_secret >= (wbs->ssecret * 100) / wbs->maxsecret)
+        // [JN] If no secrets on the map, skip counting immediately.
+        if (!totalsecrets)
+            sp_state += 2;
+        else
         {
-            cnt_secret = (wbs->ssecret * 100) / wbs->maxsecret;
-            S_StartSound(NULL, sfx_barexp);
-            sp_state++;
+            cnt_secret += 2;
+
+            if (!(bcnt & 3))
+                S_StartSound(NULL, sfx_pistol);
+
+            if (cnt_secret >= (wbs->ssecret * 100) / wbs->maxsecret)
+            {
+                cnt_secret = (wbs->ssecret * 100) / wbs->maxsecret;
+
+                if (totalsecrets)
+                    S_StartSound(NULL, sfx_barexp);
+
+                sp_state++;
+            }
         }
     }
-
     else if (sp_state == 8)
     {
         // e6y: do not play count sound after explosion sound
@@ -944,7 +1004,7 @@ static void WI_UpdateStats(void)
 static void WI_DrawStats(void)
 {
     // line height
-    int lh = 3 * SHORT(num[0]->height) / 2;
+    const int   lh = 3 * SHORT(num[0]->height) / 2;
 
     WI_SlamBackground();
 
@@ -953,37 +1013,40 @@ static void WI_DrawStats(void)
 
     WI_DrawLF();
 
-    V_DrawPatchWithShadow(SP_STATSX + 1, SP_STATSY + 1, kills, false);
+    V_DrawMenuPatch(SP_STATSX, SP_STATSY + 1, kills, false, VANILLAWIDTH);
     WI_DrawPercent(VANILLAWIDTH - SP_STATSX - 14, SP_STATSY, cnt_kills);
 
-    V_DrawPatchWithShadow(SP_STATSX + 1, SP_STATSY + lh + 1, items, false);
+    V_DrawMenuPatch(SP_STATSX, SP_STATSY + lh + 1, items, false, VANILLAWIDTH);
     WI_DrawPercent(VANILLAWIDTH - SP_STATSX - 14, SP_STATSY + lh, cnt_items);
 
-    if (totalsecret)
+    if (totalsecrets)
     {
         if (!WISCRT2)
-            M_DrawString(SP_STATSX, SP_STATSY + 2 * lh - 3, "secrets");
+            M_DrawString(SP_STATSX, SP_STATSY + 2 * lh - 2, "secrets", false, true);
         else
-            V_DrawPatchWithShadow(SP_STATSX + 1, SP_STATSY + 2 * lh + 1, sp_secret, false);
+            V_DrawMenuPatch(SP_STATSX, SP_STATSY + 2 * lh + 1, sp_secret, false, VANILLAWIDTH);
 
         WI_DrawPercent(VANILLAWIDTH - SP_STATSX - 14, SP_STATSY + 2 * lh, cnt_secret);
     }
 
-    V_DrawPatchWithShadow(SP_TIMEX + 1, SP_TIMEY + 1, timepatch, false);
-    WI_DrawTime(VANILLAWIDTH / 2 - SP_TIMEX * 2, SP_TIMEY, cnt_time);
+    V_DrawMenuPatch(SP_TIMEX + 1, SP_TIMEY + 1, timepatch, false, VANILLAWIDTH);
+    WI_DrawTime(VANILLAWIDTH / 2 - SP_TIMEX * 2 + (wbs->stime >= TICRATE * 60 * 60) * 16, SP_TIMEY, cnt_time);
 
     if (wbs->partime)
     {
-        V_DrawPatchWithShadow(VANILLAWIDTH / 2 + SP_TIMEX + (BTSX ? 0 : SP_TIMEX - FREEDOOM * 17 + 3), SP_TIMEY + 1, par, false);
-        WI_DrawTime(VANILLAWIDTH - SP_TIMEX - 2 - (BTSX || FREEDOOM) * 17, SP_TIMEY, cnt_par);
+        V_DrawMenuPatch(VANILLAWIDTH / 2 + SP_TIMEX + (BTSX ? 0 : SP_TIMEX - (FREEDOOM ? 17 : 0) + 3),
+            SP_TIMEY + 1, par, false, VANILLAWIDTH);
+        WI_DrawTime(VANILLAWIDTH - SP_TIMEX - 2 - (BTSX || FREEDOOM ? 17 : 0), SP_TIMEY, cnt_par);
     }
 }
 
 void WI_CheckForAccelerate(void)
 {
-    const uint8_t   *keystate = SDL_GetKeyboardState(NULL);
+    const uint8_t   *keyboardstate = SDL_GetKeyboardState(NULL);
 
-    if ((viewplayer->cmd.buttons & BT_ATTACK) || keystate[SDL_SCANCODE_RETURN] || keystate[SDL_SCANCODE_KP_ENTER])
+    if ((viewplayer->cmd.buttons & BT_ATTACK)
+        || keyboardstate[SDL_SCANCODE_RETURN]
+        || keyboardstate[SDL_SCANCODE_KP_ENTER])
     {
         if (!viewplayer->attackdown)
             acceleratestage = true;
@@ -993,7 +1056,7 @@ void WI_CheckForAccelerate(void)
     else
         viewplayer->attackdown = false;
 
-    if ((viewplayer->cmd.buttons & BT_USE) || keystate[SDL_SCANCODE_SPACE])
+    if ((viewplayer->cmd.buttons & BT_USE) || keyboardstate[SDL_SCANCODE_SPACE])
     {
         if (!viewplayer->usedown)
             acceleratestage = true;
@@ -1016,26 +1079,28 @@ void WI_Ticker(void)
 
     WI_LoadData();
 
-    if (!menuactive && !consoleactive)
+    if (!menuactive && !consoleactive && !paused && windowfocused)
+    {
         WI_CheckForAccelerate();
 
-    switch (state)
-    {
-        case StatCount:
-            WI_UpdateStats();
-            break;
+        switch (state)
+        {
+            case StatCount:
+                WI_UpdateStats();
+                break;
 
-        case ShowNextLoc:
-            WI_UpdateShowNextLoc();
-            break;
+            case ShowNextLoc:
+                WI_UpdateShowNextLoc();
+                break;
 
-        case NoState:
-            WI_UpdateNoState();
-            break;
+            case NoState:
+                WI_UpdateNoState();
+                break;
+        }
     }
 }
 
-typedef void (*load_callback_t)(char *, patch_t **);
+typedef void (*load_callback_t)(const char *, patch_t **);
 
 // Common load/unload function. Iterates over all the graphics
 // lumps to be loaded/unloaded into memory.
@@ -1045,7 +1110,7 @@ static void WI_LoadUnloadData(load_callback_t callback)
 
     if (gamemode == commercial)
     {
-        for (int i = 0; i < NUMCMAPS; i++)
+        for (int i = 0; i < numcmaps; i++)
         {
             M_snprintf(name, sizeof(name), "CWILV%02i", i);
             callback(name, &lnames[i]);
@@ -1069,7 +1134,7 @@ static void WI_LoadUnloadData(load_callback_t callback)
         callback("WISPLAT", &splat[0]);
 
         if (wbs->epsd < 3)
-            for (int j = 0; j < NUMANIMS[wbs->epsd]; j++)
+            for (int j = 0; j < numanims[wbs->epsd]; j++)
             {
                 anim_t  *a = &anims[wbs->epsd][j];
 
@@ -1089,7 +1154,7 @@ static void WI_LoadUnloadData(load_callback_t callback)
             }
     }
 
-    for (int i = 0; i < 10; i++)
+    for (int i = 0; i <= 9; i++)
     {
         // numbers 0-9
         M_snprintf(name, sizeof(name), "WINUM%i", i);
@@ -1127,7 +1192,7 @@ static void WI_LoadUnloadData(load_callback_t callback)
     callback("WIPAR", &par);
 }
 
-static void WI_LoadCallback(char *name, patch_t **variable)
+static void WI_LoadCallback(const char *name, patch_t **variable)
 {
     *variable = (patch_t *)W_CacheLumpName(name);
 }
@@ -1138,49 +1203,61 @@ static void WI_LoadData(void)
 
     if (gamemode == commercial)
     {
-        NUMCMAPS = 32 + (W_CheckNumForName("CWILV32") >= 0);
-        lnames = Z_Malloc(sizeof(patch_t *) * NUMCMAPS, PU_STATIC, NULL);
+        numcmaps = 32 + (W_CheckNumForName("CWILV32") >= 0);
+        lnames = Z_Malloc(numcmaps * sizeof(patch_t *), PU_STATIC, NULL);
     }
     else
-        lnames = Z_Malloc(sizeof(patch_t *) * NUMMAPS, PU_STATIC, NULL);
+        lnames = Z_Malloc(NUMMAPS * sizeof(patch_t *), PU_STATIC, NULL);
 
     WI_LoadUnloadData(&WI_LoadCallback);
 
     // Background image
-    if (FREEDOOM || hacx)
+    if (enterpic > 0 && state != StatCount)
+        lump = W_CacheLumpNum(enterpic);
+    else if (exitpic > 0)
+        lump = W_CacheLumpNum(exitpic);
+    else if (FREEDOOM || hacx)
         lump = W_CacheLastLumpName("INTERPIC");
     else if (gamemode == commercial)
     {
-        int lumpnum = P_GetMapEnterPic(gamemap);
-
-        if (lumpnum > 0)
-            lump = W_CacheLumpNum(lumpnum);
-        else if (gamemission == pack_plut)
+        if (gamemission == pack_plut)
             lump = W_CacheLumpName("INTERPI2");
         else if (gamemission == pack_tnt)
             lump = W_CacheLumpName("INTERPI3");
+        else if (nerve)
+            lump = W_CacheLumpNameFromResourceWAD("INTERPIC");
         else
             lump = W_CacheLumpName("INTERPIC");
     }
     else if (gamemode == retail && wbs->epsd == 3)
-        lump = W_CacheLumpName("INTERPI1");
+        lump = W_CacheLumpName(REKKRSL ? "INTERPIW" : "INTERPI1");
     else if (sigil && wbs->epsd == 4)
         lump = W_CacheLumpName("SIGILINT");
-    else
+    else if (sigil2 && wbs->epsd == 5)
+        lump = W_CacheLumpName("SIGILIN2");
+    else if (wbs->epsd <= 2)
     {
         char    temp[9];
 
-        M_snprintf(temp, sizeof(temp), "WIMAP%i", wbs->epsd);
-        lump = (chex || REKKRIWAD ? W_CacheLastLumpName(temp) : W_CacheLumpName(temp));
+        if (REKKR)
+        {
+            M_snprintf(temp, sizeof(temp), "WIMAP%iW", wbs->epsd);
+
+            if (W_CheckNumForName(temp) < 0)
+                M_snprintf(temp, sizeof(temp), "WIMAP%i", wbs->epsd);
+        }
+        else
+            M_snprintf(temp, sizeof(temp), "WIMAP%i", wbs->epsd);
+
+        lump = (chex ? W_CacheLastLumpName(temp) : W_CacheLumpName(temp));
     }
+    else
+        lump = W_CacheLumpName("INTERPIC");
 
-    if (SCREENWIDTH != NONWIDEWIDTH)
-        memset(screens[1], FindDominantEdgeColor(lump), SCREENAREA);
-
-    V_DrawWidePatch((SCREENWIDTH / SCREENSCALE - SHORT(lump->width)) / 2, 0, 1, lump);
+    V_DrawPagePatch(1, lump);
 }
 
-static void WI_UnloadCallback(char *name, patch_t **variable)
+static void WI_UnloadCallback(const char *name, patch_t **variable)
 {
     W_ReleaseLumpName(name);
     *variable = NULL;
@@ -1215,6 +1292,9 @@ static void WI_InitVariables(wbstartstruct_t *wbstartstruct)
 
     wbs = wbstartstruct;
 
+    enterpic = P_GetMapEnterPic(wbs->epsd + 1, wbs->next + 1);
+    exitpic = P_GetMapExitPic(gameepisode, gamemap);
+
     acceleratestage = false;
     cnt = 0;
     bcnt = 0;
